@@ -49,24 +49,24 @@ test ${kernel:0:5} = "MINGW" && kernel="MINGW"
 
 compiler="gcc"
 
-qt_version=4.8.6
+qt_version=5.4.1
 zlib_version=1.2.8
-zlib_patchlevel=1
+zlib_patchlevel=2
 libogg_version=1.3.2
 libogg_patchlevel=1
-libvorbis_version=1.3.2
-libvorbis_patchlevel=1.4
-libav_version=10.2
-libav_patchlevel=2
-libflac_version=1.3.0
-libflac_patchlevel=2
+libvorbis_version=1.3.4
+libvorbis_patchlevel=2
+libav_version=11.3
+libav_patchlevel=1
+libflac_version=1.3.1
+libflac_patchlevel=1
 id3lib_version=3.8.3
-id3lib_patchlevel=15
+id3lib_patchlevel=16
 taglib_version=1.9.1
-chromaprint_version=1.1
+chromaprint_version=1.2
 chromaprint_patchlevel=1
 mp4v2_version=2.0.0
-mp4v2_patchlevel=2
+mp4v2_patchlevel=3
 
 # Uncomment for debug build
 #ENABLE_DEBUG=--enable-debug
@@ -78,6 +78,10 @@ if ! which cmake >/dev/null; then
   exit 1
 fi
 
+if test $kernel = "MSYS_NT-6.1"; then
+kernel="MINGW"
+CONFIGURE_OPTIONS="--build=x86_64-w64-mingw32 --target=i686-w64-mingw32"
+fi
 if test $kernel = "MINGW"; then
 CMAKE_OPTIONS="-G \"MSYS Makefiles\" -DCMAKE_INSTALL_PREFIX=/usr/local"
 elif test $kernel = "Darwin"; then
@@ -89,7 +93,8 @@ CMAKE_OPTIONS="$CMAKE_OPTIONS -DCMAKE_TOOLCHAIN_FILE=$thisdir/source/mingw.cmake
 
 # Note that Ubuntu i686-w64-mingw32 is incompatible (sjlj) with the mingw (dw2)
 # used for the Qt mingw binaries >=4.8.6.
-cross_host="i586-mingw32msvc"
+# I am using a custom built cross mingw (--disable-sjlj-exceptions --enable-threads=posix)
+cross_host="i686-w64-mingw32"
 CONFIGURE_OPTIONS="--host=${cross_host}"
 fi
 
@@ -107,10 +112,17 @@ if test "$ARCH" = "i386"; then
 else
   ARCH_FLAG="-Xarch_x86_64"
 fi
-CMAKE_OPTIONS="$CMAKE_OPTIONS -DCMAKE_C_FLAGS=\"-O2 $ARCH_FLAG -mmacosx-version-min=10.5\" -DCMAKE_CXX_FLAGS=\"-O2 $ARCH_FLAG -mmacosx-version-min=10.5 -fvisibility=hidden -fvisibility-inlines-hidden\""
-export CFLAGS="-O2 $ARCH_FLAG -mmacosx-version-min=10.5"
-export CXXFLAGS="-O2 $ARCH_FLAG -mmacosx-version-min=10.5"
-export LDFLAGS="$ARCH_FLAG -mmacosx-version-min=10.5"
+if [[ $(sw_vers -productVersion) = 10.1* ]]; then
+  CMAKE_OPTIONS="$CMAKE_OPTIONS -DCMAKE_C_FLAGS=\"-O2 $ARCH_FLAG -mmacosx-version-min=10.7\" -DCMAKE_CXX_FLAGS=\"-O2 $ARCH_FLAG -mmacosx-version-min=10.7 -fvisibility=hidden -fvisibility-inlines-hidden -stdlib=libc++\" -DCMAKE_EXE_LINKER_FLAGS=\"$ARCH_FLAG -stdlib=libc++\" -DCMAKE_MODULE_LINKER_FLAGS=\"$ARCH_FLAG -stdlib=libc++\" -DCMAKE_SHARED_LINKER_FLAGS=\"$ARCH_FLAG -stdlib=libc++\""
+  export CFLAGS="-O2 $ARCH_FLAG -mmacosx-version-min=10.7"
+  export CXXFLAGS="-O2 $ARCH_FLAG -mmacosx-version-min=10.7 -stdlib=libc++"
+  export LDFLAGS="$ARCH_FLAG -mmacosx-version-min=10.7 -stdlib=libc++"
+else
+  CMAKE_OPTIONS="$CMAKE_OPTIONS -DCMAKE_C_FLAGS=\"-O2 $ARCH_FLAG -mmacosx-version-min=10.5\" -DCMAKE_CXX_FLAGS=\"-O2 $ARCH_FLAG -mmacosx-version-min=10.5 -fvisibility=hidden -fvisibility-inlines-hidden\""
+  export CFLAGS="-O2 $ARCH_FLAG -mmacosx-version-min=10.5"
+  export CXXFLAGS="-O2 $ARCH_FLAG -mmacosx-version-min=10.5"
+  export LDFLAGS="$ARCH_FLAG -mmacosx-version-min=10.5"
+fi
 fi
 
 if which wget >/dev/null; then
@@ -132,6 +144,14 @@ fixcmakeinst() {
       rmdir msys
     elif test -d MinGW; then
       mv MinGW usr
+    elif test -d msys64; then
+      rm -rf usr
+      mv msys64/usr .
+      rmdir msys64
+    elif test -d msys32; then
+      rm -rf usr
+      mv msys32/usr .
+      rmdir msys32
     fi
     cd ..
   fi
@@ -143,13 +163,13 @@ fixcmakeinst() {
 test -d source || mkdir source
 cd source
 
-test -f flac_${libflac_version}-${libflac_patchlevel}.debian.tar.gz ||
-$DOWNLOAD http://ftp.de.debian.org/debian/pool/main/f/flac/flac_${libflac_version}-${libflac_patchlevel}.debian.tar.gz
+test -f flac_${libflac_version}-${libflac_patchlevel}.debian.tar.xz ||
+$DOWNLOAD http://ftp.de.debian.org/debian/pool/main/f/flac/flac_${libflac_version}-${libflac_patchlevel}.debian.tar.xz
 test -f flac_${libflac_version}.orig.tar.xz ||
 $DOWNLOAD http://ftp.de.debian.org/debian/pool/main/f/flac/flac_${libflac_version}.orig.tar.xz
 
-test -f id3lib3.8.3_${id3lib_version}-${id3lib_patchlevel}.debian.tar.gz ||
-$DOWNLOAD http://ftp.de.debian.org/debian/pool/main/i/id3lib3.8.3/id3lib3.8.3_${id3lib_version}-${id3lib_patchlevel}.debian.tar.gz
+test -f id3lib3.8.3_${id3lib_version}-${id3lib_patchlevel}.debian.tar.xz ||
+$DOWNLOAD http://ftp.de.debian.org/debian/pool/main/i/id3lib3.8.3/id3lib3.8.3_${id3lib_version}-${id3lib_patchlevel}.debian.tar.xz
 test -f id3lib3.8.3_${id3lib_version}.orig.tar.gz ||
 $DOWNLOAD http://ftp.de.debian.org/debian/pool/main/i/id3lib3.8.3/id3lib3.8.3_${id3lib_version}.orig.tar.gz
 
@@ -158,20 +178,20 @@ $DOWNLOAD http://ftp.de.debian.org/debian/pool/main/libo/libogg/libogg_${libogg_
 test -f libogg_${libogg_version}.orig.tar.gz ||
 $DOWNLOAD http://ftp.de.debian.org/debian/pool/main/libo/libogg/libogg_${libogg_version}.orig.tar.gz
 
-test -f libvorbis_${libvorbis_version}-${libvorbis_patchlevel}.diff.gz ||
-$DOWNLOAD http://ftp.de.debian.org/debian/pool/main/libv/libvorbis/libvorbis_${libvorbis_version}-${libvorbis_patchlevel}.diff.gz
+test -f libvorbis_${libvorbis_version}-${libvorbis_patchlevel}.debian.tar.xz ||
+$DOWNLOAD http://ftp.de.debian.org/debian/pool/main/libv/libvorbis/libvorbis_${libvorbis_version}-${libvorbis_patchlevel}.debian.tar.xz
 test -f libvorbis_${libvorbis_version}.orig.tar.gz ||
 $DOWNLOAD http://ftp.de.debian.org/debian/pool/main/libv/libvorbis/libvorbis_${libvorbis_version}.orig.tar.gz
 
 test -f taglib-${taglib_version}.tar.gz ||
 $DOWNLOAD http://taglib.github.io/releases/taglib-${taglib_version}.tar.gz
 
-test -f zlib_${zlib_version}.dfsg-${zlib_patchlevel}.debian.tar.gz ||
-$DOWNLOAD http://ftp.de.debian.org/debian/pool/main/z/zlib/zlib_${zlib_version}.dfsg-${zlib_patchlevel}.debian.tar.gz
+test -f zlib_${zlib_version}.dfsg-${zlib_patchlevel}.debian.tar.xz ||
+$DOWNLOAD http://ftp.de.debian.org/debian/pool/main/z/zlib/zlib_${zlib_version}.dfsg-${zlib_patchlevel}.debian.tar.xz
 test -f zlib_${zlib_version}.dfsg.orig.tar.gz ||
 $DOWNLOAD http://ftp.de.debian.org/debian/pool/main/z/zlib/zlib_${zlib_version}.dfsg.orig.tar.gz
 
-if test "$libav_version" = "0.8.10"; then
+if test "${libav_version%.*}" = "0.8"; then
 test -f libav_${libav_version}.orig.tar.gz ||
 $DOWNLOAD http://ftp.de.debian.org/debian/pool/main/liba/libav/libav_${libav_version}.orig.tar.gz
 test -f libav_${libav_version}-${libav_patchlevel}.debian.tar.gz ||
@@ -185,20 +205,20 @@ fi
 
 test -f chromaprint_${chromaprint_version}.orig.tar.gz ||
 $DOWNLOAD http://ftp.de.debian.org/debian/pool/main/c/chromaprint/chromaprint_${chromaprint_version}.orig.tar.gz
-test -f chromaprint_${chromaprint_version}-${chromaprint_patchlevel}.debian.tar.gz ||
-$DOWNLOAD http://ftp.de.debian.org/debian/pool/main/c/chromaprint/chromaprint_${chromaprint_version}-${chromaprint_patchlevel}.debian.tar.gz
+test -f chromaprint_${chromaprint_version}-${chromaprint_patchlevel}.debian.tar.xz ||
+$DOWNLOAD http://ftp.de.debian.org/debian/pool/main/c/chromaprint/chromaprint_${chromaprint_version}-${chromaprint_patchlevel}.debian.tar.xz
 
 test -f mp4v2_${mp4v2_version}~dfsg0.orig.tar.bz2 ||
 $DOWNLOAD http://ftp.de.debian.org/debian/pool/main/m/mp4v2/mp4v2_${mp4v2_version}~dfsg0.orig.tar.bz2
-test -f mp4v2_${mp4v2_version}~dfsg0-${mp4v2_patchlevel}.debian.tar.gz ||
-$DOWNLOAD http://ftp.de.debian.org/debian/pool/main/m/mp4v2/mp4v2_${mp4v2_version}~dfsg0-${mp4v2_patchlevel}.debian.tar.gz
+test -f mp4v2_${mp4v2_version}~dfsg0-${mp4v2_patchlevel}.debian.tar.xz ||
+$DOWNLOAD http://ftp.de.debian.org/debian/pool/main/m/mp4v2/mp4v2_${mp4v2_version}~dfsg0-${mp4v2_patchlevel}.debian.tar.xz
 
 # Create patch files
 
 if test "$compiler" = "cross-mingw"; then
 test -f mingw.cmake ||
 cat >mingw.cmake <<EOF
-set(QT_PREFIX /windows/Qt/$qt_version)
+set(QT_PREFIX /windows/Qt/Qt${qt_version}/${qt_version%.?}/mingw491_32)
 
 set(CMAKE_SYSTEM_NAME Windows)
 set(CMAKE_C_COMPILER ${cross_host}-gcc)
@@ -209,14 +229,23 @@ set(CMAKE_FIND_ROOT_PATH_MODE_PROGRAM NEVER)
 set(CMAKE_FIND_ROOT_PATH_MODE_LIBRARY ONLY)
 set(CMAKE_FIND_ROOT_PATH_MODE_INCLUDE ONLY)
 
-set(QT_BINARY_DIR /usr/lib/${HOSTTYPE/i686/i386}-linux-gnu/qt4/bin)
+set(QT_BINARY_DIR /usr/lib/${HOSTTYPE/i686/i386}-linux-gnu/qt5/bin)
 set(QT_LIBRARY_DIR  \${QT_PREFIX}/lib)
-set(QT_QTCORE_LIBRARY   \${QT_PREFIX}/lib/libQtCore4.a)
+set(QT_QTCORE_LIBRARY   \${QT_PREFIX}/lib/libQt5Core.a)
 set(QT_QTCORE_INCLUDE_DIR \${QT_PREFIX}/include/QtCore)
 set(QT_MKSPECS_DIR  \${QT_PREFIX}/mkspecs)
 set(QT_MOC_EXECUTABLE  \${QT_BINARY_DIR}/moc)
 set(QT_QMAKE_EXECUTABLE  \${QT_BINARY_DIR}/qmake)
 set(QT_UIC_EXECUTABLE  \${QT_BINARY_DIR}/uic)
+
+foreach (_exe moc rcc lupdate lrelease uic)
+  if (NOT TARGET Qt5::\${_exe})
+    add_executable(Qt5::\${_exe} IMPORTED)
+    set_target_properties(Qt5::\${_exe} PROPERTIES
+      IMPORTED_LOCATION \${QT_BINARY_DIR}/\${_exe}
+    )
+  endif ()
+endforeach (_exe)
 EOF
 fi
 
@@ -360,32 +389,6 @@ diff -ru id3lib-3.8.3.orig/include/id3/globals.h id3lib-3.8.3/include/id3/global
  #  define LINKOPTION_USE_DYNAMIC    3 //if your project links id3lib dynamic
 EOF
 
-test -f id3lib-fix-utf16-stringlists.patch ||
-cat >id3lib-fix-utf16-stringlists.patch <<EOF
-diff -ru id3lib-3.8.3.orig/src/io_helpers.cpp id3lib-3.8.3/src/io_helpers.cpp
---- id3lib-3.8.3.orig/src/io_helpers.cpp	2012-08-26 19:52:21.523825799 +0200
-+++ id3lib-3.8.3/src/io_helpers.cpp	2012-08-26 19:53:02.060028394 +0200
-@@ -373,10 +373,17 @@
-     //}
-     // Right code
-     unsigned char *pdata = (unsigned char *) data.c_str();
-+    unicode_t lastCh = BOM;
-     for (size_t i = 0; i < size; i += 2)
-     {
-       unicode_t ch = (pdata[i] << 8) | pdata[i+1];
-+      if (lastCh == 0 && ch != BOM)
-+      {
-+        // Last character was NULL, so start next string with BOM.
-+        writer.writeChars((const unsigned char*) &BOM, 2);
-+      }
-       writer.writeChars((const unsigned char*) &ch, 2);
-+      lastCh = ch;
-     }
-     // End patch
-   }
-
-EOF
-
 test -f taglib-msvc.patch ||
 cat >taglib-msvc.patch <<"EOF"
 diff -ru taglib-1.8.orig/CMakeLists.txt taglib-1.8/CMakeLists.txt
@@ -404,7 +407,111 @@ diff -ru taglib-1.8.orig/CMakeLists.txt taglib-1.8/CMakeLists.txt
  	set(FRAMEWORK_INSTALL_DIR "/Library/Frameworks" CACHE STRING "Directory to install frameworks to.")
 EOF
 
-if test "$libav_version" = "0.8.10"; then
+test -f taglib_bvreplace.patch ||
+cat >taglib_bvreplace.patch <<"EOF"
+--- taglib-1.9.1/taglib/toolkit/tbytevector.cpp.orig	2013-10-08 17:50:01.000000000 +0200
++++ taglib-1.9.1/taglib/toolkit/tbytevector.cpp	2015-03-17 13:03:45.267093248 +0100
+@@ -31,6 +31,7 @@
+ #include <iostream>
+ #include <cstdio>
+ #include <cstring>
++#include <cstddef>
+ 
+ #include <tstring.h>
+ #include <tdebug.h>
+@@ -508,62 +509,40 @@
+   if(pattern.size() == 0 || pattern.size() > size())
+     return *this;
+ 
+-  const uint withSize = with.size();
+-  const uint patternSize = pattern.size();
+-  int offset = 0;
++  const size_t withSize    = with.size();
++  const size_t patternSize = pattern.size();
++  const ptrdiff_t diff = withSize - patternSize;
++
++  size_t offset = 0;
++  while (true)
++  {
++    offset = find(pattern, offset);
++    if(offset == static_cast<size_t>(-1)) // Use npos in taglib2.
++      break;
+ 
+-  if(withSize == patternSize) {
+-    // I think this case might be common enough to optimize it
+     detach();
+-    offset = find(pattern);
+-    while(offset >= 0) {
+-      ::memcpy(data() + offset, with.data(), withSize);
+-      offset = find(pattern, offset + withSize);
+-    }
+-    return *this;
+-  }
+ 
+-  // calculate new size:
+-  uint newSize = 0;
+-  for(;;) {
+-    int next = find(pattern, offset);
+-    if(next < 0) {
+-      if(offset == 0)
+-        // pattern not found, do nothing:
+-        return *this;
+-      newSize += size() - offset;
+-      break;
++    if(diff < 0) {
++      ::memmove(
++        data() + offset + withSize,
++        data() + offset + patternSize,
++        size() - offset - patternSize);
++      resize(size() + diff);
+     }
+-    newSize += (next - offset) + withSize;
+-    offset = next + patternSize;
+-  }
+-
+-  // new private data of appropriate size:
+-  ByteVectorPrivate *newData = new ByteVectorPrivate(newSize, 0);
+-  char *target = DATA(newData);
+-  const char *source = data();
+-
+-  // copy modified data into new private data:
+-  offset = 0;
+-  for(;;) {
+-    int next = find(pattern, offset);
+-    if(next < 0) {
+-      ::memcpy(target, source + offset, size() - offset);
+-      break;
++    else if(diff > 0) {
++      resize(size() + diff);
++      ::memmove(
++        data() + offset + withSize,
++        data() + offset + patternSize,
++        size() - diff - offset - patternSize);
+     }
+-    int chunkSize = next - offset;
+-    ::memcpy(target, source + offset, chunkSize);
+-    target += chunkSize;
+-    ::memcpy(target, with.data(), withSize);
+-    target += withSize;
+-    offset += chunkSize + patternSize;
+-  }
+ 
+-  // replace private data:
+-  if(d->deref())
+-    delete d;
++    ::memcpy(data() + offset, with.data(), with.size());
+ 
+-  d = newData;
++    offset += withSize;
++    if(offset > size() - patternSize)
++      break;
++  }
+ 
+   return *this;
+ }
+EOF
+
+if test "${libav_version%.*}" = "0.8"; then
 test -f libav_sws.patch ||
 cat >libav_sws.patch <<"EOF"
 --- cmdutils.c.org      2011-09-17 13:36:43.000000000 -0700
@@ -426,18 +533,6 @@ fi
 
 test -f mp4v2_win32.patch ||
 cat >mp4v2_win32.patch <<"EOF"
-diff -ruN mp4v2-2.0.0.orig/configure.ac mp4v2-2.0.0/configure.ac
---- mp4v2-2.0.0.orig/configure.ac	2012-05-21 00:11:55.000000000 +0200
-+++ mp4v2-2.0.0/configure.ac	2014-04-14 07:25:37.752968162 +0200
-@@ -20,7 +20,7 @@
- 
- m4_define([PRJ_version],ifelse(
-     PRJ_repo_type,[stable],m4_format([%s],PRJ_repo_branch),
--    m4_format([%s-r%d],PRJ_repo_branch,PRJ_repo_rev)))
-+    m4_format([%s-r%s],PRJ_repo_branch,PRJ_repo_rev)))
- 
- ###############################################################################
- # initialization
 diff -ruN mp4v2-2.0.0.orig/GNUmakefile.am mp4v2-2.0.0/GNUmakefile.am
 --- mp4v2-2.0.0.orig/GNUmakefile.am	2012-05-21 00:11:55.000000000 +0200
 +++ mp4v2-2.0.0/GNUmakefile.am	2014-04-14 07:22:35.904963506 +0200
@@ -1641,7 +1736,8 @@ echo "### Extracting zlib"
 if ! test -d zlib-${zlib_version}; then
 tar xzf source/zlib_${zlib_version}.dfsg.orig.tar.gz
 cd zlib-${zlib_version}/
-tar xzf ../source/zlib_${zlib_version}.dfsg-${zlib_patchlevel}.debian.tar.gz || true
+
+unxz -c ../source/zlib_${zlib_version}.dfsg-${zlib_patchlevel}.debian.tar.xz | tar x || true
 echo Can be ignored: Cannot create symlink to debian.series
 for f in $(cat debian/patches/debian.series); do patch -p1 <debian/patches/$f; done
 cd ..
@@ -1661,7 +1757,8 @@ echo "### Extracting libvorbis"
 if ! test -d libvorbis-${libvorbis_version}; then
 tar xzf source/libvorbis_${libvorbis_version}.orig.tar.gz
 cd libvorbis-${libvorbis_version}/
-gunzip -c ../source/libvorbis_${libvorbis_version}-${libvorbis_patchlevel}.diff.gz | patch -p1
+unxz -c ../source/libvorbis_${libvorbis_version}-${libvorbis_patchlevel}.debian.tar.xz | tar x
+for f in $(cat debian/patches/series); do patch -p1 <debian/patches/$f; done
 test -f win32/VS2010/libogg.props.orig || mv win32/VS2010/libogg.props win32/VS2010/libogg.props.orig
 sed "s/<LIBOGG_VERSION>1.2.0</<LIBOGG_VERSION>$libogg_version</" win32/VS2010/libogg.props.orig >win32/VS2010/libogg.props
 cd ..
@@ -1672,7 +1769,7 @@ echo "### Extracting libflac"
 if ! test -d flac-${libflac_version}; then
 unxz -c source/flac_${libflac_version}.orig.tar.xz | tar x
 cd flac-${libflac_version}/
-tar xzf ../source/flac_${libflac_version}-${libflac_patchlevel}.debian.tar.gz
+unxz -c ../source/flac_${libflac_version}-${libflac_patchlevel}.debian.tar.xz | tar x
 for f in $(cat debian/patches/series); do patch -p1 <debian/patches/$f; done
 patch -p1 <../source/flac_1.2.1_size_t_max_patch.diff
 if test $kernel = "Darwin"; then
@@ -1687,10 +1784,9 @@ echo "### Extracting id3lib"
 if ! test -d id3lib-${id3lib_version}; then
 tar xzf source/id3lib3.8.3_${id3lib_version}.orig.tar.gz
 cd id3lib-${id3lib_version}/
-tar xzf ../source/id3lib3.8.3_${id3lib_version}-${id3lib_patchlevel}.debian.tar.gz
-for f in $(cat debian/patches/series); do patch -p1 <debian/patches/$f; done
+unxz -c ../source/id3lib3.8.3_${id3lib_version}-${id3lib_patchlevel}.debian.tar.xz | tar x
+for f in $(cat debian/patches/series); do patch --binary -p1 <debian/patches/$f; done
 patch -p1 <../source/id3lib-3.8.3_mingw.patch
-patch -p1 <../source/id3lib-fix-utf16-stringlists.patch
 test -f makefile.win32.orig || mv makefile.win32 makefile.win32.orig
 sed 's/-W3 -WX -GX/-W3 -EHsc/; s/-MD -D "WIN32" -D "_DEBUG"/-MDd -D "WIN32" -D "_DEBUG"/' makefile.win32.orig >makefile.win32
 cd ..
@@ -1701,13 +1797,23 @@ echo "### Extracting taglib"
 if ! test -d taglib-${taglib_version}; then
 tar xzf source/taglib-${taglib_version}.tar.gz
 cd taglib-${taglib_version}/
-patch -p1 <../source/taglib-msvc.patch
+taglib_nr=${taglib_version:0:3}
+taglib_nr=${taglib_nr/./}
+if test $taglib_nr -ge 18; then
+  patch -p1 <../source/taglib-msvc.patch
+else
+  sed -i 's/^ADD_SUBDIRECTORY(bindings)/#ADD_SUBDIRECTORY(bindings)/' ./CMakeLists.txt
+  sed -i 's/^ADD_LIBRARY(tag SHARED/ADD_LIBRARY(tag STATIC/' ./taglib/CMakeLists.txt
+fi
+if test "${taglib_version}" = "1.9.1"; then
+  patch -p1 <../source/taglib_bvreplace.patch
+fi
 cd ..
 fi
 
 echo "### Extracting libav"
 
-if test "$libav_version" = "0.8.10"; then
+if test "${libav_version%.*}" = "0.8"; then
 if ! test -d libav-${libav_version}; then
 tar xzf source/libav_${libav_version}.orig.tar.gz
 cd libav-${libav_version}/
@@ -1726,10 +1832,19 @@ cd ..
 fi
 else
 if ! test -d libav-${libav_version}; then
-unxz -c source/libav_${libav_version}.orig.tar.xz | tar x
+unxz -c source/libav_${libav_version}.orig.tar.xz | tar x || true
+echo Can be ignored: Cannot create symlink to README.md
 cd libav-${libav_version}/
 unxz -c ../source/libav_${libav_version}-${libav_patchlevel}.debian.tar.xz | tar x
-for f in $(cat debian/patches/series); do patch -p1 <debian/patches/$f; done
+oldifs=$IFS
+IFS='
+'
+for f in $(cat debian/patches/series); do
+  if test "${f:0:1}" != "#"; then
+    patch -p1 <debian/patches/$f
+  fi
+done
+IFS=$oldifs
 cd ..
 fi
 fi
@@ -1739,7 +1854,7 @@ echo "### Extracting chromaprint"
 if ! test -d chromaprint-${chromaprint_version}; then
 tar xzf source/chromaprint_${chromaprint_version}.orig.tar.gz
 cd chromaprint-${chromaprint_version}/
-tar xzf ../source/chromaprint_${chromaprint_version}-${chromaprint_patchlevel}.debian.tar.gz
+unxz -c ../source/chromaprint_${chromaprint_version}-${chromaprint_patchlevel}.debian.tar.xz | tar x
 for f in $(cat debian/patches/series); do patch -p1 <debian/patches/$f; done
 cd ..
 fi
@@ -1749,7 +1864,8 @@ echo "### Extracting mp4v2"
 if ! test -d mp4v2-${mp4v2_version}; then
 tar xjf source/mp4v2_${mp4v2_version}~dfsg0.orig.tar.bz2
 cd mp4v2-${mp4v2_version}/
-tar xzf ../source/mp4v2_${mp4v2_version}~dfsg0-${mp4v2_patchlevel}.debian.tar.gz
+unxz -c ../source/mp4v2_${mp4v2_version}~dfsg0-${mp4v2_patchlevel}.debian.tar.xz | tar x
+for f in $(cat debian/patches/series); do patch -p1 <debian/patches/$f; done
 if test $kernel = "MINGW" || test "$compiler" = "cross-mingw"; then
 patch -p1 <../source/mp4v2_win32.patch
 fi
@@ -1830,6 +1946,17 @@ done
 
 else
 
+if test "$1" = "clean"; then
+  for d in zlib-${zlib_version} libogg-${libogg_version} \
+           libvorbis-${libvorbis_version} flac-${libflac_version} \
+           id3lib-${id3lib_version} taglib-${taglib_version} \
+           libav-${libav_version} chromaprint-${chromaprint_version} \
+           mp4v2-${mp4v2_version}; do
+    rm -rf $d/inst
+  done
+fi
+  
+if test ! -d zlib-${zlib_version}/inst; then
 echo "### Building zlib"
 
 cd zlib-${zlib_version}/
@@ -1848,7 +1975,9 @@ fi
 cd inst
 tar czf ../../bin/zlib-${zlib_version}.tgz usr
 cd ../..
+fi
 
+if test ! -d libogg-${libogg_version}/inst; then
 echo "### Building libogg"
 
 cd libogg-${libogg_version}/
@@ -1859,7 +1988,9 @@ make install DESTDIR=`pwd`/inst
 cd inst
 tar czf ../../bin/libogg-${libogg_version}.tgz usr
 cd ../..
+fi
 
+if test ! -d libvorbis-${libvorbis_version}/inst; then
 echo "### Building libvorbis"
 
 cd libvorbis-${libvorbis_version}/
@@ -1874,22 +2005,30 @@ make install DESTDIR=`pwd`/inst
 cd inst
 tar czf ../../bin/libvorbis-${libvorbis_version}.tgz usr
 cd ../..
+fi
 
+if test ! -d flac-${libflac_version}/inst; then
 echo "### Building libflac"
 
 cd flac-${libflac_version}/
+autoreconf -i
 configure_args="--enable-shared=no --enable-static=yes --with-ogg=$thisdir/libogg-$libogg_version/inst/usr/local $ENABLE_DEBUG $CONFIGURE_OPTIONS"
 if test $kernel = "Darwin"; then
   configure_args="$configure_args --disable-asm-optimizations"
 fi
 test -f Makefile || ./configure $configure_args
+# On msys32, an error "changed before entering" occurred, can be fixed by
+# modifying /usr/share/perl5/core_perl/File/Path.pm
+# my $Need_Stat_Check = !($^O eq 'MSWin32' || $^O eq 'msys');
 make
 mkdir -p inst
 make install DESTDIR=`pwd`/inst
 cd inst
 tar czf ../../bin/flac-${libflac_version}.tgz usr
 cd ../..
+fi
 
+if test ! -d id3lib-${id3lib_version}/inst; then
 echo "### Building id3lib"
 
 cd id3lib-${id3lib_version}/
@@ -1901,7 +2040,9 @@ make install DESTDIR=`pwd`/inst
 cd inst
 tar czf ../../bin/id3lib-${id3lib_version}.tgz usr
 cd ../..
+fi
 
+if test ! -d taglib-${taglib_version}/inst; then
 echo "### Building taglib"
 
 cd taglib-${taglib_version}/
@@ -1913,10 +2054,12 @@ fixcmakeinst
 cd inst
 tar czf ../../bin/taglib-${taglib_version}.tgz usr
 cd ../..
+fi
 
+if test ! -d libav-${libav_version}/inst; then
 echo "### Building libav"
 
-if test "$libav_version" = "0.8.10"; then
+if test "${libav_version%.*}" = "0.8"; then
 cd libav-${libav_version}
 # configure needs yasm and pr
 # On msys, make >= 3.81 is needed.
@@ -2036,6 +2179,9 @@ sed -i 's/^\(.*-Werror=missing-prototypes\)/#\1/' ./configure
 AV_CONFIGURE_OPTIONS="--cross-prefix=${cross_host}- --arch=x86 --target-os=mingw32 --sysinclude=/usr/${cross_host}/include --extra-cflags=-march=i486"
 elif test $kernel = "MINGW"; then
 AV_CONFIGURE_OPTIONS="--extra-cflags=-march=i486"
+if test $(uname) = "MSYS_NT-6.1"; then
+AV_CONFIGURE_OPTIONS="$AV_CONFIGURE_OPTIONS --target-os=mingw32"
+fi
 fi
 if test -z "$ENABLE_DEBUG"; then
 AV_CONFIGURE_OPTIONS="$AV_CONFIGURE_OPTIONS --disable-debug"
@@ -2126,7 +2272,9 @@ cd inst
 tar czf ../../bin/libav-${libav_version}.tgz usr
 cd ../..
 fi
+fi
 
+if test ! -d chromaprint-${chromaprint_version}/inst; then
 echo "### Building chromaprint"
 
 # The zlib library path was added for MinGW-builds GCC 4.7.2.
@@ -2138,11 +2286,14 @@ fixcmakeinst
 cd inst
 tar czf ../../bin/chromaprint-${chromaprint_version}.tgz usr
 cd ../..
+fi
 
+if test ! -d mp4v2-${mp4v2_version}/inst; then
 echo "### Building mp4v2"
 
 cd mp4v2-${mp4v2_version}/
-if test $kernel = "MINGW" || test "$compiler" = "cross-mingw"; then
+if test $kernel = "MINGW" || test "$compiler" = "cross-mingw" ||
+   test $kernel = "Darwin"; then
 autoreconf -i
 fi
 test -f Makefile || CXXFLAGS="-g -O2 -DMP4V2_USE_STATIC_LIB" ./configure --enable-shared=no --enable-static=yes --disable-gch $CONFIGURE_OPTIONS
@@ -2151,6 +2302,7 @@ make install DESTDIR=`pwd`/inst
 cd inst
 tar czf ../../bin/mp4v2-${mp4v2_version}.tgz usr
 cd ../..
+fi
 
 
 echo "### Installing to root directory"
@@ -2164,7 +2316,7 @@ if test $kernel = "Linux"; then
     mkdir kid3
     if test "$compiler" = "cross-mingw"; then
       cat >kid3/build.sh <<EOF
-cmake -DCMAKE_BUILD_TYPE=RelWithDebInfo -DCMAKE_TOOLCHAIN_FILE=$thisdir/source/mingw.cmake -DCMAKE_INSTALL_PREFIX= -DWITH_FFMPEG=ON -DWITH_MP4V2=ON -DCMAKE_CXX_FLAGS="-g -O2 -DMP4V2_USE_STATIC_LIB" -DDOCBOOK_XSL_DIR=/usr/share/xml/docbook/stylesheet/nwalsh ../../kid3
+cmake -DCMAKE_BUILD_TYPE=Release -DCMAKE_TOOLCHAIN_FILE=$thisdir/source/mingw.cmake -DCMAKE_INSTALL_PREFIX= -DWITH_FFMPEG=ON -DWITH_MP4V2=ON -DCMAKE_CXX_FLAGS="-g -O2 -DMP4V2_USE_STATIC_LIB" -DDOCBOOK_XSL_DIR=/usr/share/xml/docbook/stylesheet/nwalsh ../../kid3
 EOF
       cat >kid3/make_package.sh <<"EOF"
 #!/bin/sh
@@ -2173,7 +2325,6 @@ INSTDIR=kid3-$VERSION-win32
 QT_PREFIX=$(sed "s/set(QT_PREFIX \(.*\))/\1/;q" ../source/mingw.cmake)
 QT_BIN_DIR=${QT_PREFIX}/bin
 QT_TRANSLATIONS_DIR=${QT_PREFIX}/translations
-MINGW_DIR=/windows/msys/1.0/mingw/bin
 
 test -d $INSTDIR && rm -rf $INSTDIR
 mkdir -p $INSTDIR
@@ -2182,14 +2333,10 @@ echo "### Ignore make error"
 
 cp -f po/*.qm doc/*/kid3*.html $INSTDIR
 
-for f in QtCore4.dll QtNetwork4.dll QtGui4.dll QtXml4.dll phonon4.dll; do
+for f in Qt5Core.dll Qt5Network.dll Qt5Gui.dll Qt5Xml.dll Qt5Widgets.dll Qt5Multimedia.dll Qt5Qml.dll Qt5Quick.dll libgcc_s_dw2-1.dll libstdc++-6.dll libwinpthread-1.dll; do
   cp $QT_BIN_DIR/$f $INSTDIR
 done
 
-for f in libgcc_s_dw2-1.dll; do
-  cp $MINGW_DIR/$f $INSTDIR
-done
-
 for f in po/*.qm; do
   l=${f#*_};
   l=${l%.qm};
@@ -2201,10 +2348,10 @@ rm -f $INSTDIR.zip
 EOF
       chmod +x kid3/make_package.sh
     else
-      cat >kid3/build.sh <<"EOF"
-BUILDPREFIX=$(cd ..; pwd)/buildroot/usr/local
-export PKG_CONFIG_PATH=$BUILDPREFIX/lib/pkgconfig
-cmake -DWITH_TAGLIB=OFF -DHAVE_TAGLIB=1 -DTAGLIB_LIBRARIES:STRING="-L$BUILDPREFIX/lib -ltag -lz" -DTAGLIB_CFLAGS:STRING="-I$BUILDPREFIX/include/taglib -I$BUILDPREFIX/include -DTAGLIB_STATIC" -DCMAKE_CXX_FLAGS_DEBUG:STRING="-g -DID3LIB_LINKOPTION=1 -DFLAC__NO_DLL" -DCMAKE_INCLUDE_PATH=$BUILDPREFIX/include -DCMAKE_LIBRARY_PATH=$BUILDPREFIX/lib -DCMAKE_PROGRAM_PATH=$BUILDPREFIX/bin -DWITH_FFMPEG=ON -DFFMPEG_ROOT=$BUILDPREFIX -DWITH_MP4V2=ON -DCMAKE_BUILD_TYPE=Debug -DBUILD_SHARED_LIBS=OFF -DWITH_GCC_PCH=OFF -DWITH_APPS="Qt;CLI" -DCMAKE_INSTALL_PREFIX= -DWITH_BINDIR=. -DWITH_DATAROOTDIR=. -DWITH_DOCDIR=. -DWITH_TRANSLATIONSDIR=. ../../kid3
+      cat >kid3/build.sh <<EOF
+BUILDPREFIX=\$(cd ..; pwd)/buildroot/usr/local
+export PKG_CONFIG_PATH=\$BUILDPREFIX/lib/pkgconfig
+cmake -DWITH_TAGLIB=OFF -DHAVE_TAGLIB=1 -DTAGLIB_LIBRARIES:STRING="-L\$BUILDPREFIX/lib -ltag -lz" -DTAGLIB_CFLAGS:STRING="-I\$BUILDPREFIX/include/taglib -I\$BUILDPREFIX/include -DTAGLIB_STATIC" -DTAGLIB_VERSION:STRING="${taglib_version}" -DWITH_QT5=ON -DWITH_QML=OFF -DCMAKE_CXX_FLAGS_DEBUG:STRING="-g -DID3LIB_LINKOPTION=1 -DFLAC__NO_DLL" -DCMAKE_INCLUDE_PATH=\$BUILDPREFIX/include -DCMAKE_LIBRARY_PATH=\$BUILDPREFIX/lib -DCMAKE_PROGRAM_PATH=\$BUILDPREFIX/bin -DWITH_FFMPEG=ON -DFFMPEG_ROOT=\$BUILDPREFIX -DWITH_MP4V2=ON -DCMAKE_BUILD_TYPE=Debug -DBUILD_SHARED_LIBS=OFF -DWITH_GCC_PCH=OFF -DWITH_APPS="Qt;CLI" -DCMAKE_INSTALL_PREFIX= -DWITH_BINDIR=. -DWITH_DATAROOTDIR=. -DWITH_DOCDIR=. -DWITH_TRANSLATIONSDIR=. ../../kid3
 EOF
     fi
     chmod +x kid3/build.sh
