@@ -151,8 +151,8 @@ void CliCommand::timerEvent(QTimerEvent*) {
  * @param useDefault if true use cli()->tagMask() if no parameter found
  * @return tag versions.
  */
-TrackData::TagVersion CliCommand::getTagMaskParameter(int nr,
-                                                      bool useDefault) const
+Frame::TagVersion CliCommand::getTagMaskParameter(int nr,
+                                                  bool useDefault) const
 {
   int tagMask = 0;
   if (m_args.size() > nr) {
@@ -243,7 +243,6 @@ QuitCommand::QuitCommand(Kid3Cli* processor) :
 
 void QuitCommand::startCommand()
 {
-  cli()->app()->updateModified();
   if (cli()->app()->isModified() && !cli()->app()->getDirName().isEmpty()) {
     if (!(args().size() > 1 && args().at(1) == QLatin1String("force"))) {
       cli()->writeLine(tr("The current directory has been modified."));
@@ -284,35 +283,14 @@ void CdCommand::startCommand()
 
 void CdCommand::connectResultSignal()
 {
-  connect(cli()->app(),
-    SIGNAL(directoryOpened(QPersistentModelIndex,QList<QPersistentModelIndex>)),
-    this,
-    SLOT(onDirectoryOpened(QPersistentModelIndex,QList<QPersistentModelIndex>)));
+  connect(cli()->app(), SIGNAL(directoryOpened()),
+    this, SLOT(terminate()));
 }
 
 void CdCommand::disconnectResultSignal()
 {
-  disconnect(cli()->app(),
-    SIGNAL(directoryOpened(QPersistentModelIndex,QList<QPersistentModelIndex>)),
-    this,
-    SLOT(onDirectoryOpened(QPersistentModelIndex,QList<QPersistentModelIndex>)));
-}
-
-void CdCommand::onDirectoryOpened(
-    const QPersistentModelIndex& dirIndex,
-    const QList<QPersistentModelIndex>& fileIndexes)
-{
-  Q_UNUSED(dirIndex)
-  QItemSelectionModel* selModel = cli()->app()->getFileSelectionModel();
-  if (selModel && !fileIndexes.isEmpty()) {
-    foreach (const QPersistentModelIndex& fileIndex, fileIndexes) {
-      selModel->select(fileIndex,
-                       QItemSelectionModel::Select | QItemSelectionModel::Rows);
-    }
-    selModel->setCurrentIndex(fileIndexes.first(),
-                              QItemSelectionModel::NoUpdate);
-  }
-  terminate();
+  disconnect(cli()->app(), SIGNAL(directoryOpened()),
+    this, SLOT(terminate()));
 }
 
 
@@ -403,8 +381,8 @@ TagCommand::TagCommand(Kid3Cli* processor) :
 
 void TagCommand::startCommand()
 {
-  TrackData::TagVersion tagMask = getTagMaskParameter(1, false);
-  if (tagMask != TrackData::TagNone) {
+  Frame::TagVersion tagMask = getTagMaskParameter(1, false);
+  if (tagMask != Frame::TagNone) {
     cli()->setTagMask(tagMask);
   } else {
     cli()->writeTagMask();
@@ -422,12 +400,12 @@ void GetCommand::startCommand()
 {
   int numArgs = args().size();
   QString name = numArgs > 1 ? args().at(1) : QLatin1String("all");
-  TrackData::TagVersion tagMask = getTagMaskParameter(2);
+  Frame::TagVersion tagMask = getTagMaskParameter(2);
   if (name == QLatin1String("all")) {
     cli()->writeFileInformation(tagMask);
   } else {
-    foreach (TrackData::TagVersion tagBit, QList<TrackData::TagVersion>()
-             << TrackData::TagV2 << TrackData::TagV1) {
+    foreach (Frame::TagVersion tagBit, QList<Frame::TagVersion>()
+             << Frame::TagV2 << Frame::TagV1) {
       if (tagMask & tagBit) {
         QString value = cli()->app()->getFrame(tagBit, name);
         if (!value.isEmpty()) {
@@ -452,7 +430,7 @@ void SetCommand::startCommand()
   if (numArgs > 2) {
     const QString& name = args().at(1);
     const QString& value = args().at(2);
-    TrackData::TagVersion tagMask = getTagMaskParameter(3);
+    Frame::TagVersion tagMask = getTagMaskParameter(3);
     if (cli()->app()->setFrame(tagMask, name, value)) {
       cli()->updateSelectedFiles();
       cli()->updateSelection();
@@ -493,20 +471,20 @@ void ImportCommand::startCommand()
     bool ok;
     int fmtIdx = fmtName.toInt(&ok);
     if (!ok) {
-      fmtIdx = ImportConfig::instance().m_importFormatNames.indexOf(fmtName);
+      fmtIdx = ImportConfig::instance().importFormatNames().indexOf(fmtName);
       if (fmtIdx == -1) {
         QString errMsg = tr("%1 not found.").arg(fmtName);
         errMsg += QLatin1Char('\n');
         errMsg += tr("Available");
         errMsg += QLatin1String(": ");
-        errMsg += ImportConfig::instance().m_importFormatNames.join(
+        errMsg += ImportConfig::instance().importFormatNames().join(
               QLatin1String(", "));
         errMsg += QLatin1Char('.');
         setError(errMsg);
         return;
       }
     }
-    TrackData::TagVersion tagMask = getTagMaskParameter(3);
+    Frame::TagVersion tagMask = getTagMaskParameter(3);
     if (!cli()->app()->importTags(tagMask, path, fmtIdx)) {
         setError(tr("Error"));
     }
@@ -529,15 +507,13 @@ void BatchImportCommand::startCommand()
   int numArgs = args().size();
   const QString& profileName = numArgs > 1
       ? args().at(1) : QLatin1String("All");
-  TrackData::TagVersion tagMask = getTagMaskParameter(2);
-  if (BatchImportConfig::instance().getProfileByName(profileName, m_profile)) {
-    cli()->app()->batchImport(m_profile, tagMask);
-  } else {
+  Frame::TagVersion tagMask = getTagMaskParameter(2);
+  if (!cli()->app()->batchImport(profileName, tagMask)) {
     QString errMsg = tr("%1 not found.").arg(profileName);
     errMsg += QLatin1Char('\n');
     errMsg += tr("Available");
     errMsg += QLatin1String(": ");
-    errMsg += BatchImportConfig::instance().m_profileNames.join(
+    errMsg += BatchImportConfig::instance().profileNames().join(
           QLatin1String(", "));
     errMsg += QLatin1Char('.');
     setError(errMsg);
@@ -548,10 +524,8 @@ void BatchImportCommand::startCommand()
 void BatchImportCommand::connectResultSignal()
 {
   BatchImporter* importer = cli()->app()->getBatchImporter();
-  connect(importer,
-      SIGNAL(reportImportEvent(BatchImportProfile::ImportEventType,QString)),
-      this,
-      SLOT(onReportImportEvent(BatchImportProfile::ImportEventType,QString)));
+  connect(importer, SIGNAL(reportImportEvent(int,QString)),
+          this, SLOT(onReportImportEvent(int,QString)));
   connect(importer, SIGNAL(finished()),
           this, SLOT(terminate()));
 }
@@ -559,48 +533,45 @@ void BatchImportCommand::connectResultSignal()
 void BatchImportCommand::disconnectResultSignal()
 {
   BatchImporter* importer = cli()->app()->getBatchImporter();
-  disconnect(importer,
-      SIGNAL(reportImportEvent(BatchImportProfile::ImportEventType,QString)),
-      this,
-      SLOT(onReportImportEvent(BatchImportProfile::ImportEventType,QString)));
+  disconnect(importer, SIGNAL(reportImportEvent(int,QString)),
+             this, SLOT(onReportImportEvent(int,QString)));
   disconnect(importer, SIGNAL(finished()),
              this, SLOT(terminate()));
 }
 
-void BatchImportCommand::onReportImportEvent(
-    BatchImportProfile::ImportEventType type, const QString& text)
+void BatchImportCommand::onReportImportEvent(int type, const QString& text)
 {
   QString eventText;
   switch (type) {
-  case BatchImportProfile::ReadingDirectory:
+  case BatchImporter::ReadingDirectory:
     eventText = tr("Reading Directory");
     break;
-  case BatchImportProfile::Started:
+  case BatchImporter::Started:
     eventText = tr("Started");
     break;
-  case BatchImportProfile::SourceSelected:
+  case BatchImporter::SourceSelected:
     eventText = tr("Source");
     break;
-  case BatchImportProfile::QueryingAlbumList:
+  case BatchImporter::QueryingAlbumList:
     eventText = tr("Querying");
     break;
-  case BatchImportProfile::FetchingTrackList:
-  case BatchImportProfile::FetchingCoverArt:
+  case BatchImporter::FetchingTrackList:
+  case BatchImporter::FetchingCoverArt:
     eventText = tr("Fetching");
     break;
-  case BatchImportProfile::TrackListReceived:
+  case BatchImporter::TrackListReceived:
     eventText = tr("Data received");
     break;
-  case BatchImportProfile::CoverArtReceived:
+  case BatchImporter::CoverArtReceived:
     eventText = tr("Cover");
     break;
-  case BatchImportProfile::Finished:
+  case BatchImporter::Finished:
     eventText = tr("Finished");
     break;
-  case BatchImportProfile::Aborted:
+  case BatchImporter::Aborted:
     eventText = tr("Aborted");
     break;
-  case BatchImportProfile::Error:
+  case BatchImporter::Error:
     eventText = tr("Error");
   }
   if (!text.isEmpty()) {
@@ -624,11 +595,8 @@ void AlbumArtCommand::startCommand()
   int numArgs = args().size();
   if (numArgs > 1) {
     const QString& url = args().at(1);
-    Kid3Application::DownloadImageDestination dest =
-        numArgs > 2 && args().at(2) == QLatin1String("all")
-        ? Kid3Application::ImageForAllFilesInDirectory
-        : Kid3Application::ImageForSelectedFiles;
-    cli()->app()->downloadImage(url, dest);
+    cli()->app()->downloadImage(url,
+                  numArgs > 2 && args().at(2) == QLatin1String("all"));
   } else {
     showUsage();
     terminate();
@@ -674,20 +642,20 @@ void ExportCommand::startCommand()
     bool ok;
     int fmtIdx = fmtName.toInt(&ok);
     if (!ok) {
-      fmtIdx = ExportConfig::instance().m_exportFormatNames.indexOf(fmtName);
+      fmtIdx = ExportConfig::instance().exportFormatNames().indexOf(fmtName);
       if (fmtIdx == -1) {
         QString errMsg = tr("%1 not found.").arg(fmtName);
         errMsg += QLatin1Char('\n');
         errMsg += tr("Available");
         errMsg += QLatin1String(": ");
-        errMsg += ExportConfig::instance().m_exportFormatNames.join(
+        errMsg += ExportConfig::instance().exportFormatNames().join(
               QLatin1String(", "));
         errMsg += QLatin1Char('.');
         setError(errMsg);
         return;
       }
     }
-    TrackData::TagVersion tagMask = getTagMaskParameter(3);
+    Frame::TagVersion tagMask = getTagMaskParameter(3);
     if (!cli()->app()->exportTags(tagMask, path, fmtIdx)) {
       setError(tr("Error"));
     }
@@ -729,7 +697,7 @@ TagFormatCommand::TagFormatCommand(Kid3Cli* processor) :
 
 void TagFormatCommand::startCommand()
 {
-  cli()->app()->applyId3Format();
+  cli()->app()->applyTagFormat();
 }
 
 
@@ -754,15 +722,15 @@ RenameDirectoryCommand::RenameDirectoryCommand(Kid3Cli* processor) :
 
 void RenameDirectoryCommand::startCommand()
 {
-  TrackData::TagVersion tagMask = TrackData::TagNone;
+  Frame::TagVersion tagMask = Frame::TagNone;
   QString format;
   bool create = false;
   m_dryRun = false;
   for (int i = 1; i < args().size(); ++i) {
     bool ok = false;
-    if (tagMask == TrackData::TagNone) {
+    if (tagMask == Frame::TagNone) {
       tagMask = getTagMaskParameter(i, false);
-      ok = tagMask != TrackData::TagNone;
+      ok = tagMask != Frame::TagNone;
     }
     if (!ok) {
       const QString& param = args().at(i);
@@ -777,11 +745,11 @@ void RenameDirectoryCommand::startCommand()
       }
     }
   }
-  if (tagMask == TrackData::TagNone) {
+  if (tagMask == Frame::TagNone) {
     tagMask = cli()->tagMask();
   }
   if (format.isEmpty()) {
-    format = RenDirConfig::instance().m_dirFormatText;
+    format = RenDirConfig::instance().dirFormat();
   }
 
   if (!cli()->app()->renameDirectory(tagMask, format, create)) {
@@ -852,7 +820,7 @@ void NumberTracksCommand::startCommand()
   if (!ok) {
     firstTrackNr = 1;
   }
-  TrackData::TagVersion tagMask = getTagMaskParameter(2);
+  Frame::TagVersion tagMask = getTagMaskParameter(2);
   cli()->app()->numberTracks(firstTrackNr, 0, tagMask);
 }
 
@@ -868,9 +836,9 @@ void FilterCommand::startCommand()
 {
   if (args().size() > 1) {
     QString expression = args().at(1);
-    int fltIdx = FilterConfig::instance().m_filterNames.indexOf(expression);
+    int fltIdx = FilterConfig::instance().filterNames().indexOf(expression);
     if (fltIdx != -1) {
-      expression = FilterConfig::instance().m_filterExpressions.at(fltIdx);
+      expression = FilterConfig::instance().filterExpressions().at(fltIdx);
     } else if (!expression.isEmpty() &&
                !expression.contains(QLatin1Char('%'))) {
       // Probably an invalid expression
@@ -878,7 +846,7 @@ void FilterCommand::startCommand()
       errMsg += QLatin1Char('\n');
       errMsg += tr("Available");
       errMsg += QLatin1String(": ");
-      errMsg += FilterConfig::instance().m_filterNames.join(
+      errMsg += FilterConfig::instance().filterNames().join(
             QLatin1String(", "));
       errMsg += QLatin1Char('.');
       setError(errMsg);
@@ -894,21 +862,18 @@ void FilterCommand::startCommand()
 
 void FilterCommand::connectResultSignal()
 {
-  connect(cli()->app(),
-          SIGNAL(fileFiltered(FileFilter::FilterEventType,QString)),
-          this, SLOT(onFileFiltered(FileFilter::FilterEventType,QString)));
+  connect(cli()->app(), SIGNAL(fileFiltered(int,QString)),
+          this, SLOT(onFileFiltered(int,QString)));
 }
 
 void FilterCommand::disconnectResultSignal()
 {
   cli()->app()->abortFilter();
-  disconnect(cli()->app(),
-             SIGNAL(fileFiltered(FileFilter::FilterEventType,QString)),
-             this, SLOT(onFileFiltered(FileFilter::FilterEventType,QString)));
+  disconnect(cli()->app(), SIGNAL(fileFiltered(int,QString)),
+             this, SLOT(onFileFiltered(int,QString)));
 }
 
-void FilterCommand::onFileFiltered(FileFilter::FilterEventType type,
-                                   const QString& fileName)
+void FilterCommand::onFileFiltered(int type, const QString& fileName)
 {
   QString eventText;
   bool finish = false;
@@ -974,26 +939,23 @@ TagToFilenameCommand::TagToFilenameCommand(Kid3Cli* processor) :
 
 void TagToFilenameCommand::startCommand()
 {
-  TrackData::TagVersion tagMask = TrackData::TagNone;
+  Frame::TagVersion tagMask = Frame::TagNone;
   QString format;
   for (int i = 1; i < qMin(args().size(), 3); ++i) {
     bool ok = false;
-    if (tagMask == TrackData::TagNone) {
+    if (tagMask == Frame::TagNone) {
       tagMask = getTagMaskParameter(i, false);
-      ok = tagMask != TrackData::TagNone;
+      ok = tagMask != Frame::TagNone;
     }
     if (!ok && format.isEmpty()) {
       format = args().at(i);
     }
   }
-  if (tagMask == TrackData::TagNone) {
+  if (tagMask == Frame::TagNone) {
     tagMask = cli()->tagMask();
   }
   if (!format.isEmpty()) {
-    cli()->app()->setTagsToFilenameFormat(format);
-  }
-  if (cli()->app()->getTagsToFilenameFormat().isEmpty()) {
-    cli()->app()->setTagsToFilenameFormat(FileConfig::instance().m_formatText);
+    FileConfig::instance().setToFilenameFormat(format);
   }
   cli()->app()->getFilenameFromTags(tagMask);
 }
@@ -1007,27 +969,23 @@ FilenameToTagCommand::FilenameToTagCommand(Kid3Cli* processor) :
 
 void FilenameToTagCommand::startCommand()
 {
-  TrackData::TagVersion tagMask = TrackData::TagNone;
+  Frame::TagVersion tagMask = Frame::TagNone;
   QString format;
   for (int i = 1; i < qMin(args().size(), 3); ++i) {
     bool ok = false;
-    if (tagMask == TrackData::TagNone) {
+    if (tagMask == Frame::TagNone) {
       tagMask = getTagMaskParameter(i, false);
-      ok = tagMask != TrackData::TagNone;
+      ok = tagMask != Frame::TagNone;
     }
     if (!ok && format.isEmpty()) {
       format = args().at(i);
     }
   }
-  if (tagMask == TrackData::TagNone) {
+  if (tagMask == Frame::TagNone) {
     tagMask = cli()->tagMask();
   }
   if (!format.isEmpty()) {
-    cli()->app()->setFilenameToTagsFormat(format);
-  }
-  if (cli()->app()->getFilenameToTagsFormat().isEmpty()) {
-    cli()->app()->setFilenameToTagsFormat(
-          FileConfig::instance().m_formatFromFilenameText);
+    FileConfig::instance().setFromFilenameFormat(format);
   }
   cli()->app()->getTagsFromFilename(tagMask);
 }
@@ -1041,8 +999,8 @@ TagToOtherTagCommand::TagToOtherTagCommand(Kid3Cli* processor) :
 
 void TagToOtherTagCommand::startCommand()
 {
-  TrackData::TagVersion tagMask = getTagMaskParameter(1, false);
-  if (tagMask != TrackData::TagNone) {
+  Frame::TagVersion tagMask = getTagMaskParameter(1, false);
+  if (tagMask != Frame::TagNone) {
     cli()->app()->copyToOtherTag(tagMask);
   } else {
     showUsage();
@@ -1058,7 +1016,7 @@ CopyCommand::CopyCommand(Kid3Cli* processor) :
 
 void CopyCommand::startCommand()
 {
-  TrackData::TagVersion tagMask = getTagMaskParameter(1);
+  Frame::TagVersion tagMask = getTagMaskParameter(1);
   cli()->app()->copyTags(tagMask);
 }
 
@@ -1071,7 +1029,7 @@ PasteCommand::PasteCommand(Kid3Cli* processor) :
 
 void PasteCommand::startCommand()
 {
-  TrackData::TagVersion tagMask = getTagMaskParameter(1);
+  Frame::TagVersion tagMask = getTagMaskParameter(1);
   cli()->app()->pasteTags(tagMask);
 }
 
@@ -1084,7 +1042,7 @@ RemoveCommand::RemoveCommand(Kid3Cli* processor) :
 
 void RemoveCommand::startCommand()
 {
-  TrackData::TagVersion tagMask = getTagMaskParameter(1);
+  Frame::TagVersion tagMask = getTagMaskParameter(1);
   cli()->app()->removeTags(tagMask);
 }
 
