@@ -32,6 +32,7 @@
 #include "kid3application.h"
 #include "fileproxymodel.h"
 #include "frametablemodel.h"
+#include "taggedfileselection.h"
 #include "clicommand.h"
 #include "cliconfig.h"
 
@@ -191,26 +192,19 @@ QStringList splitArgs(const QString& str)
       }
     }
     do {
-      if (c == QLatin1Char('\'')) {
-        int spos = pos;
-        do {
-          if (pos >= str.size())
-            return QStringList();
-          c = str.at(pos++);
-        } while (c != QLatin1Char('\''));
-        param += str.midRef(spos, pos - spos - 1);
-      } else if (c == QLatin1Char('"')) {
+      if (c == QLatin1Char('"') || c == QLatin1Char('\'')) {
+        const QChar quote = c;
         for (;;) {
           if (pos >= str.size())
             return QStringList();
           c = str.at(pos++);
-          if (c == QLatin1Char('"'))
+          if (c == quote)
             break;
           if (c == QLatin1Char('\\')) {
             if (pos >= str.size())
               return QStringList();
             c = str.at(pos++);
-            if (c != QLatin1Char('"') && c != QLatin1Char('\\'))
+            if (c != quote && c != QLatin1Char('\\'))
               param += QLatin1Char('\\');
           }
           param += c;
@@ -246,7 +240,7 @@ Kid3Cli::Kid3Cli(Kid3Application* app,
                  AbstractCliIO* io, QObject* parent) :
   AbstractCli(io, parent),
   m_app(app),
-  m_tagMask(TrackData::TagV2V1), m_timeoutMs(0), m_fileNameChanged(false)
+  m_tagMask(Frame::TagV2V1), m_timeoutMs(0), m_fileNameChanged(false)
 {
   m_cmds << new HelpCommand(this)
          << new TimeoutCommand(this)
@@ -469,40 +463,17 @@ bool Kid3Cli::selectFile(const QStringList& paths)
   return ok;
 }
 
-/**
- * Get indexes of selected files.
- * @return selected indexes.
- */
-QList<QPersistentModelIndex> Kid3Cli::getSelection() const
-{
-  QList<QPersistentModelIndex> selection;
-  if (QItemSelectionModel* selModel = m_app->getFileSelectionModel()) {
-    foreach (const QModelIndex& index, selModel->selectedRows()) {
-      selection.append(QPersistentModelIndex(index));
-    }
-  }
-  return selection;
-}
-
 /**
  * Update the currently selected files from the frame tables.
  */
 void Kid3Cli::updateSelectedFiles()
 {
-  int selectionSize = m_selection.size();
-  if (selectionSize > 0) {
-    if (selectionSize > 1) {
-      m_app->frameModelV1()->selectChangedFrames();
-      m_app->frameModelV2()->selectChangedFrames();
-    }
-    m_app->frameModelsToTags(m_selection);
-    if (m_app->selectionSingleFile() && !m_filename.isEmpty()) {
-      if (TaggedFile* taggedFile =
-          FileProxyModel::getTaggedFileOfIndex(m_selection.first())) {
-        taggedFile->setFilename(m_filename);
-      }
-    }
+  TaggedFileSelection* selection = m_app->selectionInfo();
+  selection->selectChangedFrames();
+  if (!selection->isEmpty()) {
+    m_app->frameModelsToTags();
   }
+  selection->setFilename(m_filename);
 }
 
 /**
@@ -511,31 +482,15 @@ void Kid3Cli::updateSelectedFiles()
  */
 void Kid3Cli::updateSelection()
 {
-  m_selection = getSelection();
-  m_app->tagsToFrameModels(m_selection);
+  m_app->tagsToFrameModels();
 
-  if (m_app->selectionSingleFile()) {
-    m_filename = m_app->selectionSingleFile()->getFilename();
-    m_app->selectionSingleFile()->getDetailInfo(m_detailInfo);
-    m_tagFormatV1 = m_app->selectionSingleFile()->getTagFormatV1();
-    m_tagFormatV2 = m_app->selectionSingleFile()->getTagFormatV2();
-    m_fileNameChanged = m_app->selectionSingleFile()->isFilenameChanged();
-  } else {
-    if (m_app->selectionFileCount() > 1) {
-      m_filename.clear();
-    }
-    m_detailInfo = TaggedFile::DetailInfo();
-    m_tagFormatV1.clear();
-    m_tagFormatV2.clear();
-    m_fileNameChanged = false;
-  }
-  if (m_app->selectionTagV1SupportedCount() == 0) {
-    m_app->frameModelV1()->clearFrames();
-  }
-  if (m_app->selectionFileCount() == 0) {
-    m_app->frameModelV2()->clearFrames();
-    m_filename.clear();
-  }
+  TaggedFileSelection* selection = m_app->selectionInfo();
+  m_filename = selection->getFilename();
+  m_tagFormatV1 = selection->getTagFormatV1();
+  m_tagFormatV2 = selection->getTagFormatV2();
+  m_fileNameChanged = selection->isFilenameChanged();
+  m_detailInfo = selection->getDetailInfo();
+  selection->clearUnusedFrames();
 }
 
 /**
@@ -544,8 +499,8 @@ void Kid3Cli::updateSelection()
  */
 void Kid3Cli::writeFileInformation(int tagMask)
 {
-  if (m_detailInfo.valid) {
-    writeLine(tr("File") + QLatin1String(": ") + m_detailInfo.toString());
+  if (!m_detailInfo.isEmpty()) {
+    writeLine(tr("File") + QLatin1String(": ") + m_detailInfo);
   }
   if (!m_filename.isEmpty()) {
     QString line = m_fileNameChanged ? QLatin1String("*") : QLatin1String(" ");
@@ -555,10 +510,10 @@ void Kid3Cli::writeFileInformation(int tagMask)
     line += m_filename;
     writeLine(line);
   }
-  foreach (TrackData::TagVersion tagBit, QList<TrackData::TagVersion>()
-           << TrackData::TagV1 << TrackData::TagV2) {
+  foreach (Frame::TagVersion tagBit, QList<Frame::TagVersion>()
+           << Frame::TagV1 << Frame::TagV2) {
     if (tagMask & tagBit) {
-      FrameTableModel* ft = (tagBit == TrackData::TagV2)
+      FrameTableModel* ft = (tagBit == Frame::TagV2)
           ? m_app->frameModelV2() : m_app->frameModelV1();
       int maxLength = 0;
       bool hasValue = false;
@@ -571,7 +526,7 @@ void Kid3Cli::writeFileInformation(int tagMask)
         hasValue = hasValue || !value.isEmpty();
       }
       if (hasValue) {
-        if (tagBit == TrackData::TagV2) {
+        if (tagBit == Frame::TagV2) {
           writeLine(tr("Tag 2") + QLatin1String(": ") + m_tagFormatV2);
         } else {
           writeLine(tr("Tag 1") + QLatin1String(": ") + m_tagFormatV1);
@@ -605,16 +560,16 @@ void Kid3Cli::writeTagMask()
 {
   QString tagStr;
   switch (m_tagMask) {
-  case TrackData::TagV1:
+  case Frame::TagV1:
     tagStr = QLatin1String("1");
     break;
-  case TrackData::TagV2:
+  case Frame::TagV2:
     tagStr = QLatin1String("2");
     break;
-  case TrackData::TagV2V1:
+  case Frame::TagV2V1:
     tagStr = QLatin1String("1 & 2");
     break;
-  case TrackData::TagNone:
+  case Frame::TagNone:
     tagStr = QLatin1String("-");
     break;
   }
@@ -626,7 +581,7 @@ void Kid3Cli::writeTagMask()
  *
  * @param tagMask tag bits
  */
-void Kid3Cli::setTagMask(TrackData::TagVersion tagMask)
+void Kid3Cli::setTagMask(Frame::TagVersion tagMask)
 {
   m_tagMask = tagMask;
 }
@@ -651,7 +606,8 @@ void Kid3Cli::printFileProxyModel(const FileProxyModel* model,
   if (!model->hasChildren(parent))
     return;
 
-  QSet<QPersistentModelIndex> selection = getSelection().toSet();
+  m_app->updateCurrentSelection();
+  QSet<QPersistentModelIndex> selection = m_app->getCurrentSelection().toSet();
   for (int row = 0; row < model->rowCount(parent); ++row) {
     QString indentStr(indent, QLatin1Char(' ')), nameStr;
     QModelIndexList indexesWithChildren;
@@ -659,8 +615,7 @@ void Kid3Cli::printFileProxyModel(const FileProxyModel* model,
     QString propsStr = selection.contains(idx)
         ? QLatin1String(">") : QLatin1String(" ");
     if (TaggedFile* taggedFile = FileProxyModel::getTaggedFileOfIndex(idx)) {
-      taggedFile->readTags(false);
-      taggedFile = FileProxyModel::readWithId3V24IfId3V24(taggedFile);
+      taggedFile = FileProxyModel::readTagsFromTaggedFile(taggedFile);
       propsStr +=
           (taggedFile->isChanged() ? QLatin1String("*") : QLatin1String(" ")) +
           (taggedFile->hasTagV1() ? QLatin1Char('1') : QLatin1Char('-')) +
@@ -766,10 +721,8 @@ bool Kid3Cli::parseOptions()
   if (paths.isEmpty()) {
     paths.append(QDir::currentPath());
   }
-  connect(m_app,
-    SIGNAL(directoryOpened(QPersistentModelIndex,QList<QPersistentModelIndex>)),
-    this,
-    SLOT(onInitialDirectoryOpened(QPersistentModelIndex,QList<QPersistentModelIndex>)));
+  connect(m_app, SIGNAL(directoryOpened()),
+    this, SLOT(onInitialDirectoryOpened()));
   if (!openDirectory(expandWildcards(paths))) {
     writeErrorLine(tr("%1 does not exist").arg(paths.join(QLatin1String(", "))));
   }
@@ -779,28 +732,13 @@ bool Kid3Cli::parseOptions()
 /**
  * Select files passed as command line arguments after the initial directory has
  * been opened. Start execution of commands if existing.
- * @param dirIndex file proxy model index of opened directory
- * @param fileIndexes file proxy model indexes of selected files
  */
-void Kid3Cli::onInitialDirectoryOpened(
-    const QPersistentModelIndex& dirIndex,
-    const QList<QPersistentModelIndex>& fileIndexes)
+void Kid3Cli::onInitialDirectoryOpened()
 {
-  disconnect(m_app,
-    SIGNAL(directoryOpened(QPersistentModelIndex,QList<QPersistentModelIndex>)),
-    this,
-    SLOT(onInitialDirectoryOpened(QPersistentModelIndex,QList<QPersistentModelIndex>)));
-  QItemSelectionModel* selModel = m_app->getFileSelectionModel();
-  if (selModel && !fileIndexes.isEmpty()) {
-    foreach (const QPersistentModelIndex& fileIndex, fileIndexes) {
-      selModel->select(fileIndex,
-                       QItemSelectionModel::Select | QItemSelectionModel::Rows);
-    }
-    selModel->setCurrentIndex(fileIndexes.first(),
-                              QItemSelectionModel::NoUpdate);
-  }
+  disconnect(m_app, SIGNAL(directoryOpened()),
+    this, SLOT(onInitialDirectoryOpened()));
   if (!m_argCommands.isEmpty()) {
-    if (!dirIndex.isValid()) {
+    if (!m_app->getRootIndex().isValid()) {
       // Do not execute commands if directory could not be opened.
       m_argCommands.clear();
     }
@@ -811,7 +749,6 @@ void Kid3Cli::onInitialDirectoryOpened(
 void Kid3Cli::executeNextArgCommand()
 {
   if (m_argCommands.isEmpty()) {
-    m_app->updateModified();
     if (m_app->isModified() && !m_app->getDirName().isEmpty()) {
       // Automatically save changes in command mode.
       QStringList errorFiles = m_app->saveDirectory();
