@@ -114,7 +114,7 @@ void Kid3MainWindow::initActions()
   QMenu* fileMenu = menubar->addMenu(menuTitle);
 
   QAction* fileOpen = new QAction(this);
-  fileOpen->setStatusTip(tr("Opens a directory"));
+  fileOpen->setStatusTip(tr("Open files"));
   fileOpen->setText(tr("&Open..."));
   fileOpen->setShortcut(QKeySequence::Open);
   fileOpen->setIcon(QCM_QIcon_fromTheme("document-open"));
@@ -288,6 +288,10 @@ void Kid3MainWindow::initActions()
   editSelectAll->setText(tr("Select &All"));
   editSelectAll->setShortcut(Qt::ALT + Qt::Key_A);
   editSelectAll->setIcon(QCM_QIcon_fromTheme("edit-select-all"));
+#if defined Q_OS_MAC && QT_VERSION == 0x050400
+  // Workaround for QTBUG-43471 from QTBUG-39934
+  editSelectAll->setMenuRole(QAction::NoRole);
+#endif
   editSelectAll->setObjectName(QLatin1String("edit_select_all"));
   m_shortcutsModel->registerAction(editSelectAll, menuTitle);
   connect(editSelectAll, SIGNAL(triggered()),
@@ -310,7 +314,7 @@ void Kid3MainWindow::initActions()
   editSelectAllInDir->setObjectName(QLatin1String("select_all_in_directory"));
   m_shortcutsModel->registerAction(editSelectAllInDir, menuTitle);
   connect(editSelectAllInDir, SIGNAL(triggered()),
-    form(), SLOT(selectAllInDirectory()));
+    app(), SLOT(selectAllInDirectory()));
   editMenu->addAction(editSelectAllInDir);
 
   QAction* editPreviousFile = new QAction(this);
@@ -341,7 +345,9 @@ void Kid3MainWindow::initActions()
   editFind->setStatusTip(tr("Find"));
   editFind->setText(tr("&Find..."));
   editFind->setShortcut(QKeySequence::Find);
+#ifndef Q_OS_MAC
   editFind->setIcon(QCM_QIcon_fromTheme("edit-find"));
+#endif
   editFind->setObjectName(QLatin1String("edit_find"));
   m_shortcutsModel->registerAction(editFind, menuTitle);
   connect(editFind, SIGNAL(triggered()),
@@ -352,7 +358,9 @@ void Kid3MainWindow::initActions()
   editReplace->setStatusTip(tr("Find and replace"));
   editReplace->setText(tr("&Replace..."));
   editReplace->setShortcut(QKeySequence::Replace);
+#ifndef Q_OS_MAC
   editReplace->setIcon(QCM_QIcon_fromTheme("edit-find-replace"));
+#endif
   editReplace->setObjectName(QLatin1String("edit_replace"));
   m_shortcutsModel->registerAction(editReplace, menuTitle);
   connect(editReplace, SIGNAL(triggered()),
@@ -370,14 +378,14 @@ void Kid3MainWindow::initActions()
     app(), SLOT(applyFilenameFormat()));
   toolsMenu->addAction(toolsApplyFilenameFormat);
 
-  QAction* toolsApplyId3Format = new QAction(this);
-  toolsApplyId3Format->setStatusTip(tr("Apply Tag Format"));
-  toolsApplyId3Format->setText(tr("Apply &Tag Format"));
-  toolsApplyId3Format->setObjectName(QLatin1String("apply_id3_format"));
-  m_shortcutsModel->registerAction(toolsApplyId3Format, menuTitle);
-  connect(toolsApplyId3Format, SIGNAL(triggered()),
-    app(), SLOT(applyId3Format()));
-  toolsMenu->addAction(toolsApplyId3Format);
+  QAction* toolsApplyTagFormat = new QAction(this);
+  toolsApplyTagFormat->setStatusTip(tr("Apply Tag Format"));
+  toolsApplyTagFormat->setText(tr("Apply &Tag Format"));
+  toolsApplyTagFormat->setObjectName(QLatin1String("apply_id3_format"));
+  m_shortcutsModel->registerAction(toolsApplyTagFormat, menuTitle);
+  connect(toolsApplyTagFormat, SIGNAL(triggered()),
+    app(), SLOT(applyTagFormat()));
+  toolsMenu->addAction(toolsApplyTagFormat);
 
   QAction* toolsApplyTextEncoding = new QAction(this);
   toolsApplyTextEncoding->setStatusTip(tr("Apply Text Encoding"));
@@ -459,10 +467,10 @@ void Kid3MainWindow::initActions()
     m_viewToolBar->setText(tr("Show &Toolbar"));
     m_viewToolBar->setObjectName(QLatin1String("options_configure_toolbars"));
     m_shortcutsModel->registerAction(m_viewToolBar, menuTitle);
-    m_viewToolBar->setChecked(!MainWindowConfig::instance().m_hideToolBar);
+    m_viewToolBar->setChecked(!MainWindowConfig::instance().hideToolBar());
     settingsMenu->addAction(m_viewToolBar);
   }
-  if (MainWindowConfig::instance().m_hideToolBar)
+  if (MainWindowConfig::instance().hideToolBar())
     toolBar->hide();
 
   m_viewStatusBar = new QAction(this);
@@ -529,7 +537,9 @@ void Kid3MainWindow::initActions()
   QAction* helpAbout = new QAction(this);
   helpAbout->setStatusTip(tr("About Kid3"));
   helpAbout->setText(tr("&About Kid3"));
+#ifndef Q_OS_MAC
   helpAbout->setIcon(QCM_QIcon_fromTheme("help-about"));
+#endif
 #if QT_VERSION >= 0x040200
   helpAbout->setMenuRole(QAction::AboutRole);
 #endif
@@ -636,20 +646,20 @@ void Kid3MainWindow::addDirectoryToRecentFiles(const QString& dirName)
 void Kid3MainWindow::readConfig()
 {
   const MainWindowConfig& mainWindowConfig = MainWindowConfig::instance();
-  if (mainWindowConfig.m_hideStatusBar)
+  if (mainWindowConfig.hideStatusBar())
     statusBar()->hide();
-  m_viewStatusBar->setChecked(!mainWindowConfig.m_hideStatusBar);
-  m_settingsShowHidePicture->setChecked(!GuiConfig::instance().m_hidePicture);
-  m_settingsAutoHideTags->setChecked(GuiConfig::instance().m_autoHideTags);
+  m_viewStatusBar->setChecked(!mainWindowConfig.hideStatusBar());
+  m_settingsShowHidePicture->setChecked(!GuiConfig::instance().hidePicture());
+  m_settingsAutoHideTags->setChecked(GuiConfig::instance().autoHideTags());
   m_fileOpenRecent->loadEntries(app()->getSettings());
   m_shortcutsModel->readFromConfig(app()->getSettings());
-  if (!mainWindowConfig.m_geometry.isEmpty()) {
-    restoreGeometry(mainWindowConfig.m_geometry);
+  if (!mainWindowConfig.geometry().isEmpty()) {
+    restoreGeometry(mainWindowConfig.geometry());
   } else {
     resize(1000, 900);
   }
-  if (!mainWindowConfig.m_windowState.isEmpty()) {
-    restoreState(mainWindowConfig.m_windowState);
+  if (!mainWindowConfig.windowState().isEmpty()) {
+    restoreState(mainWindowConfig.windowState());
   }
 }
 
@@ -661,9 +671,9 @@ void Kid3MainWindow::saveConfig()
   MainWindowConfig& mainWindowConfig = MainWindowConfig::instance();
   m_fileOpenRecent->saveEntries(app()->getSettings());
   m_shortcutsModel->writeToConfig(app()->getSettings());
-  mainWindowConfig.m_hideToolBar = !m_viewToolBar->isChecked();
-  mainWindowConfig.m_geometry = saveGeometry();
-  mainWindowConfig.m_windowState = saveState();
+  mainWindowConfig.setHideToolBar(!m_viewToolBar->isChecked());
+  mainWindowConfig.setGeometry(saveGeometry());
+  mainWindowConfig.setWindowState(saveState());
   mainWindowConfig.writeToConfig(app()->getSettings());
 }
 
@@ -725,14 +735,14 @@ void Kid3MainWindow::closeEvent(QCloseEvent* ce)
 void Kid3MainWindow::readFontAndStyleOptions()
 {
   const MainWindowConfig& mainWindowConfig = MainWindowConfig::instance();
-  if (mainWindowConfig.m_useFont &&
-      !mainWindowConfig.m_fontFamily.isEmpty() &&
-      mainWindowConfig.m_fontSize > 0) {
-    QApplication::setFont(QFont(mainWindowConfig.m_fontFamily,
-                                mainWindowConfig.m_fontSize));
+  if (mainWindowConfig.useFont() &&
+      !mainWindowConfig.fontFamily().isEmpty() &&
+      mainWindowConfig.fontSize() > 0) {
+    QApplication::setFont(QFont(mainWindowConfig.fontFamily(),
+                                mainWindowConfig.fontSize()));
   }
-  if (!mainWindowConfig.m_style.isEmpty()) {
-    QApplication::setStyle(mainWindowConfig.m_style);
+  if (!mainWindowConfig.style().isEmpty()) {
+    QApplication::setStyle(mainWindowConfig.style());
   }
 }
 
@@ -751,9 +761,9 @@ void Kid3MainWindow::slotFileOpenRecentDirectory(const QString& dir)
  */
 void Kid3MainWindow::slotViewStatusBar()
 {
-  MainWindowConfig::instance().m_hideStatusBar = !m_viewStatusBar->isChecked();
+  MainWindowConfig::instance().setHideStatusBar(!m_viewStatusBar->isChecked());
   slotStatusMsg(tr("Toggle the statusbar..."));
-  if (MainWindowConfig::instance().m_hideStatusBar) {
+  if (MainWindowConfig::instance().hideStatusBar()) {
     statusBar()->hide();
   }
   else {
@@ -820,9 +830,9 @@ void Kid3MainWindow::onCommitDataRequest(QSessionManager& manager)
   // Make sure that current file is saved even if "load last opened file"
   // is not enabled.
   FileConfig& fileCfg = FileConfig::instance();
-  if (!fileCfg.m_loadLastOpenedFile) {
-    fileCfg.m_lastOpenedFile =
-        app()->getFileProxyModel()->filePath(app()->currentOrRootIndex());
+  if (!fileCfg.loadLastOpenedFile()) {
+    fileCfg.setLastOpenedFile(
+        app()->getFileProxyModel()->filePath(app()->currentOrRootIndex()));
   }
 
   // Ask user if there are unsaved data.
