@@ -27,6 +27,7 @@
 #ifndef GENERALCONFIG_H
 #define GENERALCONFIG_H
 
+#include <QObject>
 #include <QString>
 #include "isettings.h"
 #include "configstore.h"
@@ -34,7 +35,8 @@
 /**
  * Abstract base class for configurations.
  */
-class KID3_CORE_EXPORT GeneralConfig {
+class KID3_CORE_EXPORT GeneralConfig : public QObject {
+  Q_OBJECT
 public:
   /**
    * Constructor.
@@ -115,6 +117,7 @@ Derived& StoredConfig<Derived, Base>::instance() {
     obj = static_cast<Derived*>(store->configuration(Derived::s_index));
   } else {
     obj = new Derived;
+    obj->setParent(store);
     Derived::s_index = store->addConfiguration(obj);
   }
   return *obj;
