@@ -27,6 +27,7 @@
 #include "useractionsconfig.h"
 #include <QStringList>
 #include <stdlib.h>
+#include "config.h"
 
 int UserActionsConfig::s_index = -1;
 
@@ -87,6 +88,10 @@ void UserActionsConfig::readFromConfig(ISettings* config)
     if (strList.empty()) {
       break;
     }
+    if (strList.size() > 1 &&
+        strList.at(1) == QLatin1String("%{browser} http://images.google.com/images?q=%u{artist}%20%u{album}")) {
+      strList[1] = QLatin1String("%{browser} http://www.google.com/search?tbm=isch&q=%u{artist}%20%u{album}");
+    }
     m_contextMenuCommands.push_back(UserActionsConfig::MenuCommand(strList));
     ++cmdNr;
   }
@@ -104,7 +109,7 @@ void UserActionsConfig::readFromConfig(ISettings* config)
       UserActionsConfig::MenuCommand(QLatin1String("Amarok"), QLatin1String("amarok %{files}")));
 #endif
     m_contextMenuCommands.push_back(
-      UserActionsConfig::MenuCommand(QLatin1String("Google Images"), QLatin1String("%{browser} http://images.google.com/images?q=%u{artist}%20%u{album}")));
+      UserActionsConfig::MenuCommand(QLatin1String("Google Images"), QLatin1String("%{browser} http://www.google.com/search?tbm=isch&q=%u{artist}%20%u{album}")));
     m_contextMenuCommands.push_back(
       UserActionsConfig::MenuCommand(QLatin1String("Amazon"), QLatin1String("%{browser} http://www.amazon.com/s?search-alias=aps&field-keywords=%u{artist}+%u{album}")));
     m_contextMenuCommands.push_back(
@@ -132,6 +137,57 @@ void UserActionsConfig::readFromConfig(ISettings* config)
     m_contextMenuCommands.push_back(
       UserActionsConfig::MenuCommand(QLatin1String("LyrDB LRC"), QLatin1String("%{browser} http://lyrdb.com/karaoke/index.htm?q=%u{artist}+%u{title}&action=search")));
   }
+#ifdef HAVE_QML
+  if (ConfigStore::getConfigVersion() < 2) {
+    m_contextMenuCommands.push_back(
+      UserActionsConfig::MenuCommand(QLatin1String("QML Console"), QLatin1String("@qmlview %{qmlpath}/script/QmlConsole.qml"), false, true));
+    m_contextMenuCommands.push_back(
+      UserActionsConfig::MenuCommand(QLatin1String("ReplayGain to SoundCheck"), QLatin1String("@qml %{qmlpath}/script/ReplayGain2SoundCheck.qml"), false, true));
+    m_contextMenuCommands.push_back(
+      UserActionsConfig::MenuCommand(QLatin1String("Resize Album Art"), QLatin1String("@qml %{qmlpath}/script/ResizeAlbumArt.qml"), false, true));
+    m_contextMenuCommands.push_back(
+      UserActionsConfig::MenuCommand(QLatin1String("Extract Album Art"), QLatin1String("@qml %{qmlpath}/script/ExtractAlbumArt.qml"), false, true));
+    m_contextMenuCommands.push_back(
+      UserActionsConfig::MenuCommand(QLatin1String("Embed Album Art"), QLatin1String("@qml %{qmlpath}/script/EmbedAlbumArt.qml"), false, true));
+    m_contextMenuCommands.push_back(
+      UserActionsConfig::MenuCommand(QLatin1String("Embed Lyrics"), QLatin1String("@qml %{qmlpath}/script/EmbedLyrics.qml"), false, true));
+    m_contextMenuCommands.push_back(
+      UserActionsConfig::MenuCommand(QLatin1String("Text Encoding ID3v1"), QLatin1String("@qml %{qmlpath}/script/ShowTextEncodingV1.qml"), false, true));
+    m_contextMenuCommands.push_back(
+      UserActionsConfig::MenuCommand(QLatin1String("Export CSV"), QLatin1String("@qml %{qmlpath}/script/ExportCsv.qml %{directory}/export.csv"), false, true));
+  }
+#endif
+}
+
+void UserActionsConfig::setContextMenuCommands(const QList<MenuCommand>& contextMenuCommands)
+{
+  if (m_contextMenuCommands != contextMenuCommands) {
+    m_contextMenuCommands = contextMenuCommands;
+    emit contextMenuCommandsChanged();
+  }
+}
+
+QVariantList UserActionsConfig::contextMenuCommandVariantList() const
+{
+  QVariantList lst;
+  for (QList<MenuCommand>::const_iterator
+       it = m_contextMenuCommands.constBegin();
+       it != m_contextMenuCommands.constEnd();
+       ++it) {
+    lst.append(it->toStringList());
+  }
+  return lst;
+}
+
+void UserActionsConfig::setContextMenuCommandVariantList(const QVariantList& lst)
+{
+  QList<MenuCommand> commands;
+  for (QVariantList::const_iterator it = lst.constBegin();
+       it != lst.constEnd();
+       ++it) {
+    commands.append(MenuCommand(it->toStringList()));
+  }
+  setContextMenuCommands(commands);
 }
 
 
