@@ -48,7 +48,7 @@ enum DataFlags {
 BatchImporter::BatchImporter(QNetworkAccessManager* netMgr) : QObject(netMgr),
   m_downloadClient(new DownloadClient(netMgr)),
   m_currentImporter(0), m_trackDataModel(0), m_albumModel(0),
-  m_albumListItem(0), m_tagVersion(TrackData::TagNone), m_state(Idle),
+  m_albumListItem(0), m_tagVersion(Frame::TagNone), m_state(Idle),
   m_trackListNr(-1), m_sourceNr(-1), m_albumNr(-1),
   m_requestedData(0), m_importedData(0)
 {
@@ -84,12 +84,12 @@ void BatchImporter::setImporters(QList<ServerImporter*> importers,
  */
 void BatchImporter::start(const QList<ImportTrackDataVector>& trackLists,
                           const BatchImportProfile& profile,
-                          TrackData::TagVersion tagVersion)
+                          Frame::TagVersion tagVersion)
 {
   m_trackLists = trackLists;
   m_profile = profile;
   m_tagVersion = tagVersion;
-  emit reportImportEvent(BatchImportProfile::Started, profile.getName());
+  emit reportImportEvent(Started, profile.getName());
   m_trackListNr = -1;
   m_state = CheckNextTrackList;
   stateTransition();
@@ -102,7 +102,7 @@ void BatchImporter::start(const QList<ImportTrackDataVector>& trackLists,
  */
 bool BatchImporter::isAborted() const
 {
-  return m_state == Aborted;
+  return m_state == ImportAborted;
 }
 
 /**
@@ -110,7 +110,7 @@ bool BatchImporter::isAborted() const
  */
 void BatchImporter::clearAborted()
 {
-  if (m_state == Aborted) {
+  if (m_state == ImportAborted) {
     m_state = Idle;
     stateTransition();
   }
@@ -122,7 +122,7 @@ void BatchImporter::clearAborted()
 void BatchImporter::abort()
 {
   State oldState = m_state;
-  m_state = Aborted;
+  m_state = ImportAborted;
   if (oldState == Idle) {
     stateTransition();
   } else if (oldState == GettingCover) {
@@ -154,7 +154,7 @@ void BatchImporter::stateTransition()
             if (TaggedFile* taggedFile = trackList.first().getTaggedFile()) {
               FrameCollection frames;
               taggedFile->getTagsFromFilename(frames,
-                               FileConfig::instance().m_formatFromFilenameText);
+                               FileConfig::instance().fromFilenameFormat());
               m_currentArtist = frames.getArtist();
               m_currentAlbum = frames.getAlbum();
             }
@@ -171,7 +171,7 @@ void BatchImporter::stateTransition()
         m_importedData = 0;
         m_state = CheckNextSource;
       } else {
-        emit reportImportEvent(BatchImportProfile::Finished, QString());
+        emit reportImportEvent(Finished, QString());
         emit finished();
         m_state = Idle;
       }
@@ -201,7 +201,7 @@ void BatchImporter::stateTransition()
       }
     }
     if (m_currentImporter) {
-      emit reportImportEvent(BatchImportProfile::SourceSelected,
+      emit reportImportEvent(SourceSelected,
                              QString::fromLatin1(m_currentImporter->name()));
       m_state = GettingAlbumList;
     } else {
@@ -211,7 +211,7 @@ void BatchImporter::stateTransition()
     break;
   case GettingAlbumList:
     if (m_currentImporter) {
-      emit reportImportEvent(BatchImportProfile::QueryingAlbumList,
+      emit reportImportEvent(QueryingAlbumList,
                              m_currentArtist + QLatin1String(" - ") + m_currentAlbum);
       m_albumNr = -1;
       m_albumModel = 0;
@@ -246,7 +246,7 @@ void BatchImporter::stateTransition()
     break;
   case GettingTracks:
     if (m_albumListItem && m_currentImporter) {
-      emit reportImportEvent(BatchImportProfile::FetchingTrackList,
+      emit reportImportEvent(FetchingTrackList,
                              m_albumListItem->text());
       int pendingData = m_requestedData & ~m_importedData;
       // Also fetch standard tags, so that accuracy can be measured
@@ -266,13 +266,13 @@ void BatchImporter::stateTransition()
   case GettingCover:
     if (m_trackDataModel) {
       QUrl imgUrl;
-      if (m_tagVersion & TrackData::TagV2) {
-        QString coverArtUrl = m_trackDataModel->getTrackData().getCoverArtUrl();
+      if (m_tagVersion & Frame::TagV2) {
+        QUrl coverArtUrl = m_trackDataModel->getTrackData().getCoverArtUrl();
         if (!coverArtUrl.isEmpty()) {
           imgUrl = DownloadClient::getImageUrl(coverArtUrl);
           if (!imgUrl.isEmpty()) {
-            emit reportImportEvent(BatchImportProfile::FetchingCoverArt,
-                                   coverArtUrl);
+            emit reportImportEvent(FetchingCoverArt,
+                                   coverArtUrl.toString());
             m_downloadClient->startDownload(imgUrl);
           }
         }
@@ -291,8 +291,8 @@ void BatchImporter::stateTransition()
     }
     stateTransition();
     break;
-  case Aborted:
-    emit reportImportEvent(BatchImportProfile::Aborted, QString());
+  case ImportAborted:
+    emit reportImportEvent(Aborted, QString());
     break;
   }
 }
@@ -303,7 +303,7 @@ void BatchImporter::onFindFinished(const QByteArray& searchStr)
              this, SLOT(onFindFinished(QByteArray)));
   disconnect(m_currentImporter, SIGNAL(progress(QString,int,int)),
             this, SLOT(onFindProgress(QString,int,int)));
-  if (m_state == Aborted) {
+  if (m_state == ImportAborted) {
     stateTransition();
   } else if (m_currentImporter) {
     m_currentImporter->parseFindResults(searchStr);
@@ -320,7 +320,7 @@ void BatchImporter::onFindProgress(const QString& text, int step, int total)
                this, SLOT(onFindFinished(QByteArray)));
     disconnect(m_currentImporter, SIGNAL(progress(QString,int,int)),
               this, SLOT(onFindProgress(QString,int,int)));
-    emit reportImportEvent(BatchImportProfile::Error, text);
+    emit reportImportEvent(Error, text);
     m_state = CheckNextAlbum;
     stateTransition();
   }
@@ -332,13 +332,13 @@ void BatchImporter::onAlbumFinished(const QByteArray& albumStr)
              this, SLOT(onAlbumFinished(QByteArray)));
   disconnect(m_currentImporter, SIGNAL(progress(QString,int,int)),
              this, SLOT(onAlbumProgress(QString,int,int)));
-  if (m_state == Aborted) {
+  if (m_state == ImportAborted) {
     stateTransition();
   } else if (m_trackDataModel && m_currentImporter) {
     m_currentImporter->parseAlbumResults(albumStr);
 
     int accuracy = m_trackDataModel->calculateAccuracy();
-    emit reportImportEvent(BatchImportProfile::TrackListReceived,
+    emit reportImportEvent(TrackListReceived,
                            tr("Accuracy") + QLatin1Char(' ') +
                            (accuracy >= 0
                             ? QString::number(accuracy) + QLatin1Char('%')
@@ -356,15 +356,15 @@ void BatchImporter::onAlbumFinished(const QByteArray& albumStr)
             taggedFile->readTags(false);
             it->removeDisabledFrames(m_frameFilter);
             TagFormatConfig::instance().formatFramesIfEnabled(*it);
-            if (m_tagVersion & TrackData::TagV1) {
+            if (m_tagVersion & Frame::TagV1) {
               taggedFile->setFramesV1(*it, false);
             }
-            if (m_tagVersion & TrackData::TagV2) {
+            if (m_tagVersion & Frame::TagV2) {
               taggedFile->setFramesV2(*it, false);
             }
           }
         }
-        trackDataVector.setCoverArtUrl(QString());
+        trackDataVector.setCoverArtUrl(QUrl());
         m_trackLists[m_trackListNr] = trackDataVector;
       } else {
         // Revert imported data.
@@ -394,7 +394,7 @@ void BatchImporter::onAlbumProgress(const QString& text, int step, int total)
                this, SLOT(onAlbumFinished(QByteArray)));
     disconnect(m_currentImporter, SIGNAL(progress(QString,int,int)),
                this, SLOT(onAlbumProgress(QString,int,int)));
-    emit reportImportEvent(BatchImportProfile::Error, text);
+    emit reportImportEvent(Error, text);
     m_state = GettingCover;
     stateTransition();
   }
@@ -403,12 +403,12 @@ void BatchImporter::onAlbumProgress(const QString& text, int step, int total)
 void BatchImporter::onImageDownloaded(const QByteArray& data,
                                     const QString& mimeType, const QString& url)
 {
-  if (m_state == Aborted) {
+  if (m_state == ImportAborted) {
     stateTransition();
   } else {
     if (data.size() >= 1024) {
       if (mimeType.startsWith(QLatin1String("image")) && m_trackDataModel) {
-        emit reportImportEvent(BatchImportProfile::CoverArtReceived, url);
+        emit reportImportEvent(CoverArtReceived, url);
         PictureFrame frame(data, url, PictureFrame::PT_CoverFront, mimeType);
         ImportTrackDataVector trackDataVector(m_trackDataModel->getTrackData());
         for (ImportTrackDataVector::iterator it = trackDataVector.begin();
@@ -423,7 +423,7 @@ void BatchImporter::onImageDownloaded(const QByteArray& data,
       }
     } else {
       // Probably an invalid 1x1 picture from Amazon
-      emit reportImportEvent(BatchImportProfile::CoverArtReceived,
+      emit reportImportEvent(CoverArtReceived,
                              tr("Invalid File"));
     }
     m_state = CheckIfDone;
