@@ -31,10 +31,26 @@
 #include "fileconfig.h"
 #include "tagconfig.h"
 #include "formatconfig.h"
-#include "genres.h"
+#include "genremodel.h"
 #include "tracknumbervalidator.h"
 #include "pictureframe.h"
 
+namespace {
+
+QHash<int,QByteArray> getRoleHash()
+{
+  QHash<int, QByteArray> roles;
+  roles[Qt::CheckStateRole] = "checkState";
+  roles[FrameTableModel::FrameTypeRole] = "frameType";
+  roles[FrameTableModel::NameRole] = "name";
+  roles[FrameTableModel::ValueRole] = "value";
+  roles[FrameTableModel::ModifiedRole] = "modified";
+  roles[FrameTableModel::TruncatedRole] = "truncated";
+  return roles;
+}
+
+}
+
 /**
  * Constructor.
  * @param id3v1  true if model for ID3v1 frames
@@ -45,6 +61,9 @@ FrameTableModel::FrameTableModel(bool id3v1, QObject* parent) :
   m_id3v1(id3v1)
 {
   setObjectName(QLatin1String("FrameTableModel"));
+#if QT_VERSION < 0x050000
+  setRoleNames(getRoleHash());
+#endif
 }
 
 /**
@@ -112,12 +131,25 @@ QVariant FrameTableModel::data(const QModelIndex& index, int role) const
       index.column() < 0 || index.column() >= CI_NumColumns)
     return QVariant();
   FrameCollection::const_iterator it = frameAt(index.row());
+  bool isModified = false, isTruncated = false;
+  if ((role == Qt::BackgroundColorRole && index.column() == CI_Enable) ||
+      role == ModifiedRole) {
+    isModified = FileConfig::instance().markChanges() &&
+      (it->isValueChanged() ||
+      (static_cast<unsigned>((*it).getType()) < sizeof(m_changedFrames) * 8 &&
+       (m_changedFrames & (1ULL << (*it).getType())) != 0));
+  }
+  if ((role == Qt::BackgroundColorRole && index.column() == CI_Value) ||
+      role == TruncatedRole) {
+    isTruncated = (static_cast<unsigned>(index.row()) < sizeof(m_markedRows) * 8 &&
+        (m_markedRows & (1ULL << index.row())) != 0) || it->isMarked();
+  }
   if (role == Qt::DisplayRole || role == Qt::EditRole) {
     if (index.column() == CI_Enable) {
       QString displayName = getDisplayName(it->getName());
       if (it->getType() == Frame::FT_Picture &&
           it->getValue() != Frame::differentRepresentation()) {
-        QVariant fieldValue = it->getFieldValue(Frame::Field::ID_PictureType);
+        QVariant fieldValue = it->getFieldValue(Frame::ID_PictureType);
         if (fieldValue.isValid()) {
           PictureFrame::PictureType pictureType =
               static_cast<PictureFrame::PictureType>(fieldValue.toInt());
@@ -137,18 +169,20 @@ QVariant FrameTableModel::data(const QModelIndex& index, int role) const
     return m_frameSelected.at(index.row()) ? Qt::Checked : Qt::Unchecked;
   } else if (role == Qt::BackgroundColorRole) {
     if (index.column() == CI_Enable) {
-      return FileConfig::instance().m_markChanges &&
-        (it->isValueChanged() ||
-        (static_cast<unsigned>((*it).getType()) < sizeof(m_changedFrames) * 8 &&
-         (m_changedFrames & (1ULL << (*it).getType())) != 0))
-          ? QApplication::palette().mid() : Qt::NoBrush;
-    } else if (index.column() == CI_Value &&
-               static_cast<unsigned>(index.row()) < sizeof(m_markedRows) * 8) {
-      return (m_markedRows & (1ULL << index.row())) != 0
-             ? QBrush(Qt::red) : Qt::NoBrush;
+      return isModified ? QApplication::palette().mid() : Qt::NoBrush;
+    } else if (index.column() == CI_Value) {
+      return isTruncated ? QBrush(Qt::red) : Qt::NoBrush;
     }
   } else if (role == FrameTypeRole) {
     return it->getType();
+  } else if (role == NameRole) {
+    return getDisplayName(it->getName());
+  } else if (role == ValueRole) {
+    return it->getValue();
+  } else if (role == ModifiedRole) {
+    return isModified;
+  } else if (role == TruncatedRole) {
+    return isTruncated;
   }
   return QVariant();
 }
@@ -167,7 +201,8 @@ bool FrameTableModel::setData(const QModelIndex& index,
       index.row() < 0 || index.row() >= static_cast<int>(m_frames.size()) ||
       index.column() < 0 || index.column() >= CI_NumColumns)
     return false;
-  if (role == Qt::EditRole && index.column() == CI_Value) {
+  if ((role == Qt::EditRole && index.column() == CI_Value) ||
+      role == ValueRole) {
     QString valueStr(value.toString());
     FrameCollection::iterator it = frameAt(index.row());
     if (valueStr != (*it).getValue()) {
@@ -284,6 +319,18 @@ bool FrameTableModel::removeRows(int row, int count,
   return true;
 }
 
+#if QT_VERSION >= 0x050000
+/**
+ * Map role identifiers to role property names in scripting languages.
+ * @return hash mapping role identifiers to names.
+ */
+QHash<int,QByteArray> FrameTableModel::roleNames() const
+{
+  static QHash<int, QByteArray> roles = getRoleHash();
+  return roles;
+}
+#endif
+
 /**
  * Get the frame at a specific position in the collection.
  * @param row position of frame
@@ -331,6 +378,58 @@ int FrameTableModel::rowOf(FrameCollection::iterator frameIt) const {
   return row;
 }
 
+/**
+ * Mark rows.
+ * @param rowMask mask with bits of rows to mark
+ */
+void FrameTableModel::markRows(quint64 rowMask)
+{
+  quint64 changedBits = m_markedRows ^ rowMask;
+  m_markedRows = rowMask;
+
+  // Emit a change signal for all indexes affected by the change.
+  if (!changedBits)
+    return;
+
+  quint64 mask;
+  int row;
+  for (mask = 1ULL, row = 0;
+       static_cast<unsigned>(row) < sizeof(changedBits) * 8;
+       mask <<= 1, ++row) {
+    if ((changedBits & mask) != 0) {
+      // Include both the columns for Qt::BackgroundColorRole and TruncatedRole.
+      emit dataChanged(index(row, 0), index(row, 1));
+    }
+  }
+}
+
+/**
+ * Mark changed frames.
+ * @param frameMask mask with bits of frame types to mark
+ */
+void FrameTableModel::markChangedFrames(quint64 frameMask)
+{
+  quint64 changedBits = m_changedFrames ^ frameMask;
+  m_changedFrames = frameMask;
+
+  // Emit a change signal for all indexes affected by the change.
+  if (!FileConfig::instance().markChanges() || !changedBits)
+    return;
+
+  FrameCollection::const_iterator it;
+  int row;
+  for (it = m_frames.begin(), row = 0;
+       it != m_frames.end();
+       ++it, ++row) {
+    if (it->isValueChanged() ||
+        (static_cast<unsigned>((*it).getType()) < sizeof(changedBits) * 8 &&
+         (changedBits & (1ULL << (*it).getType())) != 0)) {
+      QModelIndex idx = index(row, CI_Enable);
+      emit dataChanged(idx, idx);
+    }
+  }
+}
+
 /**
  * Get frame for index.
  * @param index model index
@@ -504,9 +603,9 @@ void FrameTableModel::filterDifferent(FrameCollection& others)
  */
 void FrameTableModel::setAllCheckStates(bool checked)
 {
-  for (int row = 0; row < rowCount(); ++row) {
-    m_frameSelected[row] = checked;
-  }
+  const int numRows = rowCount();
+  m_frameSelected.fill(checked, 0, numRows);
+  emit dataChanged(index(0, CI_Enable), index(numRows - 1, CI_Enable));
 }
 
 /**
@@ -537,6 +636,8 @@ void FrameTableModel::selectChangedFrames()
        ++row, ++it) {
     if (it->isValueChanged()) {
       m_frameSelected[row] = true;
+      QModelIndex idx = index(row, CI_Enable);
+      emit dataChanged(idx, idx);
     }
   }
 }
@@ -564,9 +665,12 @@ void FrameTableModel::resizeFrameSelected()
 
 /**
  * Constructor.
+ * @param genreModel genre model
  * @param parent parent QTableView
  */
-FrameItemDelegate::FrameItemDelegate(QObject* parent) : QItemDelegate(parent),
+FrameItemDelegate::FrameItemDelegate(GenreModel* genreModel, QObject* parent) :
+  QItemDelegate(parent),
+  m_genreModel(genreModel),
   m_trackNumberValidator(new TrackNumberValidator(this)),
   m_dateTimeValidator(new QRegExpValidator(QRegExp(QLatin1String(
     // This is a simplified regular expression from
@@ -594,7 +698,7 @@ FrameItemDelegate::~FrameItemDelegate()
 void FrameItemDelegate::formatTextIfEnabled(const QString& txt)
 {
   QLineEdit* le;
-  if (TagFormatConfig::instance().m_formatWhileEditing &&
+  if (TagFormatConfig::instance().formatWhileEditing() &&
       (le = qobject_cast<QLineEdit*>(sender())) != 0) {
     QString str(txt);
     TagFormatConfig::instance().formatString(str);
@@ -633,32 +737,7 @@ QWidget* FrameItemDelegate::createEditor(
         cb->setDuplicatesEnabled(false);
       }
 
-      QStringList strList;
-      for (const char** sl = Genres::s_strList; *sl != 0; ++sl) {
-        strList += QString::fromLatin1(*sl);
-      }
-      if (TagConfig::instance().onlyCustomGenres()) {
-        cb->addItem(QLatin1String(""));
-      } else {
-        cb->addItems(strList);
-      }
-      QStringList customGenres = TagConfig::instance().customGenres();
-      if (id3v1) {
-        for (QStringList::const_iterator it = customGenres.begin();
-             it != customGenres.end();
-             ++it) {
-          if (Genres::getNumber(*it) != 255) {
-            cb->addItem(*it);
-          }
-        }
-        if (cb->count() <= 1) {
-          // No custom genres for ID3v1 => Show standard genres
-          cb->clear();
-          cb->addItems(strList);
-        }
-      } else {
-        cb->addItems(customGenres);
-      }
+      cb->setModel(m_genreModel);
       return cb;
     }
     QWidget* editor = QItemDelegate::createEditor(parent, option, index);
@@ -667,7 +746,7 @@ QWidget* FrameItemDelegate::createEditor(
         (type == Frame::FT_Comment || type == Frame::FT_Title ||
          type == Frame::FT_Artist || type == Frame::FT_Album)) {
       if (lineEdit) {
-        if (TagFormatConfig::instance().m_formatWhileEditing) {
+        if (TagFormatConfig::instance().formatWhileEditing()) {
           connect(lineEdit, SIGNAL(textChanged(QString)),
                   this, SLOT(formatTextIfEnabled(QString)));
         }
@@ -675,11 +754,11 @@ QWidget* FrameItemDelegate::createEditor(
       }
     } else {
       if (lineEdit) {
-        if (TagFormatConfig::instance().m_formatWhileEditing) {
+        if (TagFormatConfig::instance().formatWhileEditing()) {
           connect(lineEdit, SIGNAL(textChanged(QString)),
                   this, SLOT(formatTextIfEnabled(QString)));
         }
-        if (TagFormatConfig::instance().m_enableValidation) {
+        if (TagFormatConfig::instance().enableValidation()) {
           if (type == Frame::FT_Track || type == Frame::FT_Disc) {
             lineEdit->setValidator(m_trackNumberValidator);
           } else if (type == Frame::FT_Date || type == Frame::FT_OriginalDate) {
@@ -704,17 +783,7 @@ void FrameItemDelegate::setEditorData(
   QComboBox* cb = qobject_cast<QComboBox*>(editor);
   if (cb) {
     QString genreStr(index.model()->data(index).toString());
-    int genreIndex = genreStr.isNull() ? 0 :
-      Genres::getIndex(Genres::getNumber(genreStr));
-    if (TagConfig::instance().onlyCustomGenres()) {
-      genreIndex = cb->findText(genreStr);
-      if (genreIndex < 0) genreIndex = 0;
-    } else if (genreIndex <= 0) {
-      genreIndex = cb->findText(genreStr);
-      if (genreIndex < 0) genreIndex = Genres::count + 1;
-    }
-    cb->setItemText(genreIndex, genreStr);
-    cb->setCurrentIndex(genreIndex);
+    cb->setCurrentIndex(m_genreModel->getRowForGenre(genreStr));
   } else {
     QItemDelegate::setEditorData(editor, index);
   }
