@@ -33,6 +33,8 @@
 #include "frame.h"
 #include "kid3api.h"
 
+class GenreModel;
+
 /**
  * Model for table with frames.
  */
@@ -41,7 +43,11 @@ class KID3_CORE_EXPORT FrameTableModel : public QAbstractTableModel {
 public:
   /** Custom role. */
   enum Roles {
-    FrameTypeRole = Qt::UserRole + 1
+    FrameTypeRole = Qt::UserRole + 1,
+    NameRole = Qt::UserRole + 2,
+    ValueRole =  Qt::UserRole + 3,
+    ModifiedRole =  Qt::UserRole + 4,
+    TruncatedRole =  Qt::UserRole + 5
   };
 
   /** Column indices. */
@@ -144,6 +150,14 @@ public:
   virtual bool insertRows(int row, int count,
                           const QModelIndex& parent=QModelIndex());
 
+#if QT_VERSION >= 0x050000
+  /**
+   * Map role identifiers to role property names in scripting languages.
+   * @return hash mapping role identifiers to names.
+   */
+  virtual QHash<int,QByteArray> roleNames() const;
+#endif
+
   /**
    * Insert a frame.
    * @param frame frame to insert
@@ -160,16 +174,14 @@ public:
    * Mark rows.
    * @param rowMask mask with bits of rows to mark
    */
-  void markRows(quint64 rowMask) { m_markedRows = rowMask; }
+  void markRows(quint64 rowMask);
 
 
   /**
    * Mark changed frames.
    * @param frameMask mask with bits of frame types to mark
    */
-  void markChangedFrames(quint64 frameMask) {
-    m_changedFrames = frameMask;
-  }
+  void markChangedFrames(quint64 frameMask);
 
   /**
    * Get frame for index.
@@ -306,9 +318,10 @@ class KID3_CORE_EXPORT FrameItemDelegate : public QItemDelegate {
 public:
   /**
    * Constructor.
+   * @param genreModel genre model
    * @param parent parent QTableView
    */
-  explicit FrameItemDelegate(QObject* parent = 0);
+  explicit FrameItemDelegate(GenreModel* genreModel, QObject* parent = 0);
 
   /**
    * Destructor.
@@ -350,6 +363,7 @@ private slots:
   void formatTextIfEnabled(const QString& txt);
 
 private:
+  GenreModel* m_genreModel;
   QValidator* m_trackNumberValidator;
   QValidator* m_dateTimeValidator;
 };
