@@ -36,6 +36,12 @@
 
 /** Generalized frame. */
 class KID3_CORE_EXPORT Frame {
+  Q_GADGET
+  Q_ENUMS(Type)
+  Q_ENUMS(FieldId)
+  Q_ENUMS(TextEncoding)
+  Q_ENUMS(PictureType)
+  Q_ENUMS(TagVersion)
 public:
   /** Generalized frame types. */
   enum Type {
@@ -92,50 +98,88 @@ public:
     FT_UnknownFrame
   };
 
+  /**
+   * Types of fields in a frame, must be the same as id3lib's ID3_FieldID.
+   **/
+  enum FieldId {
+    ID_NoField,
+    ID_TextEnc,
+    ID_Text,
+    ID_Url,
+    ID_Data,
+    ID_Description,
+    ID_Owner,
+    ID_Email,
+    ID_Rating,
+    ID_Filename,
+    ID_Language,
+    ID_PictureType,
+    ID_ImageFormat,
+    ID_MimeType,
+    ID_Counter,
+    ID_Id,
+    ID_VolumeAdj,
+    ID_NumBits,
+    ID_VolChgRight,
+    ID_VolChgLeft,
+    ID_PeakVolRight,
+    ID_PeakVolLeft,
+    ID_TimestampFormat,
+    ID_ContentType,
+
+    // These are additional fields used by TagLib
+    ID_Price,
+    ID_Date,
+    ID_Seller,
+
+    // Additional field for METADATA_BLOCK_PICTURE
+    ID_ImageProperties
+  };
+
+  /** Text encoding for fields of type ID_TextEnc. */
+  enum TextEncoding {
+    TE_ISO8859_1 = 0,
+    TE_UTF16 = 1,
+    TE_UTF16BE = 2,
+    TE_UTF8 = 3
+  };
+
+  /** Picture type, compatible with ID3v2 and FLAC. */
+  enum PictureType {
+    PT_Other = 0,
+    PT_Png32Icon = 1,
+    PT_OtherIcon = 2,
+    PT_CoverFront = 3,
+    PT_CoverBack = 4,
+    PT_LeafletPage = 5,
+    PT_Media = 6,
+    PT_LeadArtist = 7,
+    PT_Artist = 8,
+    PT_Conductor = 9,
+    PT_Band = 10,
+    PT_Composer = 11,
+    PT_Lyricist = 12,
+    PT_RecordingLocation = 13,
+    PT_DuringRecording = 14,
+    PT_DuringPerformance = 15,
+    PT_Video = 16,
+    PT_Fish = 17,
+    PT_Illustration = 18,
+    PT_ArtistLogo = 19,
+    PT_PublisherLogo = 20
+  };
+
+  /** Tag version contained in track data. */
+  enum TagVersion {
+    TagNone = 0, /**< Empty or imported and not from a tag */
+    TagV1 = 1,   /**< Tag 1 */
+    TagV2 = 2,   /**< Tag 2 */
+    /** Tag 1 and 2 or merged from tag 2 and tag 1 (where tag 2 is not set) */
+    TagV2V1 = TagV1 | TagV2
+  };
+
   /** Field in frame. */
-  struct Field {
-    /**
-     * Types of fields in a frame, must be the same as id3lib's ID3_FieldID.
-     **/
-    enum Id {
-      ID_NoField,
-      ID_TextEnc,
-      ID_Text,
-      ID_Url,
-      ID_Data,
-      ID_Description,
-      ID_Owner,
-      ID_Email,
-      ID_Rating,
-      ID_Filename,
-      ID_Language,
-      ID_PictureType,
-      ID_ImageFormat,
-      ID_MimeType,
-      ID_Counter,
-      ID_Id,
-      ID_VolumeAdj,
-      ID_NumBits,
-      ID_VolChgRight,
-      ID_VolChgLeft,
-      ID_PeakVolRight,
-      ID_PeakVolLeft,
-      ID_TimestampFormat,
-      ID_ContentType,
-
-      // These are additional fields used by TagLib
-      ID_Price,
-      ID_Date,
-      ID_Seller
-    };
-
-    /** Text encoding for fields of type ID_TextEnc. */
-    enum TextEncoding {
-      TE_ISO8859_1 = 0,
-      TE_UTF16 = 1,
-      TE_UTF16BE = 2,
-      TE_UTF8 = 3
-    };
+  struct KID3_CORE_EXPORT Field {
 
 #ifdef Q_CC_MSVC
   /**
@@ -151,6 +195,63 @@ public:
 
     int m_id;         /**< type of field. */
     QVariant m_value; /**< value of field. */
+
+    /**
+     * Get a translated string for a field ID.
+     *
+     * @param type field ID type
+     *
+     * @return field ID type, null string if unknown.
+     */
+    static QString getFieldIdName(FieldId type);
+
+    /**
+     * List of field ID strings, NULL terminated.
+     */
+    static const char* const* getFieldIdNames();
+
+    /**
+     * Get a translated string for a text encoding.
+     *
+     * @param type text encoding type
+     *
+     * @return text encoding type, null string if unknown.
+     */
+    static QString getTextEncodingName(TextEncoding type);
+
+    /**
+     * List of text encoding strings, NULL terminated.
+     */
+    static const char* const* getTextEncodingNames();
+
+    /**
+     * Get a translated string for a timestamp format.
+     *
+     * @param type timestamp format type
+     *
+     * @return timestamp format type, null string if unknown.
+     */
+    static QString getTimestampFormatName(int type);
+
+    /**
+     * List of timestamp format strings, NULL terminated.
+     */
+    static const char* const* getTimestampFormatNames();
+
+    /**
+     * Get a translated string for a content type.
+     *
+     * @param type content type
+     *
+     * @return content type, null string if unknown.
+     */
+    static QString getContentTypeName(int type);
+
+    /**
+     * List of content type strings, NULL terminated.
+     */
+    static const char* const* getContentTypeNames();
+
   };
 
   /** list of fields. */
@@ -374,6 +475,18 @@ public:
    */
   void setDifferent() { m_value = differentRepresentation(); }
 
+  /**
+   * Check if frame is marked.
+   * @return true if marked.
+   */
+  bool isMarked() const { return m_marked; }
+
+  /**
+   * Mark frame
+   * @param marked true to mark, false to clear mark
+   */
+  void setMarked(bool marked) { m_marked = marked; }
+
   /**
    * Check if value is changed.
    * @return true if changed.
@@ -428,7 +541,7 @@ public:
    *
    * @return field value, invalid if field not found.
    */
-  QVariant getFieldValue(Field::Id id) const;
+  QVariant getFieldValue(FieldId id) const;
 
   /**
    * Check if the fields in another frame are equal.
@@ -479,7 +592,7 @@ public:
    *
    * @return true if field found and set.
    */
-  static bool setField(Frame& frame, Field::Id id, const QVariant& value);
+  static bool setField(Frame& frame, FieldId id, const QVariant& value);
 
   /**
    * Get value of a field.
@@ -489,7 +602,25 @@ public:
    *
    * @return field value, invalid if not found.
    */
-  static QVariant getField(const Frame& frame, Field::Id id);
+  static QVariant getField(const Frame& frame, FieldId id);
+
+  /**
+   * Get type of frame from translated name.
+   *
+   * @param translatedName name, spaces and case are ignored
+   *
+   * @return type.
+   */
+  static Type getTypeFromTranslatedName(const QString& translatedName);
+
+  /**
+   * Get a translated string for a frame type.
+   *
+   * @param type frame type
+   *
+   * @return frame type, null string if unknown.
+   */
+  static QString getFrameTypeName(Type type);
 
 private:
   friend class TaggedFile;
@@ -499,6 +630,7 @@ private:
   QString m_value;
   FieldList m_fieldList;
   bool m_valueChanged;
+  bool m_marked;
 };
 
 /** Filter to enable a subset of frame types. */
