@@ -34,31 +34,77 @@ class QImage;
 /** Frame containing picture. */
 class KID3_CORE_EXPORT PictureFrame : public Frame {
 public:
-  /** Picture type, compatible with ID3v2 and FLAC. */
-  enum PictureType {
-    PT_Other = 0,
-    PT_Png32Icon = 1,
-    PT_OtherIcon = 2,
-    PT_CoverFront = 3,
-    PT_CoverBack = 4,
-    PT_LeafletPage = 5,
-    PT_Media = 6,
-    PT_LeadArtist = 7,
-    PT_Artist = 8,
-    PT_Conductor = 9,
-    PT_Band = 10,
-    PT_Composer = 11,
-    PT_Lyricist = 12,
-    PT_RecordingLocation = 13,
-    PT_DuringRecording = 14,
-    PT_DuringPerformance = 15,
-    PT_Video = 16,
-    PT_Fish = 17,
-    PT_Illustration = 18,
-    PT_ArtistLogo = 19,
-    PT_PublisherLogo = 20
+  /**
+   * Additional properties for METADATA_BLOCK_PICTURE.
+   */
+  class KID3_CORE_EXPORT ImageProperties {
+  public:
+    /** Default constructor. */
+    ImageProperties() :
+      m_width(0), m_height(0), m_depth(0), m_numColors(0), m_imageHash(0) {}
+
+    /**
+     * Construct from properties in METADATA_BLOCK_PICTURE.
+     * @param width width of picture in pixels
+     * @param height height of picture in pixels
+     * @param depth color depth of picture in bits-per-pixel
+     * @param numColors number of colors used for indexed-color pictures
+     *                  (e.g. GIF), or 0 for non-indexed pictures
+     * @param data image data
+     */
+    ImageProperties(uint width, uint height, uint depth, uint numColors,
+                    const QByteArray& data) :
+      m_width(width), m_height(height), m_depth(depth), m_numColors(numColors),
+      m_imageHash(qHash(data)) {}
+
+    /**
+     * Construct properties from a new image.
+     * @param data image data
+     */
+    explicit ImageProperties(const QByteArray& data);
+
+    /**
+     * Check if the image properties are not set.
+     * @return true if not set.
+     */
+    bool isNull() const {
+      return m_width == 0 && m_height == 0 && m_depth == 0 &&
+          m_numColors == 0 && m_imageHash == 0;
+    }
+
+    /**
+     * Check if image properties are valid for an image.
+     * @param data image data
+     * @return true if valid.
+     */
+    bool isValidForImage(const QByteArray& data) const {
+      return !isNull() && qHash(data) == m_imageHash;
+    }
+
+    /** Width of picture in pixels. */
+    uint width() const { return m_width; }
+
+    /** Height of picture in pixels. */
+    uint height() const { return m_height; }
+
+    /** Color depth of picture in bits-per-pixel.*/
+    uint depth() const { return m_depth; }
+
+    /**
+     * Number of colors used for indexed-color pictures (e.g. GIF),
+     * or 0 for non-indexed pictures.
+     */
+    uint numColors() const { return m_numColors; }
+
+  private:
+    uint m_width;
+    uint m_height;
+    uint m_depth;
+    uint m_numColors;
+    uint m_imageHash;
   };
 
+
   /**
    * Constructor.
    *
@@ -74,7 +120,7 @@ public:
     const QString& description = QLatin1String(""),
     PictureType pictureType = PT_CoverFront,
     const QString& mimeType = QLatin1String("image/jpeg"),
-    Field::TextEncoding enc = Field::TE_ISO8859_1,
+    TextEncoding enc = TE_ISO8859_1,
     const QString& imgFormat = QLatin1String("JPG"));
 
   /**
@@ -99,12 +145,14 @@ public:
    * @param pictureType picture type
    * @param description description
    * @param data        binary picture data
+   * @param imgProps    optional METADATA_BLOCK_PICTURE image properties
    */
   static void setFields(
     Frame& frame,
-    Field::TextEncoding enc = Field::TE_ISO8859_1, const QString& imgFormat = QLatin1String("JPG"),
+    TextEncoding enc = TE_ISO8859_1, const QString& imgFormat = QLatin1String("JPG"),
     const QString& mimeType = QLatin1String("image/jpeg"), PictureType pictureType = PT_CoverFront,
-    const QString& description = QLatin1String(""), const QByteArray& data = QByteArray());
+    const QString& description = QLatin1String(""), const QByteArray& data = QByteArray(),
+    const ImageProperties* imgProps = 0);
 
   /**
    * Get all properties.
@@ -117,11 +165,13 @@ public:
    * @param pictureType picture type
    * @param description description
    * @param data        binary picture data
+   * @param imgProps    optional METADATA_BLOCK_PICTURE image properties
    */
   static void getFields(const Frame& frame,
-                        Field::TextEncoding& enc, QString& imgFormat,
+                        TextEncoding& enc, QString& imgFormat,
                         QString& mimeType, PictureType& pictureType,
-                        QString& description, QByteArray& data);
+                        QString& description, QByteArray& data,
+                        ImageProperties* imgProps = 0);
 
   /**
    * Check if all the fields of two picture frames are equal.
@@ -139,7 +189,7 @@ public:
    *
    * @return true if field found and set.
    */
-  static bool setTextEncoding(Frame& frame, Field::TextEncoding enc);
+  static bool setTextEncoding(Frame& frame, TextEncoding enc);
 
   /**
    * Get text encoding.
@@ -149,7 +199,7 @@ public:
    *
    * @return true if field found.
    */
-  static bool getTextEncoding(const Frame& frame, Field::TextEncoding& enc);
+  static bool getTextEncoding(const Frame& frame, TextEncoding& enc);
 
   /**
    * Set image format.
@@ -320,7 +370,32 @@ public:
    * Get list of picture type strings.
    * @return list of picture type names, NULL terminated.
    */
-  static const char** getPictureTypeNames();
+  static const char* const* getPictureTypeNames();
+
+  /**
+   * Get an untranslated string for a picture type.
+   *
+   * @param type picture type
+   *
+   * @return picture type, 0 if unknown.
+   */
+  static const char* getPictureTypeString(PictureType type);
+
+  /**
+   * List of untranslated picture type strings, NULL terminated.
+   */
+  static const char* const* getPictureTypeStrings();
+
+  /**
+   * Get picture type from an untranslated string.
+   *
+   * @param str untranslated picture type string
+   *
+   * @return picture type, PT_Other if unknown.
+   */
+  static PictureType getPictureTypeFromString(const char* str);
 };
 
+Q_DECLARE_METATYPE(PictureFrame::ImageProperties)
+
 #endif // PICTUREFRAME_H
