@@ -33,20 +33,23 @@
 #include "genres.h"
 #include "modeliterator.h"
 #include "saferename.h"
+#include "fileproxymodel.h"
 
 /**
  * Constructor.
  *
- * @param dn directory name
- * @param fn filename
- * @param idx model index
+ * @param idx index in file proxy model
  */
-TaggedFile::TaggedFile(const QString& dn, const QString& fn,
-                       const QPersistentModelIndex& idx) :
-  m_dirname(dn), m_filename(fn), m_newFilename(fn), m_index(idx),
-  m_changedFramesV1(0), m_changedFramesV2(0), m_truncation(0),
-  m_changedV1(false), m_changedV2(false)
+TaggedFile::TaggedFile(const QPersistentModelIndex& idx) :
+  m_index(idx), m_changedFramesV1(0), m_changedFramesV2(0), m_truncation(0),
+  m_changedV1(false), m_changedV2(false),
+  m_modified(false), m_marked(false)
 {
+  Q_ASSERT(m_index.model()->metaObject() == &FileProxyModel::staticMetaObject);
+  if (const FileProxyModel* model = getFileProxyModel()) {
+    m_newFilename = model->fileName(m_index);
+    m_filename = m_newFilename;
+  }
 }
 
 /**
@@ -56,6 +59,52 @@ TaggedFile::~TaggedFile()
 {
 }
 
+/**
+ * Get file proxy model.
+ * @return file proxy model.
+ */
+const FileProxyModel* TaggedFile::getFileProxyModel() const
+{
+  // The validity of this cast is checked in the constructor.
+  return static_cast<const FileProxyModel*>(m_index.model());
+}
+
+/**
+ * Get directory name.
+ *
+ * @return directory name
+ */
+QString TaggedFile::getDirname() const
+{
+  if (const FileProxyModel* model = getFileProxyModel()) {
+    return model->filePath(m_index.parent());
+  }
+  return QString();
+}
+
+/**
+ * Set file name.
+ *
+ * @param fn file name
+ */
+void TaggedFile::setFilename(const QString& fn)
+{
+  m_newFilename = fn;
+  updateModifiedState();
+}
+
+/**
+ * Get current path to file.
+ * @return absolute path.
+ */
+QString TaggedFile::currentFilePath() const
+{
+  if (const FileProxyModel* model = getFileProxyModel()) {
+    return model->filePath(m_index);
+  }
+  return QString();
+}
+
 /**
  * Get features supported.
  * @return bit mask with Feature flags set.
@@ -281,10 +330,19 @@ bool TaggedFile::isTagV1Supported() const
  */
 QString TaggedFile::getAbsFilename() const
 {
-  QDir dir(m_dirname);
+  QDir dir(getDirname());
   return QDir::cleanPath(dir.absoluteFilePath(m_newFilename));
 }
 
+/**
+ * Mark filename as unchanged.
+ */
+void TaggedFile::markFilenameUnchanged()
+{
+  m_filename = m_newFilename;
+  updateModifiedState();
+}
+
 /**
  * Mark tag 1 as changed.
  *
@@ -296,6 +354,17 @@ void TaggedFile::markTag1Changed(Frame::Type type)
   if (static_cast<unsigned>(type) < sizeof(m_changedFramesV1) * 8) {
     m_changedFramesV1 |= (1ULL << type);
   }
+  updateModifiedState();
+}
+
+/**
+ * Mark tag 1 as unchanged.
+ */
+void TaggedFile::markTag1Unchanged() {
+  m_changedV1 = false;
+  m_changedFramesV1 = 0;
+  clearTrunctionFlags();
+  updateModifiedState();
 }
 
 /**
@@ -309,6 +378,75 @@ void TaggedFile::markTag2Changed(Frame::Type type)
   if (static_cast<unsigned>(type) < sizeof(m_changedFramesV2) * 8) {
     m_changedFramesV2 |= (1ULL << type);
   }
+  updateModifiedState();
+}
+
+/**
+ * Mark tag 2 as unchanged.
+ */
+void TaggedFile::markTag2Unchanged()
+{
+  m_changedV2 = false;
+  m_changedFramesV2 = 0;
+  updateModifiedState();
+}
+
+/**
+ * Set the mask of the frame types changed in tag 2.
+ * @param mask mask of frame types
+ */
+void TaggedFile::setChangedFramesV2(quint64 mask) {
+  m_changedFramesV2 = mask;
+  m_changedV2 = mask != 0;
+  updateModifiedState();
+}
+
+void TaggedFile::updateModifiedState()
+{
+  bool modified = m_changedV1 || m_changedV2 || m_newFilename != m_filename;
+  if (m_modified != modified) {
+    m_modified = modified;
+    if (const FileProxyModel* model = getFileProxyModel()) {
+      const_cast<FileProxyModel*>(model)->notifyModificationChanged(
+            m_index, m_modified);
+    }
+  }
+}
+
+/**
+ * Notify model about changes in extra model data, e.g. the information on
+ * which the TaggedFileIconProvider depends.
+ *
+ * This method shall be called when such data changes, e.g. at the end of
+ * readTags() implementations.
+ *
+ * @param priorIsTagInformationRead prior value returned by
+ * isTagInformationRead()
+ */
+void TaggedFile::notifyModelDataChanged(bool priorIsTagInformationRead) const
+{
+  if (isTagInformationRead() != priorIsTagInformationRead) {
+    if (const FileProxyModel* model = getFileProxyModel()) {
+      const_cast<FileProxyModel*>(model)->notifyModelDataChanged(m_index);
+    }
+  }
+}
+
+/**
+ * Notify model about changes in the truncation state.
+ *
+ * This method shall be called when truncation is checked.
+ *
+ * @param priorTruncation prior value of m_truncation != 0
+ */
+void TaggedFile::notifyTruncationChanged(bool priorTruncation) const
+{
+  bool currentTruncation = m_truncation != 0;
+  if (currentTruncation != priorTruncation) {
+    if (const FileProxyModel* model = getFileProxyModel()) {
+      const_cast<FileProxyModel*>(model)->notifyModelDataChanged(m_index);
+    }
+  }
 }
 
 /**
@@ -540,16 +678,17 @@ QString TaggedFile::formatTime(unsigned seconds)
  */
 bool TaggedFile::renameFile(const QString& fnOld, const QString& fnNew) const
 {
+  QString dirname = getDirname();
   if (fnNew.toLower() == fnOld.toLower()) {
     // If the filenames only differ in case, the new file is reported to
     // already exist on case insensitive filesystems (e.g. Windows),
     // so it is checked if the new file is really the old file by
     // comparing inodes and devices. If the files are not the same,
     // another file would be overwritten and an error is reported.
-    if (QFile::exists(m_dirname + QDir::separator() + fnNew)) {
+    if (QFile::exists(dirname + QDir::separator() + fnNew)) {
       struct stat statOld, statNew;
-      if (::stat((m_dirname + QDir::separator() + fnOld).toLatin1().data(), &statOld) == 0 &&
-          ::stat((m_dirname + QDir::separator() + fnNew).toLatin1().data(), &statNew) == 0 &&
+      if (::stat((dirname + QDir::separator() + fnOld).toLatin1().data(), &statOld) == 0 &&
+          ::stat((dirname + QDir::separator() + fnNew).toLatin1().data(), &statNew) == 0 &&
           !(statOld.st_ino == statNew.st_ino &&
             statOld.st_dev == statNew.st_dev)) {
         qDebug("rename(%s, %s): %s already exists", fnOld.toLatin1().data(),
@@ -563,21 +702,21 @@ bool TaggedFile::renameFile(const QString& fnOld, const QString& fnNew) const
     // insensitive filesystems (e.g. Windows).
     QString temp_filename(fnNew);
     temp_filename.append(QLatin1String("_CASE"));
-    if (!Utils::safeRename(m_dirname, fnOld, temp_filename)) {
+    if (!Utils::safeRename(dirname, fnOld, temp_filename)) {
       qDebug("rename(%s, %s) failed", fnOld.toLatin1().data(),
              temp_filename.toLatin1().data());
       return false;
     }
-    if (!Utils::safeRename(m_dirname, temp_filename, fnNew)) {
+    if (!Utils::safeRename(dirname, temp_filename, fnNew)) {
       qDebug("rename(%s, %s) failed", temp_filename.toLatin1().data(),
              fnNew.toLatin1().data());
       return false;
     }
-  } else if (QFile::exists(m_dirname + QDir::separator() + fnNew)) {
+  } else if (QFile::exists(dirname + QDir::separator() + fnNew)) {
     qDebug("rename(%s, %s): %s already exists", fnOld.toLatin1().data(),
            fnNew.toLatin1().data(), fnNew.toLatin1().data());
     return false;
-  } else if (!Utils::safeRename(m_dirname, fnOld, fnNew)) {
+  } else if (!Utils::safeRename(dirname, fnOld, fnNew)) {
     qDebug("rename(%s, %s) failed", fnOld.toLatin1().data(),
            fnNew.toLatin1().data());
     return false;
@@ -773,15 +912,17 @@ QString TaggedFile::getTagFormatV2() const
 QString TaggedFile::checkTruncation(
   const QString& str, quint64 flag, int len)
 {
+  bool priorTruncation = m_truncation != 0;
+  QString result;
   if (static_cast<int>(str.length()) > len) {
-    QString s = str;
-    s.truncate(len);
+    result = str;
+    result.truncate(len);
     m_truncation |= flag;
-    return s;
   } else {
     m_truncation &= ~flag;
-    return QString();
   }
+  notifyTruncationChanged(priorTruncation);
+  return result;
 }
 
 /**
@@ -796,13 +937,17 @@ QString TaggedFile::checkTruncation(
 int TaggedFile::checkTruncation(int val, quint64 flag,
                                 int max)
 {
+  bool priorTruncation = m_truncation != 0;
+  int result;
   if (val > max) {
     m_truncation |= flag;
-    return max;
+    result = max;
   } else {
     m_truncation &= ~flag;
-    return -1;
+    result = -1;
   }
+  notifyTruncationChanged(priorTruncation);
+  return result;
 }
 
 /**
@@ -1086,6 +1231,33 @@ void TaggedFile::getAllFramesV2(FrameCollection& frames)
   }
 }
 
+/**
+ * Update marked property of frame.
+ * If the frame is a picture and its size exceeds the configured
+ * maximum size, the frame is marked. This method should be called in
+ * reimplementations of getAllFramesV2().
+ *
+ * @param frame frame to check
+ * @see resetMarkedState()
+ */
+void TaggedFile::updateMarkedState(Frame& frame)
+{
+  if (frame.getType() == Frame::FT_Picture) {
+    const TagConfig& tagCfg = TagConfig::instance();
+    if (tagCfg.markOversizedPictures()) {
+      QVariant data = frame.getField(frame, Frame::ID_Data);
+      if (!data.isNull()) {
+        if (data.toByteArray().size() > tagCfg.maximumPictureSize()) {
+          frame.setMarked(true);
+          m_marked = true;
+          return;
+        }
+      }
+    }
+    frame.setMarked(false);
+  }
+}
+
 /**
  * Close any file handles which are held open by the tagged file object.
  * The default implementation does nothing. If a concrete subclass holds
@@ -1133,8 +1305,10 @@ void TaggedFile::setFramesV2(const FrameCollection& frames, bool onlyChanged)
             setFrameV2(myFrame);
           } else {
             // Such a frame does not exist, add a new one.
+            Frame addedFrame(*it);
+            addFrameV2(addedFrame);
             Frame myFrame(*it);
-            addFrameV2(myFrame);
+            myFrame.setIndex(addedFrame.getIndex());
             setFrameV2(myFrame);
           }
         }
