@@ -33,6 +33,8 @@
 #include <QPersistentModelIndex>
 #include "frame.h"
 
+class FileProxyModel;
+
 /** Base class for tagged files. */
 class KID3_CORE_EXPORT TaggedFile {
 public:
@@ -46,7 +48,8 @@ public:
     TF_ID3v22      = 1 << 1, /**< Supports ID3v2.2 tags */
     TF_ID3v23      = 1 << 2, /**< Supports ID3v2.3 tags */
     TF_ID3v24      = 1 << 3, /**< Supports ID3v2.4 tags */
-    TF_OggPictures = 1 << 4  /**< Supports pictures in Ogg files */
+    TF_OggPictures = 1 << 4, /**< Supports pictures in Ogg files */
+    TF_OggFlac     = 1 << 5  /**< Supports Ogg FLAC files */
   };
 
   /** Information about file. */
@@ -79,12 +82,9 @@ public:
   /**
    * Constructor.
    *
-   * @param dn directory name
-   * @param fn filename
-   * @param idx model index
+   * @param idx index in file proxy model
    */
-  TaggedFile(const QString& dn, const QString& fn,
-             const QPersistentModelIndex& idx);
+  explicit TaggedFile(const QPersistentModelIndex& idx);
 
   /**
    * Destructor.
@@ -96,7 +96,7 @@ public:
    *
    * @param fn file name
    */
-  void setFilename(const QString& fn) { m_newFilename = fn; }
+  void setFilename(const QString& fn);
 
   /**
    * Get file name.
@@ -110,7 +110,7 @@ public:
    *
    * @return directory name
    */
-  QString getDirname() const { return m_dirname; }
+  QString getDirname() const;
 
   /**
    * Get key of tagged file format.
@@ -145,6 +145,7 @@ public:
 
   /**
    * Read tags from file.
+   * Implementations should call notifyModelDataChanged().
    *
    * @param force true to force reading even if tags were already read.
    */
@@ -603,8 +604,7 @@ public:
    *
    * @return true if file was changed.
    */
-  bool isChanged() const { return m_changedV1 || m_changedV2 ||
-      m_newFilename != m_filename; }
+  bool isChanged() const { return m_modified; }
 
   /**
    * Check if filename is changed.
@@ -636,7 +636,7 @@ public:
   /**
    * Mark tag 2 as unchanged.
    */
-  void markTag2Unchanged() { m_changedV2 = false; m_changedFramesV2 = 0; }
+  void markTag2Unchanged();
 
   /**
    * Get the mask of the frame types changed in tag 1.
@@ -654,9 +654,7 @@ public:
    * Set the mask of the frame types changed in tag 2.
    * @param mask mask of frame types
    */
-  void setChangedFramesV2(quint64 mask) {
-    m_changedFramesV2 = mask; m_changedV2 = mask != 0;
-  }
+  void setChangedFramesV2(quint64 mask);
 
   /**
    * Get the truncation flags.
@@ -697,6 +695,11 @@ public:
    */
   const QPersistentModelIndex& getIndex() const { return m_index; }
 
+  /**
+   * Check if the file is marked.
+   */
+  bool isMarked() const { return m_marked; }
+
   /**
    * Format a time string "h:mm:ss".
    * If the time is less than an hour, the hour is not put into the
@@ -708,6 +711,16 @@ public:
    */
   static QString formatTime(unsigned seconds);
 
+  /**
+   * Split a track string into number and total.
+   *
+   * @param str track
+   * @param total the total is returned here if found, else 0
+   *
+   * @return number, 0 if parsing failed, -1 if str is null
+   */
+  static int splitNumberAndTotal(const QString& str, int* total=0);
+
   /**
    * Free static resources.
    */
@@ -732,16 +745,6 @@ protected:
    */
   QString getCommentFieldName() const;
 
-  /**
-   * Split a track string into number and total.
-   *
-   * @param str track
-   * @param total the total is returned here if found, else 0
-   *
-   * @return number, 0 if parsing failed, -1 if str is null
-   */
-  static int splitNumberAndTotal(const QString& str, int* total=0);
-
     /**
    * Get the total number of tracks if it is enabled.
    *
@@ -762,12 +765,18 @@ protected:
    * Get current filename.
    * @return existing name.
    */
-  const QString& currentFilename() const { return m_filename; }
+  QString currentFilename() const { return m_filename; }
 
   /**
-   * Set current filename to new filename.
+   * Get current path to file.
+   * @return absolute path.
    */
-  void updateCurrentFilename() { m_filename = m_newFilename; }
+  QString currentFilePath() const;
+
+  /**
+   * Mark filename as unchanged.
+   */
+  void markFilenameUnchanged();
 
   /**
    * Check if tag 1 was changed.
@@ -785,11 +794,7 @@ protected:
   /**
    * Mark tag 1 as unchanged.
    */
-  void markTag1Unchanged() {
-    m_changedV1 = false;
-    m_changedFramesV1 = 0;
-    clearTrunctionFlags();
-  }
+  void markTag1Unchanged();
 
   /**
    * Check if a string has to be truncated.
@@ -820,18 +825,65 @@ protected:
    */
   void clearTrunctionFlags() { m_truncation = 0; }
 
+  /**
+   * Get file proxy model.
+   * @return file proxy model.
+   */
+  const FileProxyModel* getFileProxyModel() const;
+
+  /**
+   * Notify model about changes in extra model data, e.g. the information on
+   * which the TaggedFileIconProvider depends.
+   *
+   * This method shall be called when such data changes, e.g. at the end of
+   * readTags() implementations.
+   *
+   * @param priorIsTagInformationRead prior value returned by
+   * isTagInformationRead()
+   */
+  void notifyModelDataChanged(bool priorIsTagInformationRead) const;
+
+  /**
+   * Notify model about changes in the truncation state.
+   *
+   * This method shall be called when truncation is checked.
+   *
+   * @param priorTruncation prior value of m_truncation != 0
+   */
+  void notifyTruncationChanged(bool priorTruncation) const;
+
+  /**
+   * Reset the marked state of this file.
+   * This method should be called in reimplementations of getAllFramesV2()
+   * before calling updateMarkedState(), which will mark the file if any
+   * frame is marked.
+   * @see isMarked(), updateMarkedState()
+   */
+  void resetMarkedState() { m_marked = false; }
+
+  /**
+   * Update marked property of frame.
+   * If the frame is a picture and its size exceeds the configured
+   * maximum size, the frame is marked. This method should be called in
+   * reimplementations of getAllFramesV2().
+   *
+   * @param frame frame to check
+   * @see resetMarkedState()
+   */
+  void updateMarkedState(Frame& frame);
+
 private:
   TaggedFile(const TaggedFile&);
   TaggedFile& operator=(const TaggedFile&);
 
-  /** Directory name */
-  QString m_dirname;
+  void updateModifiedState();
+
+  /** Index of file in model */
+  QPersistentModelIndex m_index;
   /** File name */
   QString m_filename;
   /** New file name */
   QString m_newFilename;
-  /** Index of file in model */
-  QPersistentModelIndex m_index;
   /** changed tag 1 frame types */
   quint64 m_changedFramesV1;
   /** changed tag 2 frame types */
@@ -842,6 +894,10 @@ private:
   bool m_changedV1;
   /** true if ID3v2 tags were changed */
   bool m_changedV2;
+  /** true if tagged file is modified */
+  bool m_modified;
+  /** true if tagged file is marked */
+  bool m_marked;
 };
 
 #endif // TAGGEDFILE_H
