@@ -25,7 +25,6 @@
  */
 
 #include "configdialogpages.h"
-#include <cstring>
 #include <QLayout>
 #include <QPushButton>
 #include <QLabel>
@@ -55,21 +54,6 @@
 #include "contexthelp.h"
 #include "frame.h"
 
-enum { TextEncodingV1Latin1Index = 13 };
-
-/**
- * Remove aliases in braces from text encoding combo box entry.
- *
- * @param comboEntry text encoding combo box entry
- *
- * @return codec name.
- */
-static QString getTextEncodingV1CodecName(const QString& comboEntry)
-{
-  int braceIdx = comboEntry.indexOf(QLatin1String(" ("));
-  return braceIdx == -1 ? comboEntry : comboEntry.left(braceIdx);
-}
-
 /**
  * Constructor.
  */
@@ -78,9 +62,10 @@ ConfigDialogPages::ConfigDialogPages(QObject* parent) : QObject(parent),
   m_markChangesCheckBox(0), m_coverFileNameLineEdit(0),
   m_markTruncationsCheckBox(0), m_textEncodingV1ComboBox(0),
   m_totalNumTracksCheckBox(0), m_commentNameComboBox(0),
-  m_pictureNameComboBox(0), m_genreNotNumericCheckBox(0),
+  m_pictureNameComboBox(0), m_markOversizedPicturesCheckBox(0),
+  m_maximumPictureSizeSpinBox(0), m_genreNotNumericCheckBox(0),
   m_textEncodingComboBox(0), m_id3v2VersionComboBox(0),
-  m_trackNumberDigitsSpinBox(0), m_fnFormatBox(0), m_id3FormatBox(0),
+  m_trackNumberDigitsSpinBox(0), m_fnFormatBox(0), m_tagFormatBox(0),
   m_onlyCustomGenresCheckBox(0), m_genresEditModel(0),
   m_quickAccessTagsModel(0), m_playOnDoubleClickCheckBox(0),
   m_commandsTable(0), m_commandsTableModel(0), m_browserLineEdit(0),
@@ -114,72 +99,7 @@ QWidget* ConfigDialogPages::createTagsPage()
   v1GroupBoxLayout->addWidget(m_markTruncationsCheckBox, 0, 0, 1, 2);
   QLabel* textEncodingV1Label = new QLabel(tr("Text &encoding:"), v1GroupBox);
   m_textEncodingV1ComboBox = new QComboBox(v1GroupBox);
-  static const char* const codecs[] = {
-    "Apple Roman (macintosh)",
-    "Big5",
-    "big5-0",
-    "Big5-HKSCS",
-    "big5hkscs-0",
-    "EUC-JP",
-    "EUC-KR",
-    "GB18030",
-    "GBK (windows-936)",
-    "hp-roman8",
-    "IBM850",
-    "IBM866",
-    "ISO-2022-JP (JIS7)",
-    "ISO-8859-1 (latin1)",
-    "ISO-8859-2 (latin2)",
-    "ISO-8859-3 (latin3)",
-    "ISO-8859-4 (latin4)",
-    "ISO-8859-5 (cyrillic)",
-    "ISO-8859-6 (arabic)",
-    "ISO-8859-7 (greek)",
-    "ISO-8859-8 (hebrew)",
-    "ISO-8859-9 (latin5)",
-    "ISO-8859-10 (latin6)",
-    "ISO-8859-13 (baltic)",
-    "ISO-8859-14 (latin8, iso-celtic)",
-    "ISO-8859-15 (latin9)",
-    "ISO-8859-16 (latin10)",
-    "ISO-10646-UCS-2 (UTF-16)",
-    "Iscii-Bng",
-    "Iscii-Dev",
-    "Iscii-Gjr",
-    "Iscii-Knd",
-    "Iscii-Mlm",
-    "Iscii-Ori",
-    "Iscii-Pnj",
-    "Iscii-Tlg",
-    "Iscii-Tml",
-    "jisx0201*-0",
-    "KOI8-R",
-    "KOI8-U",
-    "ksc5601.1987-0",
-    "mulelao-1",
-    "Shift_JIS (SJIS, MS_Kanji)",
-    "TIS-620 (ISO 8859-11)",
-    "TSCII",
-    "UTF-8",
-    "windows-1250",
-    "windows-1251",
-    "windows-1252",
-    "windows-1253",
-    "windows-1254",
-    "windows-1255",
-    "windows-1256",
-    "windows-1257",
-    "windows-1258",
-    "WINSAMI2 (WS2)",
-    0
-  };
-  Q_ASSERT(std::strcmp(codecs[TextEncodingV1Latin1Index], "ISO-8859-1 (latin1)") == 0);
-  const char* const* str = codecs;
-  m_textEncodingV1List.clear();
-  while (*str) {
-    m_textEncodingV1List += QString::fromLatin1(*str++);
-  }
-  m_textEncodingV1ComboBox->addItems(m_textEncodingV1List);
+  m_textEncodingV1ComboBox->addItems(TagConfig::getTextEncodingV1Names());
   m_textEncodingV1ComboBox->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum));
   textEncodingV1Label->setBuddy(m_textEncodingV1ComboBox);
   v1GroupBoxLayout->addWidget(textEncodingV1Label, 1, 0);
@@ -188,7 +108,8 @@ QWidget* ConfigDialogPages::createTagsPage()
   tag1Layout->addStretch();
 
   QWidget* tag2Page = new QWidget;
-  QVBoxLayout* tag2Layout = new QVBoxLayout(tag2Page);
+  QHBoxLayout* tag2Layout = new QHBoxLayout(tag2Page);
+  QVBoxLayout* tag2LeftLayout = new QVBoxLayout;
   QGroupBox* v2GroupBox = new QGroupBox(tr("ID3v2"), tag2Page);
   QGridLayout* v2GroupBoxLayout = new QGridLayout(v2GroupBox);
   m_totalNumTracksCheckBox = new QCheckBox(tr("Use &track/total number of tracks format"), v2GroupBox);
@@ -199,9 +120,7 @@ QWidget* ConfigDialogPages::createTagsPage()
   m_genreNotNumericCheckBox = new QCheckBox(tr("&Genre as text instead of numeric string"), v2GroupBox);
   QLabel* textEncodingLabel = new QLabel(tr("Text &encoding:"), v2GroupBox);
   m_textEncodingComboBox = new QComboBox(v2GroupBox);
-  m_textEncodingComboBox->insertItem(TagConfig::TE_ISO8859_1, tr("ISO-8859-1"));
-  m_textEncodingComboBox->insertItem(TagConfig::TE_UTF16, tr("UTF16"));
-  m_textEncodingComboBox->insertItem(TagConfig::TE_UTF8, tr("UTF8"));
+  m_textEncodingComboBox->addItems(TagConfig::getTextEncodingNames());
   m_textEncodingComboBox->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum));
   textEncodingLabel->setBuddy(m_textEncodingComboBox);
   v2GroupBoxLayout->addWidget(m_genreNotNumericCheckBox, 1, 0, 1, 2);
@@ -231,20 +150,17 @@ QWidget* ConfigDialogPages::createTagsPage()
   trackNumberDigitsLabel->setBuddy(m_trackNumberDigitsSpinBox);
   v2GroupBoxLayout->addWidget(trackNumberDigitsLabel, 4, 0);
   v2GroupBoxLayout->addWidget(m_trackNumberDigitsSpinBox, 4, 1);
-  tag2Layout->addWidget(v2GroupBox);
+  tag2LeftLayout->addWidget(v2GroupBox);
   QGroupBox* vorbisGroupBox = new QGroupBox(tr("Ogg/Vorbis"), tag2Page);
   QLabel* commentNameLabel = new QLabel(tr("Co&mment field name:"), vorbisGroupBox);
   m_commentNameComboBox = new QComboBox(vorbisGroupBox);
   QLabel* pictureNameLabel = new QLabel(tr("&Picture field name:"), vorbisGroupBox);
   m_pictureNameComboBox = new QComboBox(vorbisGroupBox);
   m_commentNameComboBox->setEditable(true);
-  QStringList items;
-  items += QLatin1String("COMMENT");
-  items += QLatin1String("DESCRIPTION");
-  m_commentNameComboBox->addItems(items);
+  m_commentNameComboBox->addItems(TagConfig::getCommentNames());
   m_commentNameComboBox->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum));
   commentNameLabel->setBuddy(m_commentNameComboBox);
-  m_pictureNameComboBox->addItems(QStringList() << QLatin1String("METADATA_BLOCK_PICTURE") << QLatin1String("COVERART"));
+  m_pictureNameComboBox->addItems(TagConfig::getPictureNames());
   m_pictureNameComboBox->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum));
   pictureNameLabel->setBuddy(m_pictureNameComboBox);
   QGridLayout* vorbisGroupBoxLayout = new QGridLayout(vorbisGroupBox);
@@ -253,11 +169,23 @@ QWidget* ConfigDialogPages::createTagsPage()
   vorbisGroupBoxLayout->addWidget(pictureNameLabel, 1, 0);
   vorbisGroupBoxLayout->addWidget(m_pictureNameComboBox, 1, 1);
   vorbisGroupBox->setLayout(vorbisGroupBoxLayout);
-  tag2Layout->addWidget(vorbisGroupBox);
+  tag2LeftLayout->addWidget(vorbisGroupBox);
   if (!(tagCfg.taggedFileFeatures() & TaggedFile::TF_OggPictures)) {
     vorbisGroupBox->hide();
   }
-  QHBoxLayout* genresQuickAccessLayout = new QHBoxLayout;
+  QGroupBox* pictureGroupBox = new QGroupBox(tr("Picture"), tag2Page);
+  QHBoxLayout* pictureGroupBoxLayout = new QHBoxLayout(pictureGroupBox);
+  m_markOversizedPicturesCheckBox =
+      new QCheckBox(tr("Mark if &larger than (bytes):"));
+  m_maximumPictureSizeSpinBox = new QSpinBox;
+  m_maximumPictureSizeSpinBox->setRange(0, INT_MAX);
+  pictureGroupBoxLayout->addWidget(m_markOversizedPicturesCheckBox);
+  pictureGroupBoxLayout->addWidget(m_maximumPictureSizeSpinBox);
+  tag2LeftLayout->addWidget(pictureGroupBox);
+  tag2LeftLayout->addStretch();
+  tag2Layout->addLayout(tag2LeftLayout);
+
+  QVBoxLayout* tag2RightLayout = new QVBoxLayout;
   QGroupBox* genresGroupBox = new QGroupBox(tr("Custom &Genres"), tag2Page);
   m_onlyCustomGenresCheckBox = new QCheckBox(tr("&Show only custom genres"), genresGroupBox);
   m_genresEditModel = new QStringListModel(genresGroupBox);
@@ -266,7 +194,7 @@ QWidget* ConfigDialogPages::createTagsPage()
   vbox->addWidget(m_onlyCustomGenresCheckBox);
   vbox->addWidget(genresEdit);
   genresGroupBox->setLayout(vbox);
-  genresQuickAccessLayout->addWidget(genresGroupBox);
+  tag2RightLayout->addWidget(genresGroupBox);
 
   QGroupBox* quickAccessTagsGroupBox = new QGroupBox(tr("&Quick Access Tags"));
   QVBoxLayout* quickAccessTagsLayout = new QVBoxLayout(quickAccessTagsGroupBox);
@@ -280,14 +208,14 @@ QWidget* ConfigDialogPages::createTagsPage()
   m_quickAccessTagsModel->setStringList(unifiedFrameNames);
   quickAccessTagsListView->setModel(m_quickAccessTagsModel);
   quickAccessTagsLayout->addWidget(quickAccessTagsListView);
-  genresQuickAccessLayout->addWidget(quickAccessTagsGroupBox);
-  tag2Layout->addLayout(genresQuickAccessLayout);
+  tag2RightLayout->addWidget(quickAccessTagsGroupBox);
+  tag2Layout->addLayout(tag2RightLayout);
 
   QWidget* tag1AndTag2Page = new QWidget;
   QVBoxLayout* tag1AndTag2Layout = new QVBoxLayout(tag1AndTag2Page);
-  QString id3FormatTitle(tr("&Tag Format"));
-  m_id3FormatBox = new FormatBox(id3FormatTitle, tag1AndTag2Page);
-  tag1AndTag2Layout->addWidget(m_id3FormatBox);
+  QString tagFormatTitle(tr("&Tag Format"));
+  m_tagFormatBox = new FormatBox(tagFormatTitle, tag1AndTag2Page);
+  tag1AndTag2Layout->addWidget(m_tagFormatBox);
 
   QTabWidget* tagsTabWidget = new QTabWidget;
   if (tagCfg.taggedFileFeatures() & TaggedFile::TF_ID3v11) {
@@ -461,17 +389,17 @@ void ConfigDialogPages::setConfig()
   const ImportConfig& importCfg = ImportConfig::instance();
 
   m_fnFormatBox->fromFormatConfig(fnCfg);
-  m_id3FormatBox->fromFormatConfig(id3Cfg);
+  m_tagFormatBox->fromFormatConfig(id3Cfg);
   m_markTruncationsCheckBox->setChecked(tagCfg.markTruncations());
   m_totalNumTracksCheckBox->setChecked(tagCfg.enableTotalNumberOfTracks());
-  m_loadLastOpenedFileCheckBox->setChecked(fileCfg.m_loadLastOpenedFile);
-  m_preserveTimeCheckBox->setChecked(fileCfg.m_preserveTime);
-  m_markChangesCheckBox->setChecked(fileCfg.m_markChanges);
-  m_coverFileNameLineEdit->setText(fileCfg.m_defaultCoverFileName);
+  m_loadLastOpenedFileCheckBox->setChecked(fileCfg.loadLastOpenedFile());
+  m_preserveTimeCheckBox->setChecked(fileCfg.preserveTime());
+  m_markChangesCheckBox->setChecked(fileCfg.markChanges());
+  m_coverFileNameLineEdit->setText(fileCfg.defaultCoverFileName());
   m_onlyCustomGenresCheckBox->setChecked(tagCfg.onlyCustomGenres());
   m_genresEditModel->setStringList(tagCfg.customGenres());
   m_quickAccessTagsModel->setBitMask(tagCfg.quickAccessFrames());
-  m_commandsTableModel->setCommandList(userActionsCfg.m_contextMenuCommands);
+  m_commandsTableModel->setCommandList(userActionsCfg.contextMenuCommands());
   int idx = m_commentNameComboBox->findText(tagCfg.commentName());
   if (idx >= 0) {
     m_commentNameComboBox->setCurrentIndex(idx);
@@ -479,31 +407,22 @@ void ConfigDialogPages::setConfig()
     m_commentNameComboBox->addItem(tagCfg.commentName());
     m_commentNameComboBox->setCurrentIndex(m_commentNameComboBox->count() - 1);
   }
-  m_pictureNameComboBox->setCurrentIndex(tagCfg.pictureNameItem());
+  m_pictureNameComboBox->setCurrentIndex(tagCfg.pictureNameIndex());
   m_genreNotNumericCheckBox->setChecked(tagCfg.genreNotNumeric());
-  int textEncodingV1Index = TextEncodingV1Latin1Index;
-  int index = 0;
-  for (QStringList::const_iterator it = m_textEncodingV1List.begin();
-       it != m_textEncodingV1List.end();
-       ++it) {
-    if (getTextEncodingV1CodecName(*it) == tagCfg.textEncodingV1()) {
-      textEncodingV1Index = index;
-      break;
-    }
-    ++index;
-  }
-  m_textEncodingV1ComboBox->setCurrentIndex(textEncodingV1Index);
+  m_textEncodingV1ComboBox->setCurrentIndex(tagCfg.textEncodingV1Index());
   m_textEncodingComboBox->setCurrentIndex(tagCfg.textEncoding());
   m_id3v2VersionComboBox->setCurrentIndex(
         m_id3v2VersionComboBox->findData(tagCfg.id3v2Version()));
   m_trackNumberDigitsSpinBox->setValue(tagCfg.trackNumberDigits());
-  m_browserLineEdit->setText(networkCfg.m_browser);
-  m_playOnDoubleClickCheckBox->setChecked(guiCfg.m_playOnDoubleClick);
-  m_proxyCheckBox->setChecked(networkCfg.m_useProxy);
-  m_proxyLineEdit->setText(networkCfg.m_proxy);
-  m_proxyAuthenticationCheckBox->setChecked(networkCfg.m_useProxyAuthentication);
-  m_proxyUserNameLineEdit->setText(networkCfg.m_proxyUserName);
-  m_proxyPasswordLineEdit->setText(networkCfg.m_proxyPassword);
+  m_markOversizedPicturesCheckBox->setChecked(tagCfg.markOversizedPictures());
+  m_maximumPictureSizeSpinBox->setValue(tagCfg.maximumPictureSize());
+  m_browserLineEdit->setText(networkCfg.browser());
+  m_playOnDoubleClickCheckBox->setChecked(guiCfg.playOnDoubleClick());
+  m_proxyCheckBox->setChecked(networkCfg.useProxy());
+  m_proxyLineEdit->setText(networkCfg.proxy());
+  m_proxyAuthenticationCheckBox->setChecked(networkCfg.useProxyAuthentication());
+  m_proxyUserNameLineEdit->setText(networkCfg.proxyUserName());
+  m_proxyPasswordLineEdit->setText(networkCfg.proxyPassword());
 
   QStringList metadataPlugins;
   QStringList pluginOrder = tagCfg.pluginOrder();
@@ -511,7 +430,7 @@ void ConfigDialogPages::setConfig()
     for (int i = 0; i < pluginOrder.size(); ++i) {
       metadataPlugins.append(QString());
     }
-    foreach (const QString& pluginName, tagCfg.getAvailablePlugins()) {
+    foreach (const QString& pluginName, tagCfg.availablePlugins()) {
       int pluginIdx = pluginOrder.indexOf(pluginName);
       if (pluginIdx >= 0) {
         metadataPlugins[pluginIdx] = pluginName;
@@ -521,7 +440,7 @@ void ConfigDialogPages::setConfig()
     }
     metadataPlugins.removeAll(QString());
   } else {
-    metadataPlugins = tagCfg.getAvailablePlugins();
+    metadataPlugins = tagCfg.availablePlugins();
   }
   quint64 metadataPluginsMask = 0;
   quint64 mask = 1;
@@ -534,11 +453,12 @@ void ConfigDialogPages::setConfig()
   m_enabledMetadataPluginsModel->setStringList(metadataPlugins);
   m_enabledMetadataPluginsModel->setBitMask(metadataPluginsMask);
 
-  QStringList importPlugins = importCfg.getAvailablePlugins();
+  QStringList importPlugins = importCfg.availablePlugins();
   quint64 importPluginsMask = 0;
   mask = 1;
+  QStringList disabledPlugins = importCfg.disabledPlugins();
   for (int i = 0; i < importPlugins.size(); ++i, mask <<= 1) {
-    if (!importCfg.m_disabledPlugins.contains(importPlugins.at(i))) {
+    if (!disabledPlugins.contains(importPlugins.at(i))) {
       importPluginsMask |= mask;
     }
   }
@@ -561,33 +481,34 @@ void ConfigDialogPages::getConfig() const
   ImportConfig& importCfg = ImportConfig::instance();
 
   m_fnFormatBox->toFormatConfig(fnCfg);
-  m_id3FormatBox->toFormatConfig(id3Cfg);
+  m_tagFormatBox->toFormatConfig(id3Cfg);
   tagCfg.setMarkTruncations(m_markTruncationsCheckBox->isChecked());
   tagCfg.setEnableTotalNumberOfTracks(m_totalNumTracksCheckBox->isChecked());
-  fileCfg.m_loadLastOpenedFile = m_loadLastOpenedFileCheckBox->isChecked();
-  fileCfg.m_preserveTime = m_preserveTimeCheckBox->isChecked();
-  fileCfg.m_markChanges = m_markChangesCheckBox->isChecked();
-  fileCfg.m_defaultCoverFileName = m_coverFileNameLineEdit->text();
+  fileCfg.setLoadLastOpenedFile(m_loadLastOpenedFileCheckBox->isChecked());
+  fileCfg.setPreserveTime(m_preserveTimeCheckBox->isChecked());
+  fileCfg.setMarkChanges(m_markChangesCheckBox->isChecked());
+  fileCfg.setDefaultCoverFileName(m_coverFileNameLineEdit->text());
   tagCfg.setOnlyCustomGenres(m_onlyCustomGenresCheckBox->isChecked());
   tagCfg.setCustomGenres(m_genresEditModel->stringList());
   tagCfg.setQuickAccessFrames(m_quickAccessTagsModel->getBitMask());
-  userActionsCfg.m_contextMenuCommands = m_commandsTableModel->getCommandList();
+  userActionsCfg.setContextMenuCommands(m_commandsTableModel->getCommandList());
   tagCfg.setCommentName(m_commentNameComboBox->currentText());
-  tagCfg.setPictureNameItem(m_pictureNameComboBox->currentIndex());
+  tagCfg.setPictureNameIndex(m_pictureNameComboBox->currentIndex());
   tagCfg.setGenreNotNumeric(m_genreNotNumericCheckBox->isChecked());
-  tagCfg.setTextEncodingV1(
-    getTextEncodingV1CodecName(m_textEncodingV1ComboBox->currentText()));
+  tagCfg.setTextEncodingV1Index(m_textEncodingV1ComboBox->currentIndex());
   tagCfg.setTextEncoding(m_textEncodingComboBox->currentIndex());
   tagCfg.setId3v2Version(m_id3v2VersionComboBox->itemData(
         m_id3v2VersionComboBox->currentIndex()).toInt());
   tagCfg.setTrackNumberDigits(m_trackNumberDigitsSpinBox->value());
-  networkCfg.m_browser = m_browserLineEdit->text();
-  guiCfg.m_playOnDoubleClick = m_playOnDoubleClickCheckBox->isChecked();
-  networkCfg.m_useProxy = m_proxyCheckBox->isChecked();
-  networkCfg.m_proxy = m_proxyLineEdit->text();
-  networkCfg.m_useProxyAuthentication = m_proxyAuthenticationCheckBox->isChecked();
-  networkCfg.m_proxyUserName = m_proxyUserNameLineEdit->text();
-  networkCfg.m_proxyPassword = m_proxyPasswordLineEdit->text();
+  tagCfg.setMarkOversizedPictures(m_markOversizedPicturesCheckBox->isChecked());
+  tagCfg.setMaximumPictureSize(m_maximumPictureSizeSpinBox->value());
+  networkCfg.setBrowser(m_browserLineEdit->text());
+  guiCfg.setPlayOnDoubleClick(m_playOnDoubleClickCheckBox->isChecked());
+  networkCfg.setUseProxy(m_proxyCheckBox->isChecked());
+  networkCfg.setProxy(m_proxyLineEdit->text());
+  networkCfg.setUseProxyAuthentication(m_proxyAuthenticationCheckBox->isChecked());
+  networkCfg.setProxyUserName(m_proxyUserNameLineEdit->text());
+  networkCfg.setProxyPassword(m_proxyPasswordLineEdit->text());
 
   QStringList pluginOrder, disabledPlugins;
   for (int row = 0; row < m_enabledMetadataPluginsModel->rowCount(); ++row) {
@@ -610,5 +531,5 @@ void ConfigDialogPages::getConfig() const
             m_enabledPluginsModel->index(row).data().toString());
     }
   }
-  importCfg.m_disabledPlugins = disabledPlugins;
+  importCfg.setDisabledPlugins(disabledPlugins);
 }
