@@ -38,6 +38,10 @@
 #include <QBuffer>
 #include <QVBoxLayout>
 #include <QMimeData>
+#if QT_VERSION >= 0x050000
+#include <QMimeDatabase>
+#include <QMimeType>
+#endif
 #include "kid3application.h"
 #include "imageviewer.h"
 #include "taggedfile.h"
@@ -156,7 +160,7 @@ public:
    * @param parent parent widget
    * @param strlst list with ComboBox items, terminated by NULL
    */
-  LabeledComboBox(QWidget* parent, const char** strlst);
+  LabeledComboBox(QWidget* parent, const char* const* strlst);
 
   /**
    * Destructor.
@@ -296,7 +300,7 @@ LabeledLineEdit::~LabeledLineEdit()
  * @param strlst list with ComboBox items, terminated by NULL
  */
 LabeledComboBox::LabeledComboBox(QWidget* parent,
-         const char **strlst) : QWidget(parent)
+         const char* const* strlst) : QWidget(parent)
 {
   setObjectName(QLatin1String("LabeledComboBox"));
   QVBoxLayout* layout = new QVBoxLayout(this);
@@ -371,14 +375,6 @@ public:
   virtual ~Mp3FieldControl();
 
 protected:
-  /**
-   * Get description for ID3_Field.
-   *
-   * @param id ID of field
-   * @return description or NULL if id unknown.
-   */
-  const char* getFieldIDString(Frame::Field::Id id) const;
-
   /** field */
   Frame::Field& m_field;
 };
@@ -501,7 +497,7 @@ public:
    * @param lst list of strings with possible selections, NULL terminated
    */
   IntComboBoxControl(Frame::Field& field,
-                     const char **lst) :
+                     const char* const* lst) :
     Mp3FieldControl(field), m_ptInp(0), m_strLst(lst) {}
 
   /**
@@ -527,7 +523,7 @@ protected:
   /** Combo box widget */
   LabeledComboBox* m_ptInp;
   /** List of strings with possible selections */
-  const char** m_strLst;
+  const char* const* m_strLst;
 };
 
 /** Control to import, export and view data from binary fields */
@@ -536,14 +532,16 @@ public:
   /**
    * Constructor.
    * @param platformTools platform tools
+   * @param app application context
    * @param field      field to edit
    * @param frame      frame with fields to edit
    * @param taggedFile file
    */
-  BinFieldControl(IPlatformTools* platformTools, Frame::Field& field,
+  BinFieldControl(IPlatformTools* platformTools, Kid3Application* app,
+                  Frame::Field& field,
                   const Frame& frame, const TaggedFile* taggedFile) :
-    Mp3FieldControl(field), m_platformTools(platformTools), m_bos(0),
-    m_frame(frame), m_taggedFile(taggedFile) {}
+    Mp3FieldControl(field), m_platformTools(platformTools), m_app(app),
+    m_bos(0), m_frame(frame), m_taggedFile(taggedFile) {}
 
   /**
    * Destructor.
@@ -567,6 +565,8 @@ public:
 protected:
   /** Platform dependent tools */
   IPlatformTools* m_platformTools;
+  /** Application context */
+  Kid3Application* m_app;
   /** Import, Export, View buttons */
   BinaryOpenSave* m_bos;
   /** frame with fields to edit */
@@ -634,14 +634,16 @@ protected:
  * Constructor.
  *
  * @param platformTools platform tools
+ * @param app application context
  * @param parent parent widget
  * @param field  field containing binary data
  */
 BinaryOpenSave::BinaryOpenSave(IPlatformTools* platformTools,
+                               Kid3Application* app,
                                QWidget* parent, const Frame::Field& field) :
   QWidget(parent),
-  m_platformTools(platformTools), m_byteArray(field.m_value.toByteArray()),
-  m_isChanged(false)
+  m_platformTools(platformTools), m_app(app),
+  m_byteArray(field.m_value.toByteArray()), m_isChanged(false)
 {
   setObjectName(QLatin1String("BinaryOpenSave"));
   QHBoxLayout* layout = new QHBoxLayout(this);
@@ -701,7 +703,7 @@ void BinaryOpenSave::clipData()
 void BinaryOpenSave::loadData()
 {
   QString loadfilename = m_platformTools->getOpenFileName(this, QString(),
-        m_defaultDir.isEmpty() ? Kid3Application::getDirName() : m_defaultDir,
+        m_defaultDir.isEmpty() ? m_app->getDirName() : m_defaultDir,
         m_filter, 0);
   if (!loadfilename.isEmpty()) {
     QFile file(loadfilename);
@@ -723,7 +725,28 @@ void BinaryOpenSave::loadData()
  */
 void BinaryOpenSave::saveData()
 {
-  QString dir = m_defaultDir.isEmpty() ? Kid3Application::getDirName() : m_defaultDir;
+  QString dir = m_defaultDir.isEmpty() ? m_app->getDirName() : m_defaultDir;
+#if QT_VERSION >= 0x050000
+  QString fileName(m_defaultFile);
+  if (fileName.isEmpty()) {
+    fileName = QLatin1String("untitled");
+  }
+  QChar separator = QDir::separator();
+  if (!dir.endsWith(separator)) {
+    dir += separator;
+  }
+  QFileInfo fileInfo(fileName);
+  dir += fileInfo.completeBaseName();
+  QMimeDatabase mimeDb;
+  QString suffix = mimeDb.mimeTypeForData(m_byteArray).preferredSuffix();
+  if (suffix == QLatin1String("jpeg")) {
+    suffix = QLatin1String("jpg");
+  }
+  if (!suffix.isEmpty()) {
+    dir += QLatin1Char('.');
+    dir += suffix;
+  }
+#else
   if (!m_defaultFile.isEmpty()) {
     QChar separator = QDir::separator();
     if (!dir.endsWith(separator)) {
@@ -731,6 +754,7 @@ void BinaryOpenSave::saveData()
     }
     dir += m_defaultFile;
   }
+#endif
   QString fn = m_platformTools->getSaveFileName(
         this, QString(), dir, m_filter, 0);
   if (!fn.isEmpty()) {
@@ -755,50 +779,6 @@ void BinaryOpenSave::viewData()
   }
 }
 
-/**
- * Get description for ID3_Field.
- *
- * @param id ID of field
- * @return description or NULL if id unknown.
- */
-const char* Mp3FieldControl::getFieldIDString(Frame::Field::Id id) const
-{
-  static const char* const idStr[] = {
-    "Unknown",
-    QT_TRANSLATE_NOOP("@default", "Text Encoding"),
-    QT_TRANSLATE_NOOP("@default", "Text"),
-    QT_TRANSLATE_NOOP("@default", "URL"),
-    QT_TRANSLATE_NOOP("@default", "Data"),
-    QT_TRANSLATE_NOOP("@default", "Description"),
-    QT_TRANSLATE_NOOP("@default", "Owner"),
-    QT_TRANSLATE_NOOP("@default", "Email"),
-    QT_TRANSLATE_NOOP("@default", "Rating"),
-    QT_TRANSLATE_NOOP("@default", "Filename"),
-    QT_TRANSLATE_NOOP("@default", "Language"),
-    QT_TRANSLATE_NOOP("@default", "Picture Type"),
-    QT_TRANSLATE_NOOP("@default", "Image format"),
-    QT_TRANSLATE_NOOP("@default", "Mimetype"),
-    QT_TRANSLATE_NOOP("@default", "Counter"),
-    QT_TRANSLATE_NOOP("@default", "Identifier"),
-    QT_TRANSLATE_NOOP("@default", "Volume Adjustment"),
-    QT_TRANSLATE_NOOP("@default", "Number of Bits"),
-    QT_TRANSLATE_NOOP("@default", "Volume Change Right"),
-    QT_TRANSLATE_NOOP("@default", "Volume Change Left"),
-    QT_TRANSLATE_NOOP("@default", "Peak Volume Right"),
-    QT_TRANSLATE_NOOP("@default", "Peak Volume Left"),
-    QT_TRANSLATE_NOOP("@default", "Timestamp Format"),
-    QT_TRANSLATE_NOOP("@default", "Content Type"),
-
-    QT_TRANSLATE_NOOP("@default", "Price"),
-    QT_TRANSLATE_NOOP("@default", "Date"),
-    QT_TRANSLATE_NOOP("@default", "Seller")
-  };
-  struct not_used { int array_size_check[
-      sizeof(idStr) / sizeof(idStr[0]) == Frame::Field::ID_Seller + 1
-      ? 1 : -1 ]; };
-  return idStr[id <= Frame::Field::ID_Seller ? id : 0];
-}
-
 /**
  * Update field with data from dialog.
  */
@@ -819,8 +799,8 @@ QWidget* TextFieldControl::createWidget(QWidget* parent)
   if (m_edit == NULL)
     return NULL;
 
-  m_edit->setLabel(QCoreApplication::translate("@default",
-      getFieldIDString(static_cast<Frame::Field::Id>(m_field.m_id))));
+  m_edit->setLabel(Frame::Field::getFieldIdName(
+                     static_cast<Frame::FieldId>(m_field.m_id)));
   m_edit->setText(m_field.m_value.toString());
   m_edit->setFocus();
   return m_edit;
@@ -843,8 +823,8 @@ void LineFieldControl::updateTag()
 QWidget* LineFieldControl::createWidget(QWidget* parent)
 {
   m_edit = new LabeledLineEdit(parent);
-  m_edit->setLabel(QCoreApplication::translate("@default",
-      getFieldIDString(static_cast<Frame::Field::Id>(m_field.m_id))));
+  m_edit->setLabel(Frame::Field::getFieldIdName(
+                     static_cast<Frame::FieldId>(m_field.m_id)));
   m_edit->setText(m_field.m_value.toString());
   return m_edit;
 }
@@ -866,8 +846,8 @@ void IntFieldControl::updateTag()
 QWidget* IntFieldControl::createWidget(QWidget* parent)
 {
   m_numInp = new LabeledSpinBox(parent);
-  m_numInp->setLabel(QCoreApplication::translate("@default",
-      getFieldIDString(static_cast<Frame::Field::Id>(m_field.m_id))));
+  m_numInp->setLabel(Frame::Field::getFieldIdName(
+                       static_cast<Frame::FieldId>(m_field.m_id)));
   m_numInp->setValue(m_field.m_value.toInt());
   return m_numInp;
 }
@@ -889,8 +869,8 @@ void IntComboBoxControl::updateTag()
 QWidget* IntComboBoxControl::createWidget(QWidget* parent)
 {
   m_ptInp = new LabeledComboBox(parent, m_strLst);
-  m_ptInp->setLabel(QCoreApplication::translate("@default",
-      getFieldIDString(static_cast<Frame::Field::Id>(m_field.m_id))));
+  m_ptInp->setLabel(Frame::Field::getFieldIdName(
+                      static_cast<Frame::FieldId>(m_field.m_id)));
   m_ptInp->setCurrentItem(m_field.m_value.toInt());
   return m_ptInp;
 }
@@ -913,14 +893,14 @@ void BinFieldControl::updateTag()
  */
 QWidget* BinFieldControl::createWidget(QWidget* parent)
 {
-  m_bos = new BinaryOpenSave(m_platformTools, parent, m_field);
-  m_bos->setLabel(QCoreApplication::translate("@default",
-      getFieldIDString(static_cast<Frame::Field::Id>(m_field.m_id))));
+  m_bos = new BinaryOpenSave(m_platformTools, m_app, parent, m_field);
+  m_bos->setLabel(Frame::Field::getFieldIdName(
+                    static_cast<Frame::FieldId>(m_field.m_id)));
   if (m_taggedFile) {
     m_bos->setDefaultDir(m_taggedFile->getDirname());
   }
   if (m_frame.getType() == Frame::FT_Picture) {
-    m_bos->setDefaultFile(FileConfig::instance().m_defaultCoverFileName);
+    m_bos->setDefaultFile(FileConfig::instance().defaultCoverFileName());
     m_bos->setFilter(m_platformTools->fileDialogNameFilter(
                QList<QPair<QString, QString> >()
                << qMakePair(QCoreApplication::translate("@default",
@@ -996,6 +976,7 @@ QWidget* TimeEventFieldControl::createWidget(QWidget* parent)
  * Constructor.
  *
  * @param platformTools platform tools
+ * @param app application context
  * @param parent     parent widget
  */
 EditFrameFieldsDialog::EditFrameFieldsDialog(IPlatformTools* platformTools,
@@ -1064,47 +1045,30 @@ void EditFrameFieldsDialog::setFrame(const Frame& frame,
        fldIt != m_fields.end();
        ++fldIt) {
     Frame::Field& fld = *fldIt;
+    if (fld.m_id == Frame::ID_ImageProperties)
+      continue;
+
     switch (fld.m_value.type()) {
       case QVariant::Int:
       case QVariant::UInt:
-        if (fld.m_id == Frame::Field::ID_TextEnc) {
-          static const char* strlst[] = {
-            QT_TRANSLATE_NOOP("@default", "ISO-8859-1"),
-            QT_TRANSLATE_NOOP("@default", "UTF16"),
-            QT_TRANSLATE_NOOP("@default", "UTF16BE"),
-            QT_TRANSLATE_NOOP("@default", "UTF8"),
-            NULL
-          };
-          IntComboBoxControl* cbox = new IntComboBoxControl(fld, strlst);
+        if (fld.m_id == Frame::ID_TextEnc) {
+          IntComboBoxControl* cbox = new IntComboBoxControl(
+                fld, Frame::Field::getTextEncodingNames());
           m_fieldcontrols.append(cbox);
         }
-        else if (fld.m_id == Frame::Field::ID_PictureType) {
+        else if (fld.m_id == Frame::ID_PictureType) {
           IntComboBoxControl* cbox = new IntComboBoxControl(
                 fld, PictureFrame::getPictureTypeNames());
           m_fieldcontrols.append(cbox);
         }
-        else if (fld.m_id == Frame::Field::ID_TimestampFormat) {
-          static const char* strlst[] = {
-            QT_TRANSLATE_NOOP("@default", "Other"),
-            QT_TRANSLATE_NOOP("@default", "MPEG frames as unit"),
-            QT_TRANSLATE_NOOP("@default", "Milliseconds as unit"),
-            NULL
-          };
-          IntComboBoxControl* cbox = new IntComboBoxControl(fld, strlst);
+        else if (fld.m_id == Frame::ID_TimestampFormat) {
+          IntComboBoxControl* cbox = new IntComboBoxControl(
+                fld, Frame::Field::getTimestampFormatNames());
           m_fieldcontrols.append(cbox);
         }
-        else if (fld.m_id == Frame::Field::ID_ContentType) {
-          static const char* strlst[] = {
-            QT_TRANSLATE_NOOP("@default", "Other"),
-            QT_TRANSLATE_NOOP("@default", "Lyrics"),
-            QT_TRANSLATE_NOOP("@default", "Text transcription"),
-            QT_TRANSLATE_NOOP("@default", "Movement/part name"),
-            QT_TRANSLATE_NOOP("@default", "Events"),
-            QT_TRANSLATE_NOOP("@default", "Chord"),
-            QT_TRANSLATE_NOOP("@default", "Trivia/pop up"),
-            NULL
-          };
-          IntComboBoxControl* cbox = new IntComboBoxControl(fld, strlst);
+        else if (fld.m_id == Frame::ID_ContentType) {
+          IntComboBoxControl* cbox = new IntComboBoxControl(
+                fld, Frame::Field::getContentTypeNames());
           m_fieldcontrols.append(cbox);
         }
         else {
@@ -1114,7 +1078,7 @@ void EditFrameFieldsDialog::setFrame(const Frame& frame,
         break;
 
       case QVariant::String:
-        if (fld.m_id == Frame::Field::ID_Text) {
+        if (fld.m_id == Frame::ID_Text) {
           // Large textedit for text fields
           TextFieldControl* textctl = new TextFieldControl(fld);
           m_fieldcontrols.append(textctl);
@@ -1128,7 +1092,7 @@ void EditFrameFieldsDialog::setFrame(const Frame& frame,
       case QVariant::ByteArray:
       {
         BinFieldControl* binctl = new BinFieldControl(
-              m_platformTools, fld, frame, taggedFile);
+              m_platformTools, m_app, fld, frame, taggedFile);
         m_fieldcontrols.append(binctl);
         break;
       }
@@ -1158,7 +1122,7 @@ void EditFrameFieldsDialog::setFrame(const Frame& frame,
   }
 
   // Handle case for frames without fields, just a value.
-  m_valueField.m_id = Frame::Field::ID_Text;
+  m_valueField.m_id = Frame::ID_Text;
   if (m_fields.isEmpty()) {
     m_valueField.m_value = frame.getValue();
     m_fieldcontrols.append(new TextFieldControl(m_valueField));
