@@ -59,8 +59,8 @@
 #include "fileproxymodel.h"
 #include "fileproxymodeliterator.h"
 #include "modeliterator.h"
+#include "taggedfileselection.h"
 #include "filelist.h"
-#include "dirlist.h"
 #include "pictureframe.h"
 #include "fileconfig.h"
 #include "playlistconfig.h"
@@ -104,7 +104,8 @@ BaseMainWindowImpl::BaseMainWindowImpl(QMainWindow* mainWin,
 #if defined HAVE_PHONON || QT_VERSION >= 0x050000
   m_playToolBar(0),
 #endif
-  m_editFrameTaggedFile(0), m_findReplaceActive(false)
+  m_editFrameTaggedFile(0), m_findReplaceActive(false),
+  m_expandNotificationNeeded(false)
 {
   ContextHelp::init(m_platformTools);
 
@@ -128,17 +129,18 @@ BaseMainWindowImpl::BaseMainWindowImpl(QMainWindow* mainWin,
           this, SLOT(updateGuiControls()));
   connect(m_app, SIGNAL(frameModified(TaggedFile*)),
           this, SLOT(updateAfterFrameModification(TaggedFile*)));
-  connect(m_app, SIGNAL(fileModified()),
-          this, SLOT(updateModificationState()));
   connect(m_app, SIGNAL(confirmedOpenDirectoryRequested(QStringList)),
           this, SLOT(confirmedOpenDirectory(QStringList)));
   connect(m_app, SIGNAL(toggleExpandedRequested(QModelIndex)),
           this, SLOT(toggleExpanded(QModelIndex)));
   connect(m_app, SIGNAL(expandFileListRequested()),
           this, SLOT(expandFileList()));
-  connect(m_app,
-    SIGNAL(directoryOpened(QPersistentModelIndex,QList<QPersistentModelIndex>)),
-    this, SLOT(onDirectoryOpened()));
+  connect(m_app, SIGNAL(directoryOpened()),
+          this, SLOT(onDirectoryOpened()));
+  connect(m_app, SIGNAL(modifiedChanged(bool)),
+          this, SLOT(updateWindowCaption()));
+  connect(m_app, SIGNAL(filteredChanged(bool)),
+          this, SLOT(updateWindowCaption()));
 #if defined HAVE_PHONON || QT_VERSION >= 0x050000
   connect(m_app, SIGNAL(aboutToPlayAudio()), this, SLOT(showPlayToolBar()));
 #endif
@@ -253,12 +255,36 @@ void BaseMainWindowImpl::saveDirectory(bool updateGui)
 
   m_w->statusBar()->removeWidget(progress);
   delete progress;
-  updateModificationState();
   if (!errorFiles.empty()) {
-    m_platformTools->errorList(
-      m_w, tr("Error while writing file:\n"),
-      errorFiles,
-      tr("File Error"));
+    QStringList errorMsgs, notWritableFiles;
+    foreach (const QString& filePath, errorFiles) {
+      QFileInfo fileInfo(filePath);
+      if (!fileInfo.isWritable()) {
+        errorMsgs.append(tr("%1 is not writable").arg(fileInfo.fileName()));
+        notWritableFiles.append(filePath);
+      } else {
+        errorMsgs.append(fileInfo.fileName());
+      }
+    }
+    if (notWritableFiles.isEmpty()) {
+      m_platformTools->errorList(
+        m_w, tr("Error while writing file:\n"),
+        errorMsgs,
+        tr("File Error"));
+    } else {
+      int rc = m_platformTools->warningYesNoList(
+        m_w, tr("Error while writing file. "
+                "Do you want to change the permissions?"),
+        errorMsgs,
+        tr("File Error"));
+      if (rc == QMessageBox::Yes) {
+        foreach (const QString& filePath, notWritableFiles) {
+          QFile::setPermissions(filePath,
+              QFile::permissions(filePath) | QFile::WriteUser);
+        }
+        m_app->saveDirectory();
+      }
+    }
   }
 
   if (updateGui) {
@@ -296,10 +322,9 @@ bool BaseMainWindowImpl::saveModified(bool doNotRevert)
 
     case QMessageBox::No:
       if (!doNotRevert) {
-        if (m_form->getFileList()->selectionModel())
-          m_form->getFileList()->selectionModel()->clearSelection();
+        if (m_app->getFileSelectionModel())
+          m_app->getFileSelectionModel()->clearSelection();
         m_app->revertFileModifications();
-        m_app->setModified(false);
       }
       completed=true;
       break;
@@ -355,12 +380,12 @@ void BaseMainWindowImpl::slotFileOpen()
   updateCurrentSelection();
   if(saveModified()) {
     static QString flt = m_app->createFilterString();
-    QString filter(FileConfig::instance().m_nameFilter);
+    QString filter(FileConfig::instance().nameFilter());
     QStringList dirs = m_platformTools->getOpenFileNames(
       m_w, QString(), m_app->getDirName(), flt, &filter);
     if (!dirs.isEmpty()) {
       if (!filter.isEmpty()) {
-        FileConfig::instance().m_nameFilter = filter;
+        FileConfig::instance().setNameFilter(filter);
       }
       m_app->openDirectory(dirs);
     }
@@ -475,12 +500,13 @@ bool BaseMainWindowImpl::slotCreatePlaylist()
  */
 void BaseMainWindowImpl::setupImportDialog()
 {
-  m_app->filesToTrackDataModel(ImportConfig::instance().m_importDest);
+  m_app->filesToTrackDataModel(ImportConfig::instance().importDest());
   if (!m_importDialog) {
     QString caption(tr("Import"));
     m_importDialog =
-      new ImportDialog(m_platformTools, m_w,
-                       caption, m_app->getTrackDataModel(),
+      new ImportDialog(m_platformTools, m_w, caption,
+                       m_app->getTrackDataModel(),
+                       m_app->genreModelV2(),
                        m_app->getServerImporters(),
                        m_app->getServerTrackImporters());
     connect(m_importDialog, SIGNAL(accepted()),
@@ -519,15 +545,11 @@ void BaseMainWindowImpl::slotBatchImport()
     m_batchImportDialog = new BatchImportDialog(m_app->getServerImporters(),
                                                 m_w);
     connect(m_batchImportDialog,
-            SIGNAL(start(BatchImportProfile,TrackData::TagVersion)),
+            SIGNAL(start(BatchImportProfile,Frame::TagVersion)),
             m_app,
-            SLOT(batchImport(BatchImportProfile,TrackData::TagVersion)));
-    connect(m_app->getBatchImporter(),
-            SIGNAL(reportImportEvent(BatchImportProfile::ImportEventType,
-                                     QString)),
-            m_batchImportDialog,
-            SLOT(showImportEvent(BatchImportProfile::ImportEventType,
-                                 QString)));
+            SLOT(batchImport(BatchImportProfile,Frame::TagVersion)));
+    connect(m_app->getBatchImporter(), SIGNAL(reportImportEvent(int,QString)),
+            m_batchImportDialog, SLOT(showImportEvent(int,QString)));
     connect(m_batchImportDialog, SIGNAL(abort()),
             m_app->getBatchImporter(), SLOT(abort()));
     connect(m_app->getBatchImporter(), SIGNAL(finished()),
@@ -544,7 +566,7 @@ void BaseMainWindowImpl::slotBatchImport()
 void BaseMainWindowImpl::slotBrowseCoverArt()
 {
   if (!m_browseCoverArtDialog) {
-    m_browseCoverArtDialog = new BrowseCoverArtDialog(m_w);
+    m_browseCoverArtDialog = new BrowseCoverArtDialog(m_app, m_w);
   }
   FrameCollection frames2;
   QModelIndex index = m_form->getFileList()->currentIndex();
@@ -570,7 +592,7 @@ void BaseMainWindowImpl::slotExport()
         m_platformTools, m_w, m_app->getTextExporter());
   m_exportDialog->readConfig();
   ImportTrackDataVector trackDataVector;
-  m_app->filesToTrackData(ExportConfig::instance().m_exportSrcV1,
+  m_app->filesToTrackData(ExportConfig::instance().exportSource(),
                           trackDataVector);
   m_app->getTextExporter()->setTrackData(trackDataVector);
   m_exportDialog->showPreview();
@@ -584,7 +606,7 @@ void BaseMainWindowImpl::slotExport()
  */
 void BaseMainWindowImpl::slotSettingsAutoHideTags()
 {
-  GuiConfig::instance().m_autoHideTags = m_self->autoHideTagsAction()->isChecked();
+  GuiConfig::instance().setAutoHideTags(m_self->autoHideTagsAction()->isChecked());
   updateCurrentSelection();
   updateGuiControls();
 }
@@ -594,14 +616,14 @@ void BaseMainWindowImpl::slotSettingsAutoHideTags()
  */
 void BaseMainWindowImpl::slotSettingsShowHidePicture()
 {
-  GuiConfig::instance().m_hidePicture = !m_self->showHidePictureAction()->isChecked();
+  GuiConfig::instance().setHidePicture(!m_self->showHidePictureAction()->isChecked());
 
-  m_form->hidePicture(GuiConfig::instance().m_hidePicture);
+  m_form->hidePicture(GuiConfig::instance().hidePicture());
   // In Qt3 the picture is displayed too small if Kid3 is started with picture
   // hidden, and then "Show Picture" is triggered while a file with a picture
   // is selected. Thus updating the controls is only done for Qt4, in Qt3 the
   // file has to be selected again for the picture to be shown.
-  if (!GuiConfig::instance().m_hidePicture) {
+  if (!GuiConfig::instance().hidePicture()) {
     updateGuiControls();
   }
 }
@@ -611,22 +633,10 @@ void BaseMainWindowImpl::slotSettingsShowHidePicture()
  */
 void BaseMainWindowImpl::applyChangedConfiguration()
 {
-  m_app->saveConfig();
-  if (!TagConfig::instance().markTruncations()) {
-    m_app->frameModelV1()->markRows(0);
-  }
-  if (!FileConfig::instance().m_markChanges) {
-    m_app->frameModelV1()->markChangedFrames(0);
-    m_app->frameModelV2()->markChangedFrames(0);
+  m_app->applyChangedConfiguration();
+  if (!FileConfig::instance().markChanges()) {
     m_form->markChangedFilename(false);
   }
-  m_app->notifyConfigurationChange();
-  quint64 oldQuickAccessFrames = FrameCollection::getQuickAccessFrames();
-  if (TagConfig::instance().quickAccessFrames() != oldQuickAccessFrames) {
-    FrameCollection::setQuickAccessFrames(
-          TagConfig::instance().quickAccessFrames());
-    updateGuiControls();
-  }
 }
 
 /**
@@ -767,7 +777,13 @@ void BaseMainWindowImpl::slotNumberTracks()
     if (!totalEnabled)
       total = 0;
     TagConfig::instance().setEnableTotalNumberOfTracks(totalEnabled);
-    m_app->numberTracks(nr, total, m_numberTracksDialog->getDestination());
+    Kid3Application::NumberTrackOptions options;
+    if (m_numberTracksDialog->isTrackNumberingEnabled())
+      options |= Kid3Application::NumberTracksEnabled;
+    if (m_numberTracksDialog->isDirectoryCounterResetEnabled())
+      options |= Kid3Application::NumberTracksResetCounterForEachDirectory;
+    m_app->numberTracks(nr, total, m_numberTracksDialog->getDestination(),
+                        options);
   }
 }
 
@@ -781,12 +797,11 @@ void BaseMainWindowImpl::slotFilter()
       m_filterDialog = new FilterDialog(m_w);
       connect(m_filterDialog, SIGNAL(apply(FileFilter&)),
               m_app, SLOT(applyFilter(FileFilter&)));
-      connect(m_app, SIGNAL(fileFiltered(FileFilter::FilterEventType,QString)),
-              m_filterDialog,
-              SLOT(showFilterEvent(FileFilter::FilterEventType,QString)));
+      connect(m_app, SIGNAL(fileFiltered(int,QString)),
+              m_filterDialog, SLOT(showFilterEvent(int,QString)));
     }
     FilterConfig::instance().setFilenameFormat(
-          m_app->getTagsToFilenameFormat());
+          FileConfig::instance().toFilenameFormat());
     m_filterDialog->readConfig();
     m_filterDialog->show();
   }
@@ -812,30 +827,16 @@ void BaseMainWindowImpl::showPlayToolBar()
     m_w->addToolBar(Qt::BottomToolBarArea, m_playToolBar);
     connect(m_playToolBar, SIGNAL(errorMessage(QString)),
             this, SLOT(slotStatusMsg(QString)));
+#ifdef Q_OS_WIN32
+    // Phonon on Windows cannot play if the file is open.
+    connect(m_playToolBar, SIGNAL(aboutToPlay(QString)),
+            m_app, SLOT(closeFileHandle(QString)));
+#endif
   }
   m_playToolBar->show();
 }
 #endif
 
-/**
- * Update modification state, caption and listbox entries.
- */
-void BaseMainWindowImpl::updateModificationState()
-{
-  bool modified = false;
-  TaggedFileIterator it(m_form->getFileList()->rootIndex());
-  while (it.hasNext()) {
-    TaggedFile* taggedFile = it.next();
-    if (taggedFile->isChanged()) {
-      modified = true;
-      m_form->getFileList()->dataChanged(taggedFile->getIndex(),
-                                         taggedFile->getIndex());
-    }
-  }
-  m_app->setModified(modified);
-  updateWindowCaption();
-}
-
 /**
  * Set window title with information from directory, filter and modification
  * state.
@@ -857,20 +858,14 @@ void BaseMainWindowImpl::updateWindowCaption()
  */
 void BaseMainWindowImpl::updateCurrentSelection()
 {
-  const QList<QPersistentModelIndex>& selItems =
-    m_form->getFileList()->getCurrentSelection();
-  if (!selItems.isEmpty()) {
+  TaggedFileSelection* selection = m_app->selectionInfo();
+  if (!selection->isEmpty()) {
     m_form->frameTableV1()->acceptEdit();
     m_form->frameTableV2()->acceptEdit();
-    m_app->frameModelsToTags(selItems);
-    if (m_form->isFilenameEditEnabled()) {
-      if (TaggedFile* taggedFile =
-          FileProxyModel::getTaggedFileOfIndex(selItems.first())) {
-        taggedFile->setFilename(m_form->getFilename());
-      }
-    }
+    m_app->frameModelsToTags();
+
+    selection->setFilename(m_form->getFilename());
   }
-  updateModificationState();
 }
 
 /**
@@ -880,58 +875,27 @@ void BaseMainWindowImpl::updateCurrentSelection()
  */
 void BaseMainWindowImpl::updateGuiControls()
 {
-  m_form->getFileList()->updateCurrentSelection();
-  const QList<QPersistentModelIndex>& selItems =
-      m_form->getFileList()->getCurrentSelection();
+  m_app->tagsToFrameModels();
 
-  m_app->tagsToFrameModels(selItems);
-
-  TaggedFile::DetailInfo info;
-  if (const TaggedFile* selectedFile = m_app->selectionSingleFile()) {
-    m_form->setFilenameEditEnabled(true);
-    m_form->setFilename(selectedFile->getFilename());
-    selectedFile->getDetailInfo(info);
-    m_form->setDetailInfo(info);
-    m_form->setTagFormatV1(selectedFile->getTagFormatV1());
-    m_form->setTagFormatV2(selectedFile->getTagFormatV2());
-
-    if (FileConfig::instance().m_markChanges) {
-      m_form->markChangedFilename(selectedFile->isFilenameChanged());
-    }
-  } else {
-    if (m_app->selectionFileCount() > 1) {
-      m_form->setFilenameEditEnabled(false);
-      m_form->setFilename(Frame::differentRepresentation());
-    }
-    m_form->setDetailInfo(info);
-    m_form->setTagFormatV1(QString());
-    m_form->setTagFormatV2(QString());
-
-    if (FileConfig::instance().m_markChanges) {
-      m_form->markChangedFilename(false);
-    }
+  TaggedFileSelection* selection = m_app->selectionInfo();
+  m_form->setFilename(selection->getFilename());
+  m_form->setFilenameEditEnabled(selection->isSingleFileSelected());
+  m_form->setDetailInfo(selection->getDetailInfo());
+  m_form->setTagFormatV1(selection->getTagFormatV1());
+  m_form->setTagFormatV2(selection->getTagFormatV2());
+  if (FileConfig::instance().markChanges()) {
+    m_form->markChangedFilename(selection->isFilenameChanged());
   }
 
-  if (!GuiConfig::instance().m_hidePicture) {
-    FrameCollection::const_iterator it =
-      m_app->frameModelV2()->frames().find(
-          Frame(Frame::FT_Picture, QLatin1String(""), QLatin1String(""), -1));
-    if (it == m_app->frameModelV2()->frames().end() ||
-        it->isInactive()) {
-      m_form->setPictureData(0);
-    } else {
-      QByteArray data;
-      m_form->setPictureData(PictureFrame::getData(*it, data) ? &data : 0);
-    }
+  if (!GuiConfig::instance().hidePicture()) {
+    m_form->setPictureData(selection->getPicture());
   }
-  updateModificationState();
 
-  m_form->enableControlsV1(m_app->selectionTagV1SupportedCount() > 0 ||
-                           m_app->selectionFileCount() == 0);
+  m_form->enableControlsV1(selection->isTag1Used() || selection->isEmpty());
 
-  if (GuiConfig::instance().m_autoHideTags) {
-    m_form->hideV1(!m_app->selectionHasTagV1());
-    m_form->hideV2(!m_app->selectionHasTagV2());
+  if (GuiConfig::instance().autoHideTags()) {
+    m_form->hideV1(!selection->hasTagV1());
+    m_form->hideV2(!selection->hasTagV2());
   }
 }
 
@@ -946,45 +910,18 @@ void BaseMainWindowImpl::updateAfterFrameModification(TaggedFile* taggedFile)
     FrameCollection frames;
     taggedFile->getAllFramesV2(frames);
     m_app->frameModelV2()->transferFrames(frames);
-    updateModificationState();
   }
 }
 
-/**
- * Get type of frame from translated name.
- *
- * @param name name, spaces and case are ignored
- *
- * @return type.
- */
-static Frame::Type getTypeFromTranslatedName(QString name)
-{
-  static QMap<QString, int> strNumMap;
-  if (strNumMap.empty()) {
-    // first time initialization
-    for (int i = 0; i <= Frame::FT_LastFrame; ++i) {
-      Frame::Type type = static_cast<Frame::Type>(i);
-      strNumMap.insert(Frame::ExtendedType(type, QLatin1String("")).getTranslatedName().
-                       remove(QLatin1Char(' ')).toUpper(), type);
-    }
-  }
-  QMap<QString, int>::const_iterator it =
-    strNumMap.find(name.remove(QLatin1Char(' ')).toUpper());
-  if (it != strNumMap.end()) {
-    return static_cast<Frame::Type>(*it);
-  }
-  return Frame::FT_Other;
-}
-
 /**
  * Let user select a frame type.
+ * frameSelected() is emitted when the edit dialog is closed with the selected
+ * frame as a parameter if a frame is selected.
  *
- * @param frame is filled with the selected frame if true is returned
+ * @param frame is filled with the selected frame
  * @param taggedFile tagged file for which frame has to be selected
- *
- * @return false if no frame selected.
  */
-bool BaseMainWindowImpl::selectFrame(Frame* frame, const TaggedFile* taggedFile)
+void BaseMainWindowImpl::selectFrame(Frame* frame, const TaggedFile* taggedFile)
 {
   bool ok = false;
   if (taggedFile && frame) {
@@ -992,20 +929,19 @@ bool BaseMainWindowImpl::selectFrame(Frame* frame, const TaggedFile* taggedFile)
       m_w, tr("Add Frame"),
       tr("Select the frame ID"), taggedFile->getFrameIds(), 0, true, &ok);
     if (ok) {
-      Frame::Type type = getTypeFromTranslatedName(name);
+      Frame::Type type = Frame::getTypeFromTranslatedName(name);
       *frame = Frame(type, QLatin1String(""), name, -1);
     }
   }
-  return ok;
+  emit frameSelected(ok ? frame : 0);
 }
 
 /**
- * Return object which emits frameEdited() signal.
+ * Return object which emits frameSelected(), frameEdited() signals.
  *
- * @return object which emits frameEdited() when dialog is closed, its
- * parameter is the edited frame if the dialog is accepted.
+ * @return object which emits signals.
  */
-QObject* BaseMainWindowImpl::frameEditedEmitter()
+QObject* BaseMainWindowImpl::qobject()
 {
   return this;
 }
@@ -1081,7 +1017,7 @@ void BaseMainWindowImpl::onEditFrameDialogFinished(int result)
  */
 void BaseMainWindowImpl::renameFile()
 {
-  QItemSelectionModel* selectModel = m_form->getFileList()->selectionModel();
+  QItemSelectionModel* selectModel = m_app->getFileSelectionModel();
   FileProxyModel* model =
       qobject_cast<FileProxyModel*>(m_form->getFileList()->model());
   if (!selectModel || !model)
@@ -1144,7 +1080,7 @@ void BaseMainWindowImpl::renameFile()
  */
 void BaseMainWindowImpl::deleteFile()
 {
-  QItemSelectionModel* selectModel = m_form->getFileList()->selectionModel();
+  QItemSelectionModel* selectModel = m_app->getFileSelectionModel();
   FileProxyModel* model =
       qobject_cast<FileProxyModel*>(m_form->getFileList()->model());
   if (!selectModel || !model)
@@ -1215,11 +1151,20 @@ void BaseMainWindowImpl::toggleExpanded(const QModelIndex& index)
  */
 void BaseMainWindowImpl::expandFileList()
 {
+  m_expandNotificationNeeded = sender() == m_app;
   m_expandFileListStartTime = QDateTime::currentDateTime();
   connect(m_app->getFileProxyModelIterator(),
           SIGNAL(nextReady(QPersistentModelIndex)),
           this, SLOT(expandNextDirectory(QPersistentModelIndex)));
-  m_app->getFileProxyModelIterator()->start(m_form->getFileList()->rootIndex());
+  // If this slot is invoked from the file list menu action and the
+  // shift key is pressed, only expand the current subtree.
+  QObject* emitter = sender();
+  bool sentFromAction =
+      emitter && emitter->metaObject() == &QAction::staticMetaObject;
+  m_app->getFileProxyModelIterator()->start(
+        sentFromAction && QApplication::keyboardModifiers() == Qt::ShiftModifier
+        ? m_form->getFileList()->currentIndex()
+        : m_form->getFileList()->rootIndex());
 }
 
 /**
@@ -1261,6 +1206,10 @@ void BaseMainWindowImpl::expandNextDirectory(const QPersistentModelIndex& index)
     if (m_progressDialog) {
       m_progressDialog->reset();
     }
+    if (m_expandNotificationNeeded) {
+      m_expandNotificationNeeded = false;
+      m_app->notifyExpandFileListFinished();
+    }
   }
 }
 
@@ -1297,45 +1246,6 @@ void BaseMainWindow::init()
   m_impl->init();
 }
 
-/**
- * Let user select a frame type.
- *
- * @param frame is filled with the selected frame if true is returned
- * @param taggedFile tagged file for which frame has to be selected
- *
- * @return false if no frame selected.
- */
-bool BaseMainWindow::selectFrame(Frame* frame, const TaggedFile* taggedFile)
-{
-  return m_impl->selectFrame(frame, taggedFile);
-}
-
-/**
- * Return object which emits frameEdited() signal.
- *
- * @return object which emits frameEdited() when dialog is closed, its
- * parameter is the edited frame if the dialog is accepted.
- */
-QObject* BaseMainWindow::frameEditedEmitter()
-{
-  return m_impl;
-}
-
-/**
- * Create dialog to edit a frame and update the fields
- * if Ok is returned.
- * frameEdited() is emitted when the edit dialog is closed with the edited
- * frame as a parameter if it was accepted.
- *
- * @param frame frame to edit
- * @param taggedFile tagged file where frame has to be set
- */
-void BaseMainWindow::editFrameOfTaggedFile(const Frame* frame,
-                                           TaggedFile* taggedFile)
-{
-  m_impl->editFrameOfTaggedFile(frame, taggedFile);
-}
-
 #if defined HAVE_PHONON || QT_VERSION >= 0x050000
 /**
  * Play audio file.
