@@ -77,6 +77,17 @@ public:
   virtual int warningYesNoCancel(QWidget* parent, const QString& text,
       const QString& caption) = 0;
 
+  /**
+   * Display warning dialog with item list.
+   * @param parent parent widget
+   * @param text text
+   * @param strlist list of items
+   * @param caption caption
+   * @return QMessageBox::Yes or QMessageBox::No.
+   */
+  virtual int warningYesNoList(QWidget* parent, const QString& text,
+      const QStringList& strlist, const QString& caption) = 0;
+
   /**
    * Display dialog to select an existing file.
    * @param parent parent widget
