@@ -82,13 +82,10 @@ ID3_TextEnc getDefaultTextEncoding() { return s_defaultTextEncoding; }
 /**
  * Constructor.
  *
- * @param dn directory name
- * @param fn filename
- * @param idx model index
+ * @param idx index in file proxy model
  */
-Mp3File::Mp3File(const QString& dn, const QString& fn,
-                 const QPersistentModelIndex& idx) :
-  TaggedFile(dn, fn, idx), m_tagV1(0), m_tagV2(0)
+Mp3File::Mp3File(const QPersistentModelIndex& idx) :
+  TaggedFile(idx), m_tagV1(0), m_tagV2(0)
 {
 }
 
@@ -130,7 +127,8 @@ int Mp3File::taggedFileFeatures() const
  */
 void Mp3File::readTags(bool force)
 {
-  QByteArray fn = QFile::encodeName(getDirname() + QDir::separator() + currentFilename());
+  bool priorIsTagInformationRead = isTagInformationRead();
+  QByteArray fn = QFile::encodeName(currentFilePath());
 
   if (force && m_tagV1) {
     m_tagV1->Clear();
@@ -157,6 +155,8 @@ void Mp3File::readTags(bool force)
   if (force) {
     setFilename(currentFilename());
   }
+
+  notifyModelDataChanged(priorIsTagInformationRead);
 }
 
 /**
@@ -172,7 +172,7 @@ void Mp3File::readTags(bool force)
  */
 bool Mp3File::writeTags(bool force, bool* renamed, bool preserve)
 {
-  QString fnStr(getDirname() + QDir::separator() + currentFilename());
+  QString fnStr(currentFilePath());
   if (isChanged() && !QFileInfo(fnStr).isWritable()) {
     return false;
   }
@@ -221,11 +221,11 @@ bool Mp3File::writeTags(bool force, bool* renamed, bool preserve)
     ::utime(fn, &times);
   }
 
-  if (getFilename() != currentFilename()) {
+  if (isFilenameChanged()) {
     if (!renameFile(currentFilename(), getFilename())) {
       return false;
     }
-    updateCurrentFilename();
+    markFilenameUnchanged();
     // link tags to new file name
     readTags(true);
     *renamed = true;
@@ -1870,7 +1870,7 @@ bool Mp3File::setFrameV2(const Frame& frame)
           bool hasEnc;
           const ID3_TextEnc enc = fld->GetEncoding();
           ID3_TextEnc newEnc = static_cast<ID3_TextEnc>(
-                frame.getFieldValue(Frame::Field::ID_TextEnc).toInt(&hasEnc));
+                frame.getFieldValue(Frame::ID_TextEnc).toInt(&hasEnc));
           if (!hasEnc) {
             newEnc = enc;
           }
@@ -2132,7 +2132,7 @@ static Frame createFrameFromId3libFrame(ID3_Frame* id3Frame, int index)
   if (id3Frame->GetID() == ID3FID_USERTEXT ||
       id3Frame->GetID() == ID3FID_WWWUSER ||
       id3Frame->GetID() == ID3FID_COMMENT) {
-    QVariant fieldValue = frame.getFieldValue(Frame::Field::ID_Description);
+    QVariant fieldValue = frame.getFieldValue(Frame::ID_Description);
     if (fieldValue.isValid()) {
       QString description = fieldValue.toString();
       if (!description.isEmpty()) {
@@ -2153,13 +2153,13 @@ static Frame createFrameFromId3libFrame(ID3_Frame* id3Frame, int index)
     for (Frame::FieldList::const_iterator it = fields.begin();
          it != fields.end();
          ++it) {
-      if ((*it).m_id == Frame::Field::ID_Owner) {
+      if ((*it).m_id == Frame::ID_Owner) {
         owner = (*it).m_value.toString();
         if (!owner.isEmpty()) {
           frame.setExtendedType(Frame::ExtendedType(Frame::FT_Other,
                     frame.getInternalName() + QLatin1Char('\n') + owner));
         }
-      } else if ((*it).m_id == Frame::Field::ID_Data) {
+      } else if ((*it).m_id == Frame::ID_Data) {
         data = (*it).m_value.toByteArray();
       }
     }
@@ -2170,7 +2170,7 @@ static Frame createFrameFromId3libFrame(ID3_Frame* id3Frame, int index)
       }
     }
   } else if (id3Frame->GetID() == ID3FID_CDID) {
-    QVariant fieldValue = frame.getFieldValue(Frame::Field::ID_Data);
+    QVariant fieldValue = frame.getFieldValue(Frame::ID_Data);
     if (fieldValue.isValid()) {
       QString str;
       if (AttributeData(AttributeData::Utf16).toString(fieldValue.toByteArray(), str) &&
@@ -2179,7 +2179,7 @@ static Frame createFrameFromId3libFrame(ID3_Frame* id3Frame, int index)
       }
     }
   } else if (id3Frame->GetID() == ID3FID_UNIQUEFILEID) {
-    QVariant fieldValue = frame.getFieldValue(Frame::Field::ID_Data);
+    QVariant fieldValue = frame.getFieldValue(Frame::ID_Data);
     if (fieldValue.isValid()) {
       QByteArray ba(fieldValue.toByteArray());
       QString str(QString::fromLatin1(ba));
@@ -2189,7 +2189,7 @@ static Frame createFrameFromId3libFrame(ID3_Frame* id3Frame, int index)
       }
     }
   } else if (id3Frame->GetID() == ID3FID_POPULARIMETER) {
-    QVariant fieldValue = frame.getFieldValue(Frame::Field::ID_Rating);
+    QVariant fieldValue = frame.getFieldValue(Frame::ID_Rating);
     if (fieldValue.isValid()) {
       QString str(fieldValue.toString());
       if (!str.isEmpty()) {
@@ -2265,12 +2265,15 @@ void Mp3File::deleteFramesV2(const FrameFilter& flt)
 void Mp3File::getAllFramesV2(FrameCollection& frames)
 {
   frames.clear();
+  resetMarkedState();
   if (m_tagV2) {
     ID3_Tag::Iterator* iter = m_tagV2->CreateIterator();
     ID3_Frame* id3Frame;
     int i = 0;
     while ((id3Frame = iter->GetNext()) != 0) {
-      frames.insert(createFrameFromId3libFrame(id3Frame, i++));
+      Frame frame(createFrameFromId3libFrame(id3Frame, i++));
+      updateMarkedState(frame);
+      frames.insert(frame);
     }
 #ifdef Q_OS_WIN32
     /* allocated in Windows DLL => must be freed in the same DLL */
