@@ -70,13 +70,10 @@
 /**
  * Constructor.
  *
- * @param dn directory name
- * @param fn filename
- * @param idx model index
+ * @param idx index in file proxy model
  */
-M4aFile::M4aFile(const QString& dn, const QString& fn,
-                 const QPersistentModelIndex& idx) :
-  TaggedFile(dn, fn, idx), m_fileRead(false)
+M4aFile::M4aFile(const QPersistentModelIndex& idx) :
+  TaggedFile(idx), m_fileRead(false)
 {
 }
 
@@ -448,12 +445,17 @@ QString M4aFile::taggedFileKey() const
  */
 void M4aFile::readTags(bool force)
 {
+  bool priorIsTagInformationRead = isTagInformationRead();
   if (force || !m_fileRead) {
     m_metadata.clear();
     markTag2Unchanged();
     m_fileRead = true;
-    QByteArray fnIn = QFile::encodeName(
-      getDirname() + QDir::separator() + currentFilename());
+    QByteArray fnIn =
+#ifdef Q_OS_WIN32
+        currentFilePath().toUtf8();
+#else
+        QFile::encodeName(currentFilePath());
+#endif
 
     MP4FileHandle handle = MP4Read(fnIn);
     if (handle != MP4_INVALID_FILE_HANDLE) {
@@ -575,6 +577,8 @@ void M4aFile::readTags(bool force)
   if (force) {
     setFilename(currentFilename());
   }
+
+  notifyModelDataChanged(priorIsTagInformationRead);
 }
 
 /**
@@ -591,7 +595,7 @@ void M4aFile::readTags(bool force)
 bool M4aFile::writeTags(bool force, bool* renamed, bool preserve)
 {
   bool ok = true;
-  QString fnStr(getDirname() + QDir::separator() + currentFilename());
+  QString fnStr(currentFilePath());
   if (isChanged() && !QFileInfo(fnStr).isWritable()) {
     return false;
   }
@@ -891,11 +895,11 @@ bool M4aFile::writeTags(bool force, bool* renamed, bool preserve)
     }
   }
 
-  if (getFilename() != currentFilename()) {
+  if (isFilenameChanged()) {
     if (!renameFile(currentFilename(), getFilename())) {
       return false;
     }
-    updateCurrentFilename();
+    markFilenameUnchanged();
     // link tags to new file name
     readTags(true);
     *renamed = true;
