@@ -45,13 +45,10 @@
 /**
  * Constructor.
  *
- * @param dn directory name
- * @param fn filename
- * @param idx model index
+ * @param idx index in file proxy model
  */
-FlacFile::FlacFile(const QString& dn, const QString& fn,
-                   const QPersistentModelIndex& idx) :
-  OggFile(dn, fn, idx), m_chain(0)
+FlacFile::FlacFile(const QPersistentModelIndex& idx) :
+  OggFile(idx), m_chain(0)
 {
 }
 
@@ -76,14 +73,17 @@ static void getPicture(Frame& frame, const FLAC::Metadata::Picture* pic)
 {
   QByteArray ba(reinterpret_cast<const char*>(pic->get_data()),
     pic->get_data_length());
+  PictureFrame::ImageProperties imgProps(
+        pic->get_width(), pic->get_height(), pic->get_depth(),
+        pic->get_colors(), ba);
   PictureFrame::setFields(
     frame,
-    Frame::Field::TE_ISO8859_1, QLatin1String(""),
+    Frame::TE_ISO8859_1, QLatin1String(""),
     QString::fromLatin1(pic->get_mime_type()),
     static_cast<PictureFrame::PictureType>(pic->get_type()),
     QString::fromUtf8(
       reinterpret_cast<const char*>(pic->get_description())),
-    ba);
+    ba, &imgProps);
   frame.setExtendedType(Frame::ExtendedType(Frame::FT_Picture, QLatin1String("Picture")));
 }
 
@@ -95,23 +95,20 @@ static void getPicture(Frame& frame, const FLAC::Metadata::Picture* pic)
  */
 static void setPicture(const Frame& frame, FLAC::Metadata::Picture* pic)
 {
-  Frame::Field::TextEncoding enc;
+  Frame::TextEncoding enc;
   PictureFrame::PictureType pictureType = PictureFrame::PT_CoverFront;
   QString imgFormat, mimeType, description;
   QByteArray ba;
+  PictureFrame::ImageProperties imgProps;
   PictureFrame::getFields(frame, enc, imgFormat, mimeType,
-                          pictureType, description, ba);
-  QImage image;
-  if (image.loadFromData(ba)) {
-    pic->set_width(image.width());
-    pic->set_height(image.height());
-    pic->set_depth(image.depth());
-#if QT_VERSION >= 0x040600
-    pic->set_colors(image.colorCount());
-#else
-    pic->set_colors(image.numColors());
-#endif
+                          pictureType, description, ba, &imgProps);
+  if (!imgProps.isValidForImage(ba)) {
+    imgProps = PictureFrame::ImageProperties(ba);
   }
+  pic->set_width(imgProps.width());
+  pic->set_height(imgProps.height());
+  pic->set_depth(imgProps.depth());
+  pic->set_colors(imgProps.numColors());
   pic->set_mime_type(mimeType.toLatin1());
   pic->set_type(
     static_cast<FLAC__StreamMetadata_Picture_Type>(pictureType));
@@ -138,12 +135,13 @@ QString FlacFile::taggedFileKey() const
  */
 void FlacFile::readTags(bool force)
 {
+  bool priorIsTagInformationRead = isTagInformationRead();
   if (force || !m_fileRead) {
     m_comments.clear();
     markTag2Unchanged();
     m_fileRead = true;
-    QByteArray fnIn = QFile::encodeName(getDirname() + QDir::separator() + currentFilename());
-    m_fileInfo.read(0); // just to start invalid
+    QByteArray fnIn = QFile::encodeName(currentFilePath());
+    readFileInfo(m_fileInfo, 0); // just to start invalid
     if (!m_chain) {
       m_chain = new FLAC::Metadata::Chain;
     }
@@ -162,7 +160,7 @@ void FlacFile::readTags(bool force)
             if (proto) {
               FLAC::Metadata::StreamInfo* si =
                 dynamic_cast<FLAC::Metadata::StreamInfo*>(proto);
-              m_fileInfo.read(si);
+              readFileInfo(m_fileInfo, si);
               delete proto;
             }
           } else if (mdt == FLAC__METADATA_TYPE_VORBIS_COMMENT) {
@@ -220,6 +218,8 @@ void FlacFile::readTags(bool force)
   if (force) {
     setFilename(currentFilename());
   }
+
+  notifyModelDataChanged(priorIsTagInformationRead);
 }
 
 /**
@@ -236,7 +236,7 @@ void FlacFile::readTags(bool force)
 bool FlacFile::writeTags(bool force, bool* renamed, bool preserve)
 {
   if (isChanged() &&
-    !QFileInfo(getDirname() + QDir::separator() + currentFilename()).isWritable()) {
+    !QFileInfo(currentFilePath()).isWritable()) {
     return false;
   }
 
@@ -354,11 +354,11 @@ bool FlacFile::writeTags(bool force, bool* renamed, bool preserve)
       return false;
     }
   }
-  if (getFilename() != currentFilename()) {
+  if (isFilenameChanged()) {
     if (!renameFile(currentFilename(), getFilename())) {
       return false;
     }
-    updateCurrentFilename();
+    markFilenameUnchanged();
     // link tags to new file name
     readTags(true);
     *renamed = true;
@@ -419,7 +419,7 @@ bool FlacFile::addFrameV2(Frame& frame)
   if (frame.getType() == Frame::FT_Picture) {
     if (frame.getFieldList().empty()) {
       PictureFrame::setFields(
-        frame, Frame::Field::TE_ISO8859_1, QLatin1String("JPG"), QLatin1String("image/jpeg"),
+        frame, Frame::TE_ISO8859_1, QLatin1String("JPG"), QLatin1String("image/jpeg"),
         PictureFrame::PT_CoverFront, QLatin1String(""), QByteArray());
     }
     PictureFrame::setDescription(frame, frame.getValue());
@@ -478,6 +478,7 @@ void FlacFile::getAllFramesV2(FrameCollection& frames)
        it != m_pictures.end();
        ++it) {
     (*it).setIndex(i++);
+    updateMarkedState(*it);
     frames.insert(*it);
   }
 }
@@ -567,19 +568,21 @@ QString FlacFile::getFileExtension() const
 
 /**
  * Read information about a FLAC file.
+ * @param info file info to fill
  * @param fn file name
  * @return true if ok.
  */
-bool FlacFile::FileInfo::read(FLAC::Metadata::StreamInfo* si)
+bool FlacFile::readFileInfo(FileInfo& info,
+                            FLAC::Metadata::StreamInfo* si) const
 {
   if (si && si->is_valid()) {
-    valid = true;
-    channels = si->get_channels();
-    sampleRate = si->get_sample_rate();
-    duration = si->get_total_samples() / sampleRate;
-    bitrate = si->get_bits_per_sample() * sampleRate;
+    info.valid = true;
+    info.channels = si->get_channels();
+    info.sampleRate = si->get_sample_rate();
+    info.duration = si->get_total_samples() / info.sampleRate;
+    info.bitrate = si->get_bits_per_sample() * info.sampleRate;
   } else {
-    valid = false;
+    info.valid = false;
   }
-  return valid;
+  return info.valid;
 }
