@@ -145,13 +145,10 @@ long oggtell(void* stream)
 /**
  * Constructor.
  *
- * @param dn directory name
- * @param fn filename
- * @param idx model index
+ * @param idx index in file proxy model
  */
-OggFile::OggFile(const QString& dn, const QString& fn,
-                 const QPersistentModelIndex& idx) :
-  TaggedFile(dn, fn, idx), m_fileRead(false)
+OggFile::OggFile(const QPersistentModelIndex& idx) :
+  TaggedFile(idx), m_fileRead(false)
 {
 }
 
@@ -188,13 +185,14 @@ int OggFile::taggedFileFeatures() const
  */
 void OggFile::readTags(bool force)
 {
+  bool priorIsTagInformationRead = isTagInformationRead();
   if (force || !m_fileRead) {
     m_comments.clear();
     markTag2Unchanged();
     m_fileRead = true;
-    QString fnIn = getDirname() + QDir::separator() + currentFilename();
+    QString fnIn = currentFilePath();
 
-    if (m_fileInfo.read(fnIn)) {
+    if (readFileInfo(m_fileInfo, fnIn)) {
       QFile fpIn(fnIn);
       if (fpIn.open(QIODevice::ReadOnly)) {
         vcedit_state* state = ::vcedit_new_state();
@@ -229,6 +227,8 @@ void OggFile::readTags(bool force)
   if (force) {
     setFilename(currentFilename());
   }
+
+  notifyModelDataChanged(priorIsTagInformationRead);
 }
 
 /**
@@ -246,15 +246,16 @@ bool OggFile::writeTags(bool force, bool* renamed, bool preserve)
 {
   QString dirname = getDirname();
   if (isChanged() &&
-    !QFileInfo(dirname + QDir::separator() + currentFilename()).isWritable()) {
+    !QFileInfo(currentFilePath()).isWritable()) {
     return false;
   }
 
   if (m_fileRead && (force || isTag2Changed())) {
     bool writeOk = false;
     // we have to rename the original file and delete it afterwards
-    QString tempFilename(currentFilename() + QLatin1String("_KID3"));
-    if (!renameFile(currentFilename(), tempFilename)) {
+    QString filename = currentFilename();
+    QString tempFilename(filename + QLatin1String("_KID3"));
+    if (!renameFile(filename, tempFilename)) {
       return false;
     }
     QString fnIn = dirname + QDir::separator() + tempFilename;
@@ -324,16 +325,16 @@ bool OggFile::writeTags(bool force, bool* renamed, bool preserve)
     }
     markTag2Unchanged();
     QDir(dirname).remove(tempFilename);
-    if (getFilename() != currentFilename()) {
-      updateCurrentFilename();
+    if (isFilenameChanged()) {
+      markFilenameUnchanged();
       *renamed = true;
     }
-  } else if (getFilename() != currentFilename()) {
+  } else if (isFilenameChanged()) {
     // tags not changed, but file name
     if (!renameFile(currentFilename(), getFilename())) {
       return false;
     }
-    updateCurrentFilename();
+    markFilenameUnchanged();
     *renamed = true;
   }
   return true;
@@ -406,7 +407,7 @@ static const char* getVorbisNameFromType(Frame::Type type)
       sizeof(names) / sizeof(names[0]) == Frame::FT_LastFrame + 1
       ? 1 : -1 ]; };
   if (type == Frame::FT_Picture &&
-      TagConfig::instance().pictureNameItem() == TagConfig::VP_COVERART) {
+      TagConfig::instance().pictureNameIndex() == TagConfig::VP_COVERART) {
     return "COVERART";
   }
   return type <= Frame::FT_LastFrame ? names[type] : "UNKNOWN";
@@ -806,7 +807,9 @@ bool OggFile::setFrameV2(const Frame& frame)
   if (index != -1 && index < static_cast<int>(m_comments.size())) {
     QString value = frame.getValue();
     if (frame.getType() == Frame::FT_Picture) {
-      PictureFrame::getFieldsToBase64(frame, value);
+      Frame newFrame(frame);
+      PictureFrame::setDescription(newFrame, value);
+      PictureFrame::getFieldsToBase64(newFrame, value);
       if (!value.isEmpty() && frame.getInternalName() == QLatin1String("COVERART")) {
         QString mimeType;
         PictureFrame::getMimeType(frame, mimeType);
@@ -841,7 +844,7 @@ bool OggFile::addFrameV2(Frame& frame)
   if (frame.getType() == Frame::FT_Picture) {
     if (frame.getFieldList().empty()) {
       PictureFrame::setFields(
-        frame, Frame::Field::TE_ISO8859_1, QLatin1String(""), QLatin1String("image/jpeg"),
+        frame, Frame::TE_ISO8859_1, QLatin1String(""), QLatin1String("image/jpeg"),
         PictureFrame::PT_CoverFront, QLatin1String(""), QByteArray());
     }
     frame.setExtendedType(Frame::ExtendedType(Frame::FT_Picture, name));
@@ -883,6 +886,7 @@ bool OggFile::deleteFrameV2(const Frame& frame)
 void OggFile::getAllFramesV2(FrameCollection& frames)
 {
   frames.clear();
+  resetMarkedState();
   QString name;
   int i = 0;
   for (OggFile::CommentList::const_iterator it = m_comments.begin();
@@ -896,6 +900,7 @@ void OggFile::getAllFramesV2(FrameCollection& frames)
       if (name == QLatin1String("COVERART")) {
         PictureFrame::setMimeType(frame, getTextField(QLatin1String("COVERARTMIME")));
       }
+      updateMarkedState(frame);
       frames.insert(frame);
     } else {
       frames.insert(Frame(type, (*it).getValue(), name, i++));
@@ -953,40 +958,41 @@ QStringList OggFile::getFrameIds() const
 #ifdef HAVE_VORBIS
 /**
  * Read information about an Ogg/Vorbis file.
+ * @param info file info to fill
  * @param fn file name
  * @return true if ok.
  */
-bool OggFile::FileInfo::read(const QString& fn)
+bool OggFile::readFileInfo(FileInfo& info, const QString& fn) const
 {
   static ::ov_callbacks ovcb = {
     oggread, oggseek, oggclose, oggtell
   };
-  valid = false;
+  info.valid = false;
   QFile fp(fn);
   if (fp.open(QIODevice::ReadOnly)) {
     OggVorbis_File vf;
     if (::ov_open_callbacks(&fp, &vf, 0, 0, ovcb) == 0) {
       vorbis_info* vi = ::ov_info(&vf, -1);
       if (vi) {
-        valid = true;
-        version = vi->version;
-        channels = vi->channels;
-        sampleRate = vi->rate;
-        bitrate = vi->bitrate_nominal;
-        if (bitrate <= 0) {
-          bitrate = vi->bitrate_upper;
+        info.valid = true;
+        info.version = vi->version;
+        info.channels = vi->channels;
+        info.sampleRate = vi->rate;
+        info.bitrate = vi->bitrate_nominal;
+        if (info.bitrate <= 0) {
+          info.bitrate = vi->bitrate_upper;
         }
-        if (bitrate <= 0) {
-          bitrate = vi->bitrate_lower;
+        if (info.bitrate <= 0) {
+          info.bitrate = vi->bitrate_lower;
         }
       }
-      duration = static_cast<long>(::ov_time_total(&vf, -1));
+      info.duration = static_cast<long>(::ov_time_total(&vf, -1));
       ::ov_clear(&vf); // closes file, do not use ::fclose()
     } else {
       fp.close();
     }
   }
-  return valid;
+  return info.valid;
 }
 #endif // HAVE_VORBIS
 
