@@ -46,10 +46,10 @@
 #define TAGLIB_VERSION 0x010800
 #endif
 
-/** Convert QString @a s to a TagLib::String. */
-#define QSTRING_TO_TSTRING(s) TagLib::String(s.toUtf8().data(), TagLib::String::UTF8)
-
 class QTextCodec;
+#if TAGLIB_VERSION >= 0x010800
+class FileIOStream;
+#endif
 
 #if TAGLIB_VERSION >= 0x010600 && defined TAGLIB_WITH_MP4
   namespace TagLib {
@@ -65,12 +65,9 @@ public:
   /**
    * Constructor.
    *
-   * @param dn directory name
-   * @param fn filename
-   * @param idx model index
+   * @param idx index in file proxy model
    */
-  TagLibFile(const QString& dn, const QString& fn,
-             const QPersistentModelIndex& idx);
+  explicit TagLibFile(const QPersistentModelIndex& idx);
 
   /**
    * Destructor.
@@ -611,6 +608,7 @@ private:
    */
   static QString getTagFormat(const TagLib::Tag* tag, TagType& type);
 
+#if TAGLIB_VERSION < 0x010800
   /**
    * Register open TagLib file, so that the number of open files can be limited.
    * If the number of open files exceeds a limit, files are closed.
@@ -625,6 +623,7 @@ private:
    * @param tagLibFile file which is no longer open
    */
   static void deregisterOpenFile(TagLibFile* tagLibFile);
+#endif
 
   /**
    * Set the text codec to be used for tag 1.
@@ -640,10 +639,18 @@ private:
    */
   static void setDefaultTextEncoding(TagConfig::TextEncoding textEnc);
 
+  bool m_tagInformationRead;
+  bool m_hasTagV1;
+  bool m_hasTagV2;
+  bool m_isTagV1Supported;
+
+  bool m_fileRead;           /**< true if file has been read */
+
   TagLib::FileRef m_fileRef; /**< file reference */
   TagLib::Tag* m_tagV1;      /**< ID3v1 tags */
   TagLib::Tag* m_tagV2;      /**< ID3v2 tags */
 #if TAGLIB_VERSION >= 0x010800
+  FileIOStream* m_stream;
   int m_id3v2Version;        /**< 3 for ID3v2.3, 4 for ID3v2.4, 0 if none */
 #endif
   int m_activatedFeatures;   /**< TF_ID3v23, TF_ID3v24, or 0 */
@@ -671,18 +678,13 @@ private:
   Pictures m_pictures;
 #endif
 
-  bool m_tagInformationRead;
-  bool m_hasTagV1;
-  bool m_hasTagV2;
-  bool m_isTagV1Supported;
-
-  bool m_fileRead;           /**< true if file has been read */
-
   /** default text encoding */
   static TagLib::String::Type s_defaultTextEncoding;
 
+#if TAGLIB_VERSION < 0x010800
   /** list of TagLib files with open file descriptor */
   static QList<TagLibFile*> s_openFiles;
+#endif
 };
 
 #endif // TAGLIBFILE_H
